@echo off
setlocal enableextensions enabledelayedexpansion


	REM #################################################################################
:CHECKHELP

	(echo.%*|findstr /L /i /c:"/help" /c:"/h" /c:"/?" >nul: 2>&1) && (goto :HELP)

:CHECKHELP_END
goto :SETUP %*



	REM #################################################################################
:SETUP
	call "%~dp0AutoCams.setup.bat" "%~dpnx0"
	!bat_CALL! "%~dp0indent_setup.bat"

	!bat_echolog! =============================================================
	!bat_echolog! %date% %time% - Version: !bat_vers! - [%~dpnx0 %*]
	!bat_echolog! [!bat_batchfile!:SETUP %*]  Please wait...
	!bat_pushecho!
	
	!bat_log! start /low /min SCHTASKS /End /tn Monitor_Poll_Client
	          start /low /min SCHTASKS /End /tn Monitor_Poll_Client
	
	set bat_returncode=0

	!bat_CALL! "!bat_exedir!\ReadParams.bat"

	!bat_WAIT! 3	"!bat_batchfile!:SETUP"


	call :PARSEARGS %*
	if !errorlevel! neq 0 (
		call :DELETABLE
		goto :DONE
	)

	call :MAIN %*

	!bat_log! start /low /min SCHTASKS /Run /tn Monitor_Poll_Client
	          start /low /min SCHTASKS /Run /tn Monitor_Poll_Client

goto :DONE



	REM #################################################################################
:MAIN
!bat_pushlog! [CALL !bat_batchfile!:MAIN] %*

	call :SETUP_DIRS
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	call :MAKE_CALSTARS
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	call :COPY_CAL_FILES
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	call :ZIP_CALSTARS
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	!bat_logexeclog! set bat_cmdline="!bat_exedir!\Autocams.bat" "!bat_capturedfilesdir!" "!bat_archivedfilesdir!" "!bat_camera!" autonomous /remote
    
	call :PREP_QUEUE "!bat_ZipFileNameFull!" "!bat_capturedfilesdir!" "!bat_caldir!" "!bat_cmdline!"
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	call :UPLOAD_QUEUE
	set bat_returncode=!errorlevel!
	if !bat_returncode! neq 0 (
		goto :MAIN_END
	)

	call :CLEANUP
	if !errorlevel! neq 0 (goto :MAIN_END)

	


:MAIN_END
!bat_popecho! [RETURN !bat_batchfile!:MAIN] []
!bat_log!
!bat_return! !bat_returncode!






	REM #################################################################################
	REM Set up the directories for performing what needs to be done to create
	REM  - Ensure that the bat_capturedfilesdir exists
	REM 
	REM RETURNS:
	REM     Nothing
	REM 
:SETUP_DIRS
!bat_pushlog! [CALL !bat_batchfile!:SETUP_DIRS] %*

	set bat_returncode=0

	if /i "!bat_calstarsdir!" equ "" (
		call :ERROR 1 "bat_calstarsdir is blank [!bat_calstarsdir!]"
		goto :SETUP_DIRS_END
	)
	
	if /i "!bat_calstarsdir!" equ "!bat_exedir!" (
		call :ERROR 1 "bat_calstarsdir setting will cause a dangerous delete of all the files from the CAMS dir!!! [!bat_calstarsdir!]"
		goto :SETUP_DIRS_END
	)
	
		REM Delete these files in case moving them in a previous attempt was interrupted...
	!bat_logexeclog! DEL /Q "!bat_capturedfilesdir!\CALSTARS??????_*.zip"
	!bat_logexeclog! DEL /Q "!bat_capturedfilesdir!\CALSTARS??????_*.txt"
	
	!bat_logexeclog! RD /S /Q "!bat_capturedfilesdir!\CALSTARS\"
	

:SETUP_DIRS_END
!bat_popecho! [RETURN !bat_batchfile!:SETUP_DIRS] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3	:SETUP_DIRS_END
!bat_return! !bat_returncode!





	REM #################################################################################
	REM This routine generates the CALSTARS files from the bat_capturedfilesdir.
	REM It accomplishes this by using the FTP_CalStarsExtractor.exe. 
	REM 
	REM REMOTE_skip=36
	REM REMOTE_nstars=300
	REM 
	REM RETURNS:
	REM     bat_calstars_file           CALSTARS000750_2016_12_31_14_37_59.txt
	REM     bat_TempCalFromZipName      CALSTARS000750_2016_12_31_14_37_59
	REM     bat_TempCalFromZipFull      C:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALSTARS\CALSTARS000750_2016_12_31_14_37_59.zip
	REM     bat_camera                  000750
	REM     bat_calstars_caldir         C:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALSTARS\Cal
	REM     
:MAKE_CALSTARS
!bat_pushlog! [CALL !bat_batchfile!:MAKE_CALSTARS] %*

	set bat_returncode=0

	!bat_echolog!
	!bat_pushecho! [!date! !time! !bat_batchfile!:MAKE_CALSTARS_1] ... Ready to start FTP_CalStarExtractor.exe
	!bat_echolog!

		REM Launch the utility that makes CALSTARS*.txt files, one .txt file for each camera.  
		REM
	!bat_echolog! START /WAIT "FTP_CalStarExtractor - !bat_capturedfilesdir!" "!bat_exedir!\FTP_CalStarExtractor.exe" !bat_CALIBRATION.REMOTE_skip! !bat_CALIBRATION.REMOTE_nstars! "!bat_capturedfilesdir!"
	              START /WAIT "FTP_CalStarExtractor - !bat_capturedfilesdir!" "!bat_exedir!\FTP_CalStarExtractor.exe" !bat_CALIBRATION.REMOTE_skip! !bat_CALIBRATION.REMOTE_nstars! "!bat_capturedfilesdir!"
	!bat_logexeclog! set bat_returncode=!errorlevel!

:MAKE_CALSTARS_1
!bat_log! !bat_batchfile!:MAKE_CALSTARS_1

	!bat_echolog!
	!bat_popecho! [!date! !time! !bat_batchfile!:MAKE_CALSTARS_1] ... Back from FTP_CalStarExtractor.exe
	!bat_echolog!


	
	set bat_calstars_file=
	set bat_calstars_caldir=
	set bat_TempCalFromZipName=
	set bat_TempCalFromZipExt=.zip
	set bat_TempCalFromZipFull=
	set bat_camera=

		REM Iterate the list of CALSTARS  files backward to determine the bat_camera, and zip file name.
		REM 
		REM IMPORTANT - the bat_TempCalFromZipName doesn't have the sequence number yet.
		REM
	!bat_pushlog! for /f "usebackq tokens=*" ..a in ^(`dir "!bat_capturedfilesdir!\CALSTARS??????_????_??_??_??_??_??.txt" /s/b /a-d /o-n`^) do ^(
	for               /f "usebackq tokens=*" %%a in  (`dir "!bat_capturedfilesdir!\CALSTARS??????_????_??_??_??_??_??.txt" /s/b /a-d /o-n`) do (
		!bat_log! .........................
			REM bat_calstars_file=CALSTARS000750_2016_12_31_14_37_59.txt
		!bat_logexeclog! set bat_calstars_file=%%~nxa
		
			REM bat_TempCalFromZipName=CALSTARS000750_2016_12_31_14_37_59
		!bat_logexeclog! set bat_TempCalFromZipName=%%~na
		
			REM bat_TempCalFromZipFile=CALSTARS000750_2016_12_31_14_37_59.zip
		!bat_logexeclog! set bat_TempCalFromZipFile=!bat_TempCalFromZipName!.zip
		
			REM bat_TempCalFromZipFull=C:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALSTARS\CALSTARS000750_2016_12_31_14_37_59.zip
		!bat_logexeclog! set bat_TempCalFromZipFull=!bat_calstarsdir!\!bat_TempCalFromZipName!!bat_TempCalFromZipExt!
		
			REM bat_camera=000750
		!bat_logexeclog! set bat_camera=!bat_TempCalFromZipName:~9,3!
		
			REM bat_calstars_caldir=C:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALSTARS\Cal
		!bat_logexeclog! set bat_calstars_caldir=!bat_calstarsdir!\Cal
	)
	!bat_poplog! ^)
	
	if not defined bat_calstars_file (
		call :ERROR 31 "failed to produce a CALSTARS file in "!bat_capturedfilesdir!"
		set bat_returncode=!errorlevel! 
	)


:MAKE_CALSTARS_END
!bat_popecho! [RETURN !bat_batchfile!:MAKE_CALSTARS] [bat_returncode=!bat_returncode!, 
!bat_log! `                                          bat_calstars_caldir=!bat_calstars_caldir!,
!bat_log! `                                          bat_TempCalFromZipName=!bat_TempCalFromZipName!,
!bat_log! `                                          bat_TempCalFromZipFile=!bat_TempCalFromZipFile!,
!bat_log! `                                          bat_TempCalFromZipFull=!bat_TempCalFromZipFull!,
!bat_log! `                                          bat_camera=!bat_camera!,
!bat_log! `                                          bat_calstars_caldir=!bat_calstars_caldir!,
!bat_log!
rem !bat_WAIT! 3	:MAKE_CALSTARS_END
!bat_return! !bat_returncode!






	REM #################################################################################
	REM This routine needs to copy pre-existing cal files matching the cameras found in
	REM the bat_capturedfilesdir.  Remember, the GCFC.log file should already exist, 
	REM which provides a list of cameras based on the FF file names. 
	REM The ..\Cal dir as well as the ..\Cal\BinFiles dir should be queried to find aa
	REM a cal file that is before the bat_FF_last value for that camera. 
	REM
	REM IMPORTANT - the bat_TempCalFromZipName doesn't have the sequence number yet.
	REM     
	REM RETURNS:
	REM     bat_FFfirst                FF_000750_20161231_143759_543_0000000.bin
	REM     bat_FFlast                 FF_000750_20170101_032517_938_0058112.bin
	REM     bat_FF_time_max            20161230_150159_543
	REM     bat_cal_include            CAL_000750_20161230_150000_000.txt
	REM     bat_cal_temp_time          20161230_150000_000
	REM     bat_cal_include            C:\CAMS\Cal\CAL_000750_20161230_150000_000.txt
	REM     
	REM     
	REM     
:COPY_CAL_FILES
!bat_pushlog! [CALL !bat_batchfile!:COPY_CAL_FILES] %*

	set bat_returncode=0

	REM Validate and purge the bat_calstarsdir dir
	(
			REM if exist "c:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALSTARS"
		!bat_log! if exist "!bat_calstarsdir!" 
		if           exist "!bat_calstarsdir!" (
			!bat_logexeclog! RD /S /Q "!bat_calstarsdir!"
		)
		
		if not exist "!bat_calstarsdir!" (
				REM Since the previous line should have succeeded, this should always execute.  
				REM The only reason it wouldn't is if there was a file open in an editor or something.
			!bat_logexeclog! MD "!bat_calstarsdir!"
		) else (
			CALL :ERROR 0 "WARNING - Was not able to completely remove [!bat_calstarsdir!].  Might be because a file is open in an editor?"
		)

		if /i "!bat_calstarsdir!" equ "!bat_exedir!" (
			call :ERROR 42 "bat_calstarsdir setting will cause a dangerous delete of all the files from the CAMS dir!!! [!bat_calstarsdir!]"
			goto :SETUP_DIRS_END
		)


		if exist "!bat_calstarsdir!\*.*" (
			!bat_logexeclog! del /q "!bat_calstarsdir!\*.*"
		)

	)


		REM Assemble the Cal folder files, including camerasites.txt, camera files, and CAL files...
	(
		if not exist "!bat_calstars_caldir!" (
			!bat_logexeclog! MD "!bat_calstars_caldir!"
		)

		if /i "!bat_calstars_caldir!" equ "!bat_exedir!" (
			call :ERROR 43 "bat_calstars_caldir setting will cause a dangerous delete of all the files from the CAMS dir!!! [!bat_calstars_caldir!]"
			goto :SETUP_DIRS_END
		)


		if exist "!bat_calstars_caldir!\*.*" (
			!bat_logexeclog! DEL /q "!bat_calstars_caldir!\*.*"
		)
		
			REM Copy the client-side CameraSites.txt file so the remote cal server can use that instead of its own...
		!bat_logexeclog! COPY /y "!bat_caldir!\camerasites.txt" "!bat_calstars_caldir!"
		
			REM Copy from the bat_caldir only those camera files that are referred to in the camerasites file. 
		!bat_CALL! "!bat_exedir!\CopyCamerasFromSitesFile.bat" "!bat_caldir!" "!bat_calstars_caldir!"
	)
	
	
if "!bat_calstars_caldir!" equ "" (!bat_pause! [!bat_batchfile!:COPY_CAL_FILES] "!bat_calstars_caldir!" )
if "!bat_calstars_archivedfilesdir!" equ "" (!bat_pause! [!bat_batchfile!:COPY_CAL_FILES] "!bat_calstars_archivedfilesdir!" )


		REM Set up the temp CapturedFiles that will go into the ZIP...
	(
		if not exist "!bat_calstars_capturedfilesdir!" (
			!bat_logexeclog! MD "!bat_calstars_capturedfilesdir!"
		)


			REM MOVE the new CALSTARS0*.txt a temp dir ["!bat_calstarsdir!"]
			REM 
			REM The reason they don't go to the bat_calstars_capturedfilesdir is because
			REM on the remote cal server, we need to first reconstitute the FF files via the
			REM CALSTARS first.  Then, move these FF files into the CapturedFiles dir.  We 
			REM do this just in case one of the reconstituted FF files has the same name.
			REM
			REM We are choosing 36 minutes because, after analysis, it has been determined that
			REM the number of calibration stars is increased if there is just a little bit of
			REM sky fog in the background due to the end of twilight.  
			REM
		!bat_logexeclog! MOVE /y "!bat_capturedfilesdir!\CALSTARS0???_*.txt" "!bat_calstars_capturedfilesdir!"

			REM Copy one FF file for each camera that is about 36 minutes after capture start
		if NOT DEFINED bat_CAPTURE.REMOTE_FF_Frame_Number (
			if /i "!bat_CAPTURE.fps!" equ "2997" (
				!bat_logexeclog! set bat_CALIBRATION.REMOTE_FF_Frame_Number=0064000
			) else (
				!bat_logexeclog! set bat_CALIBRATION.REMOTE_FF_Frame_Number=0054016
			)
		)
		
		!bat_logexeclog! COPY /y "!bat_capturedfilesdir!\FF*_!bat_CALIBRATION.REMOTE_FF_Frame_Number!.bin" "!bat_calstarsdir!"
	)

	


	REM Validate and copy the detect file
	(
		!bat_logexeclog! set bat_detectfile
		
		if not defined bat_detectfile (
			call :ERROR 44 "bat_detectfile not defined [!bat_detectfile!]"
			goto :SETUP_DIRS_END
		)

		if not exist "!bat_detectfile!" (
			call :ERROR 45 "No valid detect file exists: [!bat_detectfile!]"
			goto :SETUP_DIRS_END
		)

if "!bat_calstars_archivedfilesdir!" equ "" (!bat_pause! [!bat_batchfile!:COPY_CAL_FILES] 4444444 "!bat_calstars_archivedfilesdir!" )
if "!bat_calstars_archivedfilesdir!" equ "" (!bat_pause! [!bat_batchfile!:COPY_CAL_FILES] "!bat_calstars_archivedfilesdir!" )
if "!bat_calstars_caldir!" equ "" (!bat_pause! [!bat_batchfile!:COPY_CAL_FILES] "!bat_calstars_caldir!" )

		if not exist "!bat_calstars_archivedfilesdir!" (
			!bat_logexeclog! MD "!bat_calstars_archivedfilesdir!"
		)
		

if "!bat_calstars_archivedfilesdir!" equ "" (!bat_pause! [!bat_batchfile!:COPY_CAL_FILES] 4444444 "!bat_calstars_archivedfilesdir!" )

		if "!bat_calstars_archivedfilesdir!" equ "" (
			call :ERROR 46 "bat_calstars_archivedfilesdir=!bat_calstars_archivedfilesdir!" 
			goto :SETUP_DIRS_END
		)

		!bat_logexeclog! COPY /y "!bat_detectfile!" "!bat_calstars_archivedfilesdir!"
	)



		REM Create the GCFC.log file as bat_GCFC_tempfile unless it already exists
		REM This GCFC.log file is a simple text file in the bat_capturedfilesdir that
		REM contains a list of cameras found in the CapturedFiles dir
		REM
	    REM                                                                                 !bat_CALL! "!bat_exedir!\CreateCameraListFile.bat" "!bat_capturedfilesdir!"
	!bat_START! /wait "Please Wait - CreateCameraListFile.bat - [!bat_capturedfilesdir!]" /low cmd /c ""!bat_exedir!\CreateCameraListFile.bat" "!bat_capturedfilesdir!""
	
		REM Outer loop: 
		REM For each camera in the GCFC file, iterate in reverse order and determine which CAL
		REM file falls before the last FF file in the capturedfiles dir.
		REM If you find it, copy it to the CapturedFiles dir. 
		REM We need to do this so that we can find the bat_FFlast timestamp so that
		REM the next most recent cal file to that is included with the zip. 
		REM
		REM Performs the loop in reverse camera order so that the last one processed 
		REM [and thus, the last one to call GetFirstLastFF_filenames.bat] leaves 
		REM bat_FFfirst and bat_FFlast as the first/last of the camera of the CALSTARS file.
		REM Also, bat_FF_time_max is left with the timestamp of the newest FF file 
		REM in the capturedfiles dir.
		REM 
	!bat_pushlog! for /f "usebackq tokens=*" ..a in ^(`type "!bat_GCFC_tempfile!"][SORT /r`^) do ^(
	for               /f "usebackq tokens=*" %%a in  (`type "!bat_GCFC_tempfile!"^|SORT /r`) do (
		!bat_logexeclog! set bat_camera_temp=%%a

			rem Usage %~0 "^<var_firstfile^>"    "^<var_lastfile^>"   "^<target directory path^>" ["^<camera^>"]
rem 		!bat_CALL! "!bat_exedir!\GetFirstLastFF_filenames.bat" bat_FFfirst bat_FFlast "!bat_capturedfilesdir!" "%%a"

			REM This is the date/time to look for.  No cal file can be included that is greater.
			REM The idea is to iterate, in ascending order, the cal files, filtered by camera,
			REM Set a variable to the value.  If the next iteration is beyond what we're looking for,
			REM then BREAK the loop. 
			REM
			REM   Example: bat_FF_time_max=20161230_150159_543
			REM
		!bat_logexeclog! set bat_FF_time_max=!bat_FFlast:~6,19!
		
		set bat_cal_temp_time=
		set bat_cal_include=

			REM Inner loop to: 
			REM Loop through the CAL files in the Cal dir, filter by camera. 
		!bat_pushlog! for /f "usebackq tokens=*" ..b in ^(`dir "!bat_caldir!\CAL_!bat_camera_temp!_*.txt" /b /a-d /on`^) do ^(
		for               /f "usebackq tokens=*" %%b in  (`dir "!bat_caldir!\CAL_!bat_camera_temp!_*.txt" /b /a-d /on`) do (
		
				REM CAL_000750_20161230_150000_000.txt
			set bat_cal_temp=%%b
			
				REM 20161230_150000_000
			set bat_cal_temp_time=!bat_cal_temp:~7,19!

				REM if "20161230_150000_000" LEQ "20161230_150159_543" [then it qualified]
			if /i "!bat_cal_temp_time!" LEQ "!bat_FF_time_max!" (
			
					REM bat_cal_include=C:\CAMS\Cal\CAL_000750_20161230_150000_000.txt
				set bat_cal_include=!bat_caldir!\%%b
			)
			
		)&	REM for /f "usebackq tokens=*" ..b in ^(`dir "!bat_caldir!\CAL_!bat_camera_temp!_*.txt" /b /a-d /on`^)
		!bat_poplog! ^) [bat_cal_include=!bat_cal_include!]
		
			REM If a CAL file wasn't found, try looking for it in the ..\Cal\BinFiles dir...
			REM 
		if not defined bat_cal_include (
			REM If you get here, it wasn't found in ..\Cal, but it might be in ..\Cal\BinFiles...

				REM Loop through the CAL files in the Cal dir, filter by camera. 
			!bat_log! for /f "usebackq tokens=*" ..b in ^(`dir "!bat_caldir!\BinFiles\CAL_!bat_camera_temp!_*.txt" /b /a-d /on`^) do ...
			for           /f "usebackq tokens=*" %%b in  (`dir "!bat_caldir!\BinFiles\CAL_!bat_camera_temp!_*.txt" /b /a-d /on`) do (
				set bat_cal_temp=%%b
				
					REM CAL_000252_20170915_041056_382.txt
				set bat_cal_temp_time=!bat_cal_temp:~11,19!

				if /i "!bat_cal_temp_time!" LEQ "!bat_FF_time_max!" (
					set bat_cal_include=!bat_caldir!\%%b
				)
			)&	REM for /f "usebackq tokens=*" ..b in ^(`dir "!bat_caldir!\BinFiles\CAL_!bat_camera_temp!_!bat_FF_time_max!.txt" /b /a-d /on`^)
		)
		
			REM If still can't find a CAL file in either directory, then indicate an error
		if not defined bat_cal_include (
			CALL :ERROR 47 "Can't find a matching Cal file for [!bat_camera!, !bat_FF_time_max!]"
		) else (
			!bat_logexeclog! COPY /y "!bat_cal_include!"   "!bat_calstars_caldir!"
		)
	)&	REM	for /f "usebackq tokens=*" ..a in ^(`type "!bat_GCFC_tempfile!`^) 
	!bat_poplog! ^)
	

:COPY_CAL_FILES_END
!bat_popecho! [RETURN !bat_batchfile!:COPY_CAL_FILES] [bat_returncode=!bat_returncode!, bat_calstarsdir=!bat_calstarsdir!]
!bat_log!
rem !bat_WAIT! 3	:COPY_CAL_FILES_END
!bat_return! !bat_returncode!





	REM #################################################################################
	REM 
	REM bat_TempCalFromZipName means the zip file name on the ftp server
	REM     
	REM RETURNS:
	REM     bat_ftpfile2                       [from :GET_SERVER_FILE_LIST]  !bat_exedir!\ftp.temp_2.txt
	REM     bat_TempCalFromZipName             CALASTARS_000750_2016_12_31_14_37_59_01
	REM     bat_ZipFileNameFull                C:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALASTARS_000750_2016_12_31_14_37_59_01.zip
	REM     bat_ZipSeq                         [from :GET_ZIP_SEQ]
	REM     bat_TempCalFromZipSeq              [from :GET_ZIP_SEQ]
	REM     bat_TempMD5Dir                     [from :CREATE_MD5_FILE] C:\CAMS\CapturedFiles\2016_12_31_14_37_59
	REM     bat_TempMD5Name                    [from :CREATE_MD5_FILE] CALASTARS_000750_2016_12_31_14_37_59_01_md5
	REM     bat_TempMD5File                    [from :CREATE_MD5_FILE] CALASTARS_000750_2016_12_31_14_37_59_01_md5.txt
	REM     bat_TempMD5Full                    [from :CREATE_MD5_FILE] C:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALASTARS_000750_2016_12_31_14_37_59_01_md5.txt
	REM     
:ZIP_CALSTARS
!bat_pushlog! [CALL !bat_batchfile!:ZIP_CALSTARS] %*

		REM Produces !bat_ftpfile2!, which is the list of all CALSTARS*.* files on the FTP server.
	CALL :GET_SERVER_FILE_LIST

	call :GET_ZIP_SEQ "!bat_TempCalFromZipName!"
	
	!bat_logexeclog! set bat_TempCalFromZipName=!bat_TempCalFromZipName!_!bat_TempCalFromZipSeq!
	!bat_logexeclog! set bat_TempCalFromZipFile=!bat_TempCalFromZipName!.zip

	!bat_logexeclog! set bat_ZipFileNameFull=!bat_capturedfilesdir!\!bat_TempCalFromZipFile!

		REM Make the ZIP file now...
	!bat_logexeclog! call "!bat_exedir!\7zip.bat" -r "!bat_ZipFileNameFull!" "!bat_calstarsdir!\*"
	set bat_returncode=!errorlevel!

	call :CREATE_MD5_FILE "!bat_ZipFileNameFull!"
	set bat_returncode=!errorlevel!

:ZIP_CALSTARS_END
!bat_popecho! [RETURN !bat_batchfile!:ZIP_CALSTARS] [bat_returncode=!bat_returncode!
!bat_log! `                                         bat_ZipFileNameFull=!bat_ZipFileNameFull!]
!bat_log!
rem !bat_WAIT! 3	:ZIP_CALSTARS_END
!bat_return! !bat_returncode!






	REM #################################################################################
	REM Fetch the list of ALL CALSTARS*.* files on the FTP server so we can perform 
	REM filtered searches using FINDSTR later.
	REM 
	REM Returns:
	REM    bat_ftpfile2			    a temp file containing a list of all the CALSTARS*.* 
	REM                             files on the server
	REM    
:GET_SERVER_FILE_LIST
!bat_pushlog! [CALL !bat_batchfile!:GET_SERVER_FILE_LIST] 

		REM Set the name of the file containing the server files list...
	!bat_logexeclog! set bat_ftpfile2=!bat_exedir!\ftp.temp_2.txt

		!bat_CALL! "!bat_exedir!\ReadRemoteConfig.bat"

			REM Now download a list of matching files on the server. 
		rem  site=bat_calftpsite
		!bat_log! echo.user !bat_calftpuser! } "!bat_ftpfile!"
		
		echo.user !bat_calftpuser!>"!bat_ftpfile!"
		echo.!bat_calftppassword!>>"!bat_ftpfile!"
		echo.bin>>"!bat_ftpfile!"
		echo.cd !bat_calftpdir!>>"!bat_ftpfile!"
		echo.literal pasv>>"!bat_ftpfile!"
		echo.lcd "!bat_calstarsdir!">>!bat_ftpfile!"
		echo.literal pasv>>"!bat_ftpfile!"
		echo.ls "CALSTARS??????_*.*">>!bat_ftpfile!"
		echo.quit>>"!bat_ftpfile!"

		!bat_echolog! CALL FTP -n -s:"!bat_ftpfile!" !bat_calftpsite!][FINDSTR /i /r /c:"^CALSTARS......_.*" } "!bat_ftpfile2!"
		              CALL FTP -n -s:"!bat_ftpfile!" !bat_calftpsite! |FINDSTR /i /r /c:"^CALSTARS......_.*">"!bat_ftpfile2!"
		set bat_returncode=!errorlevel!
	
			REM CALSTARS000257_2017_09_15_03_00_00.txt
		!bat_echolog! SORT "!bat_ftpfile2!" ][ SORT /+14 }"!bat_ftpfile2!.bak"
		              SORT "!bat_ftpfile2!"  | SORT /+14 >"!bat_ftpfile2!.bak"
		!bat_logexeclog! MOVE /y "!bat_ftpfile2!.bak"  "!bat_ftpfile2!"
	
		!bat_logexec! TYPE "!bat_ftpfile2!"
		set bat_returncode=!errorlevel!

:GET_SERVER_FILE_LIST_END
!bat_popecho! [RETURN !bat_batchfile!:GET_SERVER_FILE_LIST] [bat_returncode=!bat_returncode!, bat_ftpfile2=!bat_ftpfile2!]
!bat_log!
rem !bat_WAIT! 3	:GET_SERVER_FILE_LIST_END
!bat_return! !bat_returncode!







	REM #################################################################################
	REM Gets the next sequence number after the last CALSTARS*.zip file that already
	REM exists on the server.  
	REM 
	REM RETURNS:
	REM     bat_ZipSeq              NUMERIC value without leading "0" as the next one in 
	REM                             sequence and it is used in the creation of the new 
	REM                             Zip file.
	REM 
	REM     bat_TempCalFromZipSeq   TEXT version of the sequnce value with leading "0". 
	REM 
	REM This needs to change so that it lists files on the FTP server...
	REM
:GET_ZIP_SEQ [IN "!bat_TempCalFromZipName!"]
!bat_pushlog! [CALL !bat_batchfile!:GET_ZIP_SEQ] %*

		REM This section lists the already existing zip files on the server
		REM to determine what the last file in sequence is.  Then establish the next one in
		REM sequence afte that.  Reading them to get the current sequence should be another 
		REM routine.
		REM 
		REM FINDSTR /i /r /c:"CALSTARS000750_2016_12_31_14_37_59_...zip" "ftp_temp_6.txt"
		REM           to get: CALSTARS000750_2016_12_31_14_37_59_01.zip

	set bat_tempzip=
	set bat_tempzipname=
	
		REM Filter by the given CALSTARS filename [example: CALSTARS0750_2016_12_31_14_37_59.txt]
		REM If there is more than one matching zip file, then we can determine the sequence number by 
		REM incrementing the sequence number
		
	!bat_pushlog! for /f "usebackq tokens=*" ..a in ^(`FINDSTR /i /r /c:"%~1_...zip" "!bat_ftpfile2!"`^) do ^(
	for               /f "usebackq tokens=*" %%a in  (`FINDSTR /i /r /c:"%~1_...zip" "!bat_ftpfile2!"`) do (
		set bat_tempzip=%%a
		set bat_tempzipname=%%~na
	)
	!bat_poplog! ^)
	
	if not defined bat_tempzipname (
		set /a bat_ZipSeq=0
	) else (
		set /a bat_ZipSeq=!bat_tempzipname:~-2!
	)
	
	set /a bat_ZipSeq+=1
	
	set bat_TempCalFromZipSeq=00!bat_ZipSeq!
	set bat_TempCalFromZipSeq=!bat_TempCalFromZipSeq:~-2!
	

:GET_ZIP_SEQ_END
	set bat_tempzip=
	set bat_tempzipname=
!bat_popecho! [RETURN !bat_batchfile!:GET_ZIP_SEQ] [bat_ZipSeq=!bat_ZipSeq!, bat_TempCalFromZipSeq=!bat_TempCalFromZipSeq!]
!bat_log!
rem !bat_WAIT! 3	:GET_ZIP_SEQ_END
!bat_return!





	REM #################################################################################
	REM This routine creates the MD5 hash flag file that will be uploaded alongside
	REM the CALSTARS*.zip file. 
	REM 
	REM RETURNS:
	REM     bat_TempMD5Dir               C:\CAMS\CapturedFiles\2016_12_31_14_37_59
	REM     bat_TempMD5Name              CALASTARS0750_2016_12_31_14_37_59_01_md5
	REM     bat_TempMD5File              CALASTARS0750_2016_12_31_14_37_59_01_md5.txt
	REM     bat_TempMD5Full              C:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALASTARS0750_2016_12_31_14_37_59_01_md5.txt
	REM     
:CREATE_MD5_FILE [ IN "!bat_TempCalFromZipName!"]
!bat_pushlog! [CALL !bat_batchfile!:CREATE_MD5_FILE] 

	                 set bat_TempMD5Dir=%~dp1::
	                 set bat_TempMD5Dir=!bat_TempMD5Dir:\::=!
	!bat_logexeclog! set bat_TempMD5Dir=!bat_TempMD5Dir::::=!
	
	!bat_logexeclog! set bat_TempMD5Name=%~n1_md5
	!bat_logexeclog! set bat_TempMD5File=%~n1_md5.txt
	!bat_logexeclog! set bat_TempMD5Full=%~dp1!bat_TempMD5File!

		REM Make the MD5 file now of the zip file [%~1]...
	!bat_CALL! "!bat_exedir!\md5.exe" -o"!bat_TempMD5Full!" "%~1"
	set bat_returncode=!errorlevel!


:CREATE_MD5_FILE_END
!bat_popecho! [RETURN !bat_batchfile!:CREATE_MD5_FILE] [bat_returncode=!bat_returncode!,
!bat_log! `                                         bat_TempMD5Dir=!bat_TempMD5Dir!,
!bat_log! `                                         bat_TempMD5Name=!bat_TempMD5Name!,
!bat_log! `                                         bat_TempMD5File=!bat_TempMD5File!,
!bat_log! `                                         bat_TempMD5Full=!bat_TempMD5Full!,
!bat_log! `                                         zip file=%~1]
!bat_log!
rem !bat_WAIT! 3	:CREATE_MD5_FILE_END
!bat_return! !bat_returncode!








	REM #################################################################################
	REM Updates the queue file with a queue entry containing the information required to
	REM re-incorporate the results into the capture session.  
	REM 
	REM Also copies the zip file that is uploaded to the AutoCal_Remote_Queue dir. 
	REM 
:PREP_QUEUE [IN "bat_ZipFileNameFull" "bat_capturedfilesdir" "bat_caldir" "bat_autocal_continue_cmdline"]
!bat_pushlog! [CALL !bat_batchfile!:PREP_QUEUE] [%1, %2, %3, %4]

		REM Make the queue entry to the local queue [bat_autocal_remote_queue] so it can be compared later
	!bat_logexec! echo.%~1,%~2,%~3,%4}}"!bat_autocal_remote_queue!"
	              echo.%~1,%~2,%~3,%4>>"!bat_autocal_remote_queue!"

		REM Move the zip file to the queue folder on the server
	!bat_logexeclog! MOVE /y "!bat_ZipFileNameFull!" "!bat_autocal_remote_queue_dir!\"
	!bat_logexeclog! set bat_ZipFileNameFull=!bat_autocal_remote_queue_dir!\!bat_TempCalFromZipFile!
	
		REM Move the _md5.txt file to the queue folder on the server
	!bat_logexeclog! MOVE /y "!bat_TempMD5Full!" "!bat_autocal_remote_queue_dir!\"
	!bat_logexeclog! set bat_TempMD5Full=!bat_autocal_remote_queue_dir!\!bat_TempMD5File!
	!bat_logexeclog! set bat_TempMD5Dir=!bat_autocal_remote_queue_dir!
	
	!bat_logexeclog! RD /S /Q "%~dp1CALSTARS"
	
:PREP_QUEUE_END
!bat_popecho! [RETURN !bat_batchfile!:PREP_QUEUE] [bat_returncode=!bat_returncode!
!bat_log! `                                         bat_ZipFileNameFull=!bat_ZipFileNameFull!
!bat_log! `                                         bat_TempMD5Full=!bat_TempMD5Full!]
!bat_log!
rem !bat_WAIT! 3	:PREP_QUEUE_END
!bat_return!




	REM #################################################################################
	REM This script is responsible for mapping a drive letter to the server/share where
	REM the file will be uploaded.  This could be a mapped share or ftp. However,
	REM to download, another share will need to be used.  It might be that we need to
	REM write a custom script to call for the upload...
	REM 
	REM Should use "!bat_exedir!\!bat_CALIBRATION.REMOTE_config_file!"
	REM
	REM RETURNS:
	REM    bat_mapped_From              Location of the local server cache dir. 
	REM 
:MAP_DRIVE
!bat_pushlog! [CALL !bat_batchfile!:MAP_DRIVE] %*

	set bat_returncode=0

	!bat_echolog! net use m: /d
	!bat_echolog! net use m: \\server\uploadFROMshare /user:user password /persistent:no
	!bat_echolog! net use N: /d
	!bat_echolog! net use N: \\server\uploadRESULTSshare /user:user password /persistent:no
	!bat_echolog! set bat_returncode=!errorlevel!

	REM Check to ensure that you have a valid drive mapping...
	
	REM until we fix this, hard-code the connection to test locally...
	!bat_logexeclog! set bat_mapped_From=!bat_CALIBRATION.REMOTE_server_dir!
	!bat_logexeclog! set bat_mapped_Results=!bat_CALIBRATION.REMOTE_server_dir!

:MAP_DRIVE_END
!bat_popecho! [RETURN !bat_batchfile!:MAP_DRIVE] [bat_returncode=!bat_returncode!, bat_mapped_From=!bat_mapped_From!, bat_mapped_Results=!bat_mapped_Results!]
!bat_log!
rem !bat_WAIT! 3	:MAP_DRIVE_END
!bat_return!




	REM #################################################################################
	REM 
	REM RETURNS:
	REM     bat_calftpsite
	REM     bat_calftpuser
	REM     bat_calftppassword
	REM     bat_calftpdir
	REM 
	REM     bat_zipfile
	REM     bat_ftpfile
	REM     
:UPLOAD_ZIP [IN bat_ZipFileNameFull]
!bat_pushlog! [CALL !bat_batchfile!:UPLOAD_ZIP] %*

rem	!bat_logexeclog! COPY /y "!bat_autocal_remote_queue_dir!\!bat_TempCalFromZipName!!bat_TempCalFromZipExt!" "!bat_mapped_From!"
rem	set bat_returncode=!errorlevel!
	
	!bat_CALL! "!bat_exedir!\ReadRemoteConfig.bat"
	

		REM Set these variables so that the ftp_transfer_seti.bat script can use them.
	!bat_logexeclog! set bat_ftpsite=!bat_calftpsite!
	!bat_logexeclog! set bat_ftpuser=!bat_calftpuser!
	!bat_logexeclog! set bat_ftppassword=!bat_calftppassword!
	!bat_logexeclog! set bat_ftpdir=!bat_calftpdir!
	
		REM We use bat_zipfile instead of bat_ZipFileNameFull to be compatible with the ftp_transfer_seti.bat script. 
	!bat_logexeclog! set bat_zipfile=%~1

		REM Upload the ZIP file and perform all the normal md5 and other validation checks
	!bat_log!  "!bat_exedir!\ftp_transfer_seti.bat" FTPTRANSFER "!bat_zipfile!"
	!bat_CALL! "!bat_exedir!\ftp_transfer_seti.bat" FTPTRANSFER "!bat_zipfile!"
	
		REM Move the zip file to the queue folder on the server
rem	!bat_logexeclog! MOVE /y "!bat_ZipFileNameFull!" "!bat_autocal_remote_queue_dir!\"
rem	!bat_logexeclog! set bat_ZipFileNameFull=!bat_autocal_remote_queue_dir!\!bat_TempCalFromZipFile!
	
	
	!bat_echolog!
	!bat_echolog! Now build the "!bat_ftpfile!" FTP command file for uploading the MD5 file to the FTP server.
	!bat_echolog!
	
		REM Now upload the matching MD5 flag file...
	echo.user !bat_calftpuser!>"!bat_ftpfile!"
	echo.!bat_calftppassword!>>"!bat_ftpfile!"
	echo.bin>>"!bat_ftpfile!"
	echo.cd !bat_calftpdir!>>"!bat_ftpfile!"
	echo.literal pasv>>"!bat_ftpfile!"
	echo.lcd "!bat_TempMD5Dir!">>!bat_ftpfile!"
	echo.literal pasv>>"!bat_ftpfile!"
	echo.put "!bat_TempMD5File!">>!bat_ftpfile!"
	echo.quit>>"!bat_ftpfile!"

	!bat_logexec! type "!bat_ftpfile!
	!bat_echolog!
	
	!bat_logexec! dir "!bat_TempMD5Full!"
	!bat_echolog!
	
		REM Now upload the MD5 flag file
	!bat_logexec! CALL FTP -n -s:"!bat_ftpfile!" !bat_calftpsite!
	set bat_returncode=!errorlevel!



:UPLOAD_ZIP_END
!bat_popecho! [RETURN !bat_batchfile!:UPLOAD_ZIP] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3	:UPLOAD_ZIP_END
!bat_return!









	REM #################################################################################
	REM This routine cleans up all the temp files and stuff before exiting the program.
	REM I need to figure out a way to handle the case when the network is down and things
	REM can be processed again. Also, I need to figure out a way for the 
	REM AutoCal_LocalDownload.bat script to reinsert itself into the autocams autonomous 
	REM processing mid-stream after AZ_AUTOCAL. 
	REM 
:CLEANUP
!bat_pushlog! [CALL !bat_batchfile!:CLEANUP] %*



:CLEANUP_END
!bat_popecho! [RETURN !bat_batchfile!:CLEANUP] [!bat_getdirname!, !bat_mainspec!]
!bat_log!
rem !bat_WAIT! 3	:CLEANUP_END
!bat_return!




	REM ############################################################################################
	REM This routine might make a good template for scanning for matching CALSTARS files.
	REM
:PROCESS_REMAINING_QUEUE
!bat_pushlog! [CALL !bat_batchfile!:PROCESS_REMAINING_QUEUE] %*


:PROCESS_REMAINING_QUEUE_LOOP
	!bat_CALL! "!bat_exedir!\GetFileCount.bat" "!bat_TempCalFrom!\CALSTARS???????_????_??_??_??_??_??.zip"
	set bat_CountFrom=!errorlevel!
	
	if not defined bat_CountFrom (
		goto :PROCESS_REMAINING_QUEUE_END
	)

	if !bat_CountFrom! LSS 1 (
		goto :PROCESS_REMAINING_QUEUE_END
	)
	
	!bat_echolog!
	!bat_echolog! ######################################################
	!bat_echolog! There are "!bat_CountFrom!" files to process...
	!bat_echolog!
	
	set bat_TempCalFromZipFull=
	set bat_capturedfilesdir=
	set bat_TempCalFromZipName=
	set bat_TempCalFromZipExt=
rem	set bat_TempCalFrom=!bat_TempCalFrom!
	set bat_captureddate=
	set bat_capturedtime=
	set bat_captureddatetime=
	set bat_calstarsdir=
	set bat_capturedfilesdir=
	set bat_archivedfilesdir=
	set bat_TempCalResults=





	
		REM Loop through the remaining files and skip all but the oldest one...
	!bat_pushlog! for /f "usebackq tokens=*" ..a in ^(`dir "!bat_TempCalFrom!\CALSTARS???????_????_??_??_??_??_??.zip" /a-d /b /o-d`^) do ^(
	for               /f "usebackq tokens=*" %%a in  (`dir "!bat_TempCalFrom!\CALSTARS???????_????_??_??_??_??_??.zip" /a-d /b /o-d`) do (
		!bat_logexeclog! set bat_TempCalFromZipFull=!bat_TempCalFrom!\%%a
	)
	!bat_poplog! ^)
	

	
	call :PARSEZIP "!bat_TempCalFromZipFull!"

	call :MAIN

	
		REM Go back to the counter section in case new files have arrived...
		REM If we had used a FOR loop, it would not accommodate new files arriving
		REM after the loop started.
	goto :PROCESS_REMAINING_QUEUE_LOOP

	

:PROCESS_REMAINING_QUEUE_END
!bat_popecho! [RETURN !bat_batchfile!:PROCESS_REMAINING_QUEUE] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3	:PROCESS_REMAINING_QUEUE_END
!bat_return!



	REM ########################################################################
	REM This routine re-launches the MonitorDirs.exe program to begin monitoring for new
	REM uploads.  This script should be used to launch the monitor in the first place
	REM just in case new files were uploaded while the monitor was not running. 
	REM 
:RELAUNCH_MONITOR
!bat_pushlog! [CALL !bat_batchfile!:RELAUNCH_MONITOR] %*

	process | Find /i "monitordir.exe"
	if !errorlevel! equ 0 (
		goto :RELAUNCH_MONITOR_END
	)

	!bat_echolog! Relaunching the MonitorDir.exe program...
	!bat_echolog!
	
	!bat_echolog! !bat_CALIBRATION.REMOTE_autocal_localupload_cmdline!
	              !bat_CALIBRATION.REMOTE_autocal_localupload_cmdline!
	

:RELAUNCH_MONITOR_END
!bat_popecho! [RETURN !bat_batchfile!:RELAUNCH_MONITOR] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3	:RELAUNCH_MONITOR_END
!bat_return!




	REM ########################################################################
	REM 
:DELETABLE
!bat_pushlog! [CALL !bat_batchfile!:DELETABLE]


	!bat_logexeclog! set bat_autocal_remote_queue_dir=!bat_camsdir!\AutoCal_Remote_Queue
	!bat_logexeclog! set bat_autocal_remote_queue=!bat_autocal_remote_queue_dir!\AutoCal_Remote_Queue.txt
	
	!bat_echolog! bat_TempCalFromZipName=!bat_TempCalFromZipName!
	
		REM Convert: "CALSTARS0000750_2016_12_31_14_37_59_01_md5.txt" 
		REM to:      "CALSTARS0000750_2016_12_31_14_37_59_01_Results.zip"
		REM    
	!bat_logexeclog! set bat_TempCalResultsZipName=!bat_TempMD5Name:_md5.txt=_Results.zip!
	
		REM Convert: "CALSTARS0000750_2016_12_31_14_37_59_01_md5.txt" 
		REM to:      "CALSTARS0000750_2016_12_31_14_37_59_01_Results.txt"
		REM 
	!bat_logexeclog! set bat_TempCalResultsMD5Name=!bat_TempMD5Name:_md5.txt=_Results.txt!
	


	REM ====================================================================================
	REM creating the _deletable.txt file
	REM 
	
		REM Convert: "CALSTARS0000750_2016_12_31_14_37_59_01_Results" 
		REM to:      "CALSTARS0000750_2016_12_31_14_37_59_01_Deletable.txt"
		REM 
	!bat_logexeclog! set bat_DeleteFileName=!bat_TempCalFromZipName:_Results=_Deletable.txt!
	!bat_logexeclog! set bat_DeleteFileFull=!bat_CalStarsDir!\!bat_TempCalFromZipName:_Results=_Deletable.txt!
	
	echo."!bat_TempCalFromZipName!">"!bat_DeleteFileFull!"
	echo."!bat_TempMD5Name!">>"!bat_DeleteFileFull!"
	echo."!bat_TempCalResultsZipName!">>"!bat_DeleteFileFull!"
	echo."!bat_TempCalResultsMD5Name!">>"!bat_DeleteFileFull!"
	echo."!bat_DeleteFileName!">>"!bat_DeleteFileFull!"

		!bat_log! `    Note: Building the "..\ftp.temp.txt" file...

			REM Put _Deletable.txt on the ftp
		echo.user !bat_calftpuser!>"!bat_ftpfile!"
		echo.!bat_calftppassword!>>"!bat_ftpfile!"
		echo.bin>>"!bat_ftpfile!"
		echo.cd !bat_calftpdir!>>"!bat_ftpfile!"
		echo.literal pasv>>"!bat_ftpfile!"
		echo.lcd "!bat_CalStarsDir!">>!bat_ftpfile!"
		echo.literal pasv>>"!bat_ftpfile!"
		echo.put "!bat_DeleteFileName!">>"!bat_ftpfile!"
		echo.quit>>"!bat_ftpfile!"

		!bat_logexec! TYPE "!bat_ftpfile!"

		!bat_logexec! call FTP -n -s:"!bat_ftpfile!" "!bat_calftpsite!"
		set bat_returncode=!bat_logexec_returncode!

		!bat_echolog!
		!bat_echolog! '    FTP exit code=!bat_returncode!
		!bat_echolog!

:DELETABLE_END
!bat_popecho! [RETURN !bat_batchfile!:CLEANUP] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3
!bat_return!





	REM #################################################################################
	REM This script will be called like this:
	REM 
	REM   AutoCal_LocalUpload.bat /capturedir "^<bat_capturedfilesdir^>" "^<bat_caldir^>"
	REM
	REM bat_capturedfilesdir should not be expected to be correct when this script is
	REM launched. 
	REM
:PARSEARGS
!bat_pushlog! [CALL !bat_batchfile!:PARSEARGS] %*

	set bat_returncode=0

		REM Set the DEFAULT working dirs
	!bat_logexeclog! set bat_capturedfilesdir=
	!bat_logexeclog! set bat_captureddirname=
	!bat_logexeclog! set bat_caldir=
	!bat_logexeclog! set bat_camera=&	rem 
	!bat_logexeclog! set bat_archivedfilesdir=


		REM Set default for bat_calstarsdir.  bat_calstarsdir is the base
		REM directory for all the work is done in this routine.
		REM
	!bat_logexeclog! set bat_calstarsdir=

		REM ==================================================================================
		REM Set defaults that might be missing from autocams.params.ini
		REM
	if not defined bat_CALIBRATION.REMOTE_skip (
	
			REM Set the default for FTP_CalStarsExtraction.exe to skip at least 5 minutes 
			REM between sampled FF files [71 for NTSC and 59 for PAL]
			REM 5 minutes in the equation below is 305 seconds.  The extra 5 seconds 
			REM helps round it up to the next second.
			REM
		set /a bat_CALIBRATION.REMOTE_skip=305 * !bat_CAPTURE.fps! / 25600
		!bat_logexeclog! set bat_CALIBRATION.REMOTE_skip=71
	)

	if not defined bat_CALIBRATION.REMOTE_nstars (
			REM Set the number of stars to include in the CALSTARS and reconsituted FF files at 300
		!bat_logexeclog! set bat_CALIBRATION.REMOTE_nstars=300
	)

	if not defined bat_CALIBRATION.REMOTE_autocal_enabled (
		set bat_CALIBRATION.REMOTE_autocal_enabled=true
	)

	if not defined bat_CALIBRATION.REMOTE_autocal_remote_cmdline (
		!bat_logexeclog! set bat_CALIBRATION.remote_autocal_remote_cmdline=start "" /low /min "!bat_exedir!\MonitorDir.exe"        /FilenameChange /RunOnce /AddFileName /Creation /Activate     "!bat_calstarsdir!" "!bat_exedir!\AutoCal_Remote.bat /zip"
	)

	if not defined remote_autocal_localupload_cmdline (
		!bat_logexeclog! set bat_CALIBRATION.REMOTE_autocal_localupload_cmdline=start "" /low /min "!bat_exedir!\monitordir.exe"   /FilenameChange /RunOnce /AddFileName /Creation /Activate     "!bat_calstarsdir!" "!bat_exedir!\AutoCal_LocalDownload.bat /zip"
	)

		REM This config file contains details of connecting to a remote autocal server
	if not defined bat_CALIBRATION.REMOTE_config_file (
		!bat_logexeclog! set bat_CALIBRATION.REMOTE_config_file=AutoCal_Remote_config.txt
	)

	

:PARSEARGS_LOOP
rem echo.:PARSEARGS_LOOP [%1] [%2]

	if /i "%~1" equ "" (
		goto :PARSEARGS_LOOPBREAK
	)
	
	if /i "%~1" equ "/capturedir" (
		!bat_logexeclog! set bat_capturedfilesdir=%~2

		shift
		shift
		goto :PARSEARGS_LOOP
	)

	if /i "%~1" equ "/caldir" (
		!bat_logexeclog! set bat_caldir=%~2
		shift
		shift
		goto :PARSEARGS_LOOP

	)
	
	if /i "%~1" equ "/detect" (
		!bat_logexeclog! set bat_detectfile=%~2
		shift
		shift
		goto :PARSEARGS_LOOP

	)
	
		REM /session can be used when the bat_capturedfilesdir does not have a dirname that actually is a CAMS date
	if /i "%~1" equ "/session" (
		!bat_logexeclog! set bat_captureddirname=%~2
		shift
		shift
		goto :PARSEARGS_LOOP

	)
	

:PARSEARGS_LOOPBREAK
!bat_log! [CALL !bat_batchfile!:PARSEARGS_LOOPBREAK] %*


		REM bat_capturedfilesdir should be a date_time in the format:
		REM    "C:\CAMS\CapturedFiles\2016_01_26_14_36_59"
		REM      or 
		REM    "C:\CAMS\SubmissionFiles\750\CapturedFiles\2016_01_26_14_36_59"
		REM    
	if not defined bat_capturedfilesdir (
		set bat_returncode=1
		!bat_echolog!
		!bat_echolog!
		!bat_echolog! `    ERROR [!bat_returncode!] - bat_capturedfilesdir is not defined.
		!bat_echolog!
		!bat_echolog!
		!bat_WAIT! 10
		goto :PARSEARGS_END
	)
	
	
	if not defined bat_captureddirname (
		!bat_logexeclog! set bat_captureddirname=!bat_capturedfilesdir:~-19!
	)

	!bat_logexeclog! set bat_captureddate=!bat_captureddirname:~0,10!
	!bat_logexeclog! set bat_capturedtime=!bat_captureddirname:~-8!
	!bat_logexeclog! set bat_captureddatetime=!bat_captureddate!_!bat_capturedtime!
	
	!bat_logexeclog! set bat_archivedfilesdir=!bat_capturedfilesdir:CapturedFiles=ArchivedFiles!

	!bat_CALL! "!bat_exedir!\GetDetectFile.bat" bat_detectfile "!bat_archivedfilesdir!" "!bat_capturedfilesdir!"
	!bat_TITLE!

		REM Set up the variables for the working directories
	!bat_logexeclog! set bat_calstarsdir=!bat_capturedfilesdir!\CALSTARS
	!bat_logexeclog! set bat_calstars_capturedfilesdir=!bat_calstarsdir!\CapturedFiles\!bat_captureddatetime!
	!bat_logexeclog! set bat_calstars_archivedfilesdir=!bat_calstarsdir!\ArchivedFiles\!bat_captureddatetime!
	!bat_logexeclog! set bat_calstars_caldir=!bat_calstarsdir!\Cal

	if not defined bat_capturedfilesdir (
		set bat_returncode=2
		!bat_echolog!
		!bat_echolog! ERROR - Unable to preceed because /capturedir argument is missing.
		!bat_echolog!
		goto :PARSEARGS_END
	)
	
	if not defined bat_caldir (
		set bat_returncode=3
		!bat_echolog!
		!bat_echolog! ERROR - Unable to preceed because /caldir argument is missing.
		!bat_echolog!
		goto :PARSEARGS_END
	)
	

	!bat_logexeclog! set bat_DirListingCalStars=!bat_calstars_capturedfilesdir!\DirListingCalstars.txt"

	!bat_logexeclog! set bat_GCFC_tempfile=!bat_capturedfilesdir!\GCFC.log
	
	!bat_logexeclog! set bat_autocal_remote_queue_dir=!bat_camsdir!\AutoCal_Remote_Queue
	!bat_logexeclog! set bat_autocal_remote_queue=!bat_autocal_remote_queue_dir!\AutoCal_Remote_Queue.txt
	!bat_logexeclog! set bat_ftp_tempfile5=!bat_camsdir!\ftp.temp_queue.txt

	!bat_logexeclog! set bat_ftpfile=!bat_exedir!\ftp.temp.txt



:PARSEARGS_END
!bat_poplog! [RETURN !bat_batchfile!:PARSEARGS] [bat_returncode=!bat_returncode!,
!bat_log! `                                      bat_capturedfilesdir=!bat_capturedfilesdir!,
!bat_log! `                                      bat_caldir=!bat_caldir!,
!bat_log! `                                      bat_captureddatetime=!bat_captureddatetime!,
!bat_log! `                                      bat_capturedfilesdir=!bat_capturedfilesdir!,
!bat_log! `                                      bat_archivedfilesdir=!bat_archivedfilesdir!,
!bat_log! `                                      bat_detectfile=!bat_detectfile!,
!bat_log! `                                      bat_autocal_remote_queue_dir=!bat_autocal_remote_queue_dir!,
!bat_log! `                                      bat_autocal_remote_queue=!bat_autocal_remote_queue_dir!\AutoCal_Remote_Queue.txt
!bat_log! `                                      bat_calstarsdir=!bat_calstarsdir!,
!bat_log! `                                      bat_calstars_capturedfilesdir=!bat_calstars_capturedfilesdir!,
!bat_log! `                                      bat_calstars_archivedfilesdir=!bat_calstars_archivedfilesdir!,
!bat_log! `                                      bat_calstars_caldir=!bat_calstars_caldir!,
!bat_log! `                                      bat_DirListingCalStars=!bat_DirListingCalStars!,
!bat_log! `                                     ]
!bat_log!
rem !bat_WAIT! 3	:PARSEARGS_END
!bat_return! !bat_returncode!



:GET_QUEUE_ITEMS
!bat_pushlog! [CALL !bat_batchfile!:GET_QUEUE_ITEMS] [%1, %2, %3]

			REM making downloadlist from queue

			REM If ftp.temp_queue.txt already exists, delete it...
		if exist "!bat_ftp_tempfile5!" (
			!bat_logexeclog! DEL /q "!bat_ftp_tempfile5!"
		)

			REM The first field in the queue is the original zip file [without the _Results suffix] 
			REM and save all the queue entries into a "ftp.temp_queue.txt" file.  That file 
			REM contains the name of all the "*_Results.zip" files to search for. 
			REM 
		!bat_pushlog! for /f "usebackq tokens=1* delims=," ..A in ^("!bat_autocal_remote_queue!"^) do ^(
		for               /f "usebackq tokens=1* delims=," %%A in  ("!bat_autocal_remote_queue!") do (
		  set bat_temp=%%~dpnA
		  !bat_logexeclog! set bat_search=!bat_temp!.zip
		  echo.!bat_search!>>!bat_ftp_tempfile5!
		)
		!bat_poplog! ^)

			REM Output the resulting list of "_Results.zip" files to process...
		!bat_logexec! TYPE "!bat_ftp_tempfile5!"
		set bat_returncode=!errorlevel!
		

:GET_QUEUE_ITEMS_END
!bat_poplog! [RETURN !bat_batchfile!:GET_QUEUE_ITEMS] [!bat_returncode!]
!bat_log!
!bat_WAIT! 3	:GET_QUEUE_ITEMS_END
!bat_return! !bat_returncode!




	REM ######################################################################
	REM
:FILTER_LIST
!bat_pushlog! [CALL !bat_batchfile!:FILTER_LIST] [%1, %2, %3]




		REM Now iterate the list of source files and exclude those with ".zip" for the master list of zip files.  Some of those may
		REM need to be excluded because of the presence of matching "_Results" files. 
		REM 
		REM For each "_Results.zip" file found on the server, 
		REM
	!bat_pushlog! for /f "usebackq tokens=*" ..a in ^(`FINDSTR /i /r /c:"CALSTARS......_...._.._.._.._.._.._..\.zip" "!bat_ftp_tempfile5!"`^) do ^(
	for               /f "usebackq tokens=*" %%a in  (`FINDSTR /i /r /c:"CALSTARS......_...._.._.._.._.._.._..\.zip" "!bat_ftp_tempfile5!"`) do (
		!bat_pushlog! [[[[ %%~na ]]]]

			REM	c:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALSTARS000750_2016_12_31_14_37_59_01.zip
		!bat_logexeclog! set bat_capturedzipfile=%%a

						 set bat_capturedfilesdir=%%~dpa::
						 set bat_capturedfilesdir=!bat_capturedfilesdir:\::=!
		!bat_logexeclog! set bat_capturedfilesdir=!bat_capturedfilesdir:::=!

			REM	C:\CAMS\Temp\CALSTARS000750_2016_12_31_14_37_59_01.zip
		!bat_logexeclog! set bat_tempzipfile=!bat_ftp_tempdir!\%%~nxa
		!bat_logexeclog! set bat_tempzipname=%%~na&	rem	CALSTARS000750_2016_12_31_14_37_59_01

		
			REM	CALSTARS0750_2016_12_31_14_37_59_01.zip
		!bat_logexeclog! set bat_ZipFileNameFull=!bat_autocal_remote_queue_dir!\!bat_tempzipname!.zip
		
		
		
			REM Convert: "CALSTARS000750_2016_12_31_14_37_59_01" 
			REM to:      "CALSTARS000750_2016_12_31_14_37_59_01.txt"
			REM 
		rem !bat_logexeclog! set bat_resultsflagfile=!bat_tempzipname:=_Results.txt!

			REM Convert: "CALSTARS000750_2016_12_31_14_37_59_01" 
			REM to:      "CALSTARS000750_2016_12_31_14_37_59_01_downloaded.txt"
			REM 
		rem !bat_logexeclog! set bat_downloadedflagfile=!bat_tempzipname:_Results=_Deletable.txt!



			REM First, check if there is an MD5 file [which is named as "CALSTARS0*_Results.txt"] to see if it's finished uploading. 
			REM
			REM Only download if there is [A] a "*_Results.txt" flag file and [B] if there is NOT a "*_downloaded.txt" flag file.

		!bat_log!
		!bat_logexeclog! FINDSTR /r /i /c:"!bat_tempzipname!.zip" "!bat_ftpfile2!"
		if /i "!errorlevel!" NEQ "0" (

			!bat_log! **** UPLOADING no ".zip" file on the server: "!bat_ZipFileNameFull!"...
			!bat_log!
			
			call :UPLOAD_ZIP "!bat_ZipFileNameFull!"
			set bat_returncode=!errorlevel!
			if !bat_returncode! neq 0 (
				goto :DELETABLE
				
			)

		) else (

			!bat_log! **** NO UPLOADING ".zip" file already on the server: "!bat_ZipFileNameFull!"...
			!bat_log!
				
			
			
		)

		!bat_poplog!............................
	)
	!bat_poplog! ^) end of: for /f "usebackq tokens=*" ..a in ^(`FINDSTR /i /r /c:"CALSTARS......_...._.._.._.._.._.._../.zip" "!bat_ftpfile2!"`^) 


:FILTER_LIST_END
!bat_poplog! [RETURN !bat_batchfile!:FILTER_LIST] [!bat_returncode!]
!bat_log!
!bat_WAIT! 3	:FILTER_LIST_END
!bat_return! !bat_returncode!



	REM ########################################################################
	REM 
:UPLOAD_QUEUE
 !bat_log! [CALL !bat_batchfile!:UPLOAD_QUEUE] %*
  
   CALL :GET_QUEUE_ITEMS
   
   CALL :FILTER_LIST



:UPLOAD_QUEUE_END
!bat_poplog! [RETURN !bat_batchfile!:UPLOAD_QUEUE] [bat_returncode=!bat_returncode!]
!bat_log!
!bat_return! !bat_returncode!


	REM ########################################################################
	REM 
:ERROR [IN bat_returncode IN "error string"]
!bat_log! [CALL !bat_batchfile!:ERROR] %*

	set bat_returncode=%~1
	!bat_echolog!
	!bat_echolog!
	!bat_echolog! [!date! !time! !bat_batchfile!:SETUP_DIRS] ERROR [!%~1!] - %~2
	!bat_echolog!
	!bat_echolog!
	
	!bat_WAIT! 15


:ERROR_END
!bat_poplog! [RETURN !bat_batchfile!:ERROR] [bat_returncode=!bat_returncode!]
!bat_log!
!bat_return! !bat_returncode!





	REM #################################################################################
:HELP
	@echo off
	echo.
	echo.%~nx0
	echo.
	echo Usage: %~nx0 /capturedir "bat_capturedfilesdir" /caldir "!bat_caldir!" /detect "!bat_detectfile!" /session "!bat_capturedfilesdir!"
	echo.   
	echo.Performs the first phase of the AutoCal_Remote procedure by uploading the CALSTARS and associated
	echo.Cal files into a zip file and uploading it to a remote CAMS calibration server, which is 
	echo.USA-based, and performs the autocal function remotely and sends the results back. 
	echo.
	echo.The argument pairs can be placed in any order.
	echo.  arg1             /capturedir bat_capturedfilesdir
	echo.  arg2             /caldir bat_caldir
	echo.
	echo.QUEUE FORMAT:
	echo.   Queue Name: ".\CAMS\AutoCal_Remote_Queue\AutoCal_Remote_Queue.txt"
	echo.      The queue items will be entered as follows:
	echo.         CALSTARS??????_????????_??????.zip,"^<bat_capturedfilesdir^>","^<bat_caldir^>","^<command line^>""
	echo.
	echo.   The original source zip file that got uploaded will remain in the queue folder
	echo.   until after its results have been safely downloaded.  That way, if necessary, it
	echo.   can be resubmitted easier. 
	echo.
	echo.Steps involved:
	echo.PHASE ONE
	echo.  1.  When AutoCal.bat is run locally, if it's run using the /remote command line switch, it 
	echo.      behaves differently.  Instead of doing autocal, it performs the AutoCal_Remote.bat
	echo.      procedures.
	echo.  2.  Run FTP_CalStarExtractor.exe. 
	echo.  3.  Zip the CALSTARS*.txt files, along with the camerasites.txt file and the most recent CAL file 
	echo.      for each camera into a zip file. The Zip file will be stored in the ".\CAMS\AutoCal_Remote_Queue\" dir.
	echo.  4.  Move the zip file to the ".\CAMS\AutoCal_Remote_Queue\" folder.
	echo.  4.  Add item to the remote queue.  Include the zip file name, bat_capturedfilesdir, bat_caldir
	echo.  4.  Upload the zip file to the server [which triggers phase two on the remote server]
	echo.  5.  Wait for the server to create your results using the MonitorDir program locally, set to 
	echo.      invoke AutoCal_LocalDownload.bat when it wakes up. 
	echo.PHASE TWO on the remote server:
	echo.     A] Moves the zip file from the TestCal\From folder to the Zip folder
	echo.     B] Unzips the zip file to the TestCal folder
	echo.     C] Reconstitutes the FF files from the CALSTARS files
	echo.     D] Move the CAL, FF, camerasites, and log files to TestCal\CapturedFiles\{date time} folder
	echo.     E] Launch AutoCal.bat /remote.  This produces a CAL file for each camera into the 
	echo.        CapturedFiles dir as well as a BinFiles dir, containing the CAL files and their
	echo.        corresponding FF files.
	echo.     F] Package the results into a zip file and move it into the TestCal\From dir. 
	echo.        [which triggers phase three on the local system]
	echo.        If there are any errors during the AutoCal_Remote.bat script, the error is logged into
	echo.        the ERRORS.txt file and that ERRORS.txt file is placed into the ..._Results.zip file
	echo.        into the From dir, which triggers Phase Three.
	echo.     G] Clean up any leftover files
	echo.     H] Process any remaining zip files in the TestCal\From dir that may have been uploaded 
	echo.        either before MonitorDir was loaded or while the current session was being processed.
	echo.     I] Re-launch the MonitorDir program
	echo.PHASE THREE
	echo.    1]  The monitor triggers this script when a file appears in the dir [remember, it may not
	echo.        be a file that pertains to one of the local queue items]
	echo.    2]  This script first checks to see if the triggered item pertains to the local queue items.  
	echo.    3]  If it does not pertain to any of the queue items, then it should go back and relaunch
	echo.        MonitorDir to wait for more results
	echo.    4]  If it does pertain to at least one of the queue items, then it should do the following steps...
	echo.        A]  It must download the file
	echo.        B]  Remove the file from the server
	echo.        C]  Unzip the file to a temp dir
	echo.        D]  Move the contents to their appropriate areas [.\Cal, .\Cal\BinFiles, .\{capturedfilesdir}]
	echo.        E]  Remove the downloaded file and any temp files used to do the work
	echo.     *  F]  Run the script specified in the queue item to complete the autonomous processing. 
	echo.        G]  Loop back to step 2 above to process any remaining queue items. There can be a lot of 
	echo.            reasons why they didn't get processed before. 
	echo.
	echo.

exit /b 99




	REM #################################################################################
:DONE

	
	!bat_popecho! [goto !bat_batchfile!:DONE] [bat_returncode=!bat_returncode!]
	!bat_echolog! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	!bat_echolog!
	
	

exit /b !bat_returncode!
