  @echo off
setlocal enabledelayedexpansion enableextensions


:CHECKHELP

	echo.%*|find /i "/?">nul:
	if %errorlevel% equ 0 goto :HELP
	echo.%*|find /i "/h">nul:
	if %errorlevel% equ 0 goto :HELP
	echo.%*|find /i "/help">nul:
	if %errorlevel% equ 0 goto :HELP

		rem You must have exactly two arguments...
	if /i "%~2" equ "" (goto :HELP)


:SETUP
	call "%~dp0AutoCams.setup.bat" "%~dpnx0"
	!bat_CALL! "%~dp0indent_setup.bat"

    if not defined bat_LOCALIZE.format.date (
    	!bat_CALL! "!bat_exedir!\ReadParams.bat"
    )
 

	!bat_log!
	!bat_log! =============================================================
	!bat_log! [%date% %time%] - Version: !bat_vers! - %~dpnx0 [%*]
	!bat_log! [!bat_batchfile!:SETUP %*]  Please wait...
	!bat_pushlog!


	set bat_startlocation=!bat_exedir!&					rem C:\CAMS
	set bat_startdrive=%~d0&				rem C:
	SET bat_returncode=0
	
	if /i "!bat_startlocation:~-5!" neq "\CAMS" (
		!bat_echolog! '     ERROR: You must start from x:\CAMS directory. 
		exit /b 1
	)
	
	set bat_coincidencedir=%~1
	set bat_targetdir=!bat_coincidencedir!
	
	set bat_caldir=!bat_coincidencedir!

	!bat_CALL! "!bat_exedir!\ParseDir.bat" "!bat_coincidencedir!"
	set bat_basedir=!bat_retval1!
	
	
	set bat_nstars=%~2
	set bat_OminusC=%~3

goto :MAIN

	


:MAIN
!bat_pushlog! [GOTO !bat_batchfile!:MAIN %*]

	set bat_returncode=0


	call :RECONSTITUTE_CALS "!bat_coincidencedir!" "!bat_caldir!"

:MAIN_END
!bat_poplog! [END !bat_batchfile!:MAIN] [bat_returncode=!bat_returncode!]
!bat_log! :DONE
!bat_log!
goto :DONE





	rem =============================================================
	rem ====          RECONSTITUTE_CALS                          ====
	rem 
	rem Calls the FTP_CalStarsReconstitution.exe to create FF*.bin
	rem files from the CALSTARS*.txt files.
	rem
	rem Arguments:
	rem     bat_coincidencedir
	rem
	rem     bat_nstars
	rem
	rem     bat_OminusC
	rem
	rem Returns:
	rem     bat_returncode
	rem
	rem  3.0] If there are any CALSTARS*.txt files.  Note: Each CALSTARS file
	rem       has the information for reconstituting multiple FF*.bin files 
	rem       for a single camera.  They are generated using 
	rem       FTP_CalStarsExtraction.exe with 100 300, meaning that it skips 
	rem       every 100 files [which is about 14.2 minutes].  
	rem 
	rem       3.1]  Check if there is a bin file for each CALSTARS file. If there is
	rem             already, we can skip this step. Otherwise, if there is a single
	rem             CALSTARS file that's not represented by a bin file, then
	rem             we need to reconstitute.
	rem 
	rem       3.2]  Run FTP_CalStarsReconstitution.exe  "!bat_coincidencedir!"
	rem 
	rem       3.3]  For each CalFileList.log entry that doesn't exist in the bat_coincidencedir
	rem             scan for it in a previous directory and then copy it to the bat_coincidencedir.
	rem             If it still isn't found, either scan for it in the CAMS\Cal directory, or
	rem             simply add it to a CalFileList_missing.log file and let the user know about it. 
	rem             We have to do this before FTP_MeteorCal_AutoUpdate.exe because that program 
	rem             requires at least one previous cal file for each camera. 
	rem 
	rem       3.4]  Run FTP_MeteorCal_AutoUpdate.exe 50 1.6 "!bat_coincidencedir!" "!bat_coincidencedir!"
	rem 
	rem       3.5]  Run FTP_ApplyCal2detectinfo.exe "!bat_coincidencedir!"
	rem 
	rem       3.6]  Rebuild the CalFileList.log file based on the newly
	rem             modified detect file.
	rem 
	rem       3.7]  Delete the FF files that were not used.
	rem =============================================================
	
:RECONSTITUTE_CALS [IN] "!bat_coincidencedir!"
!bat_pushlog! [call !bat_batchfile!:RECONSTITUTE_CALS %~1 %~2 %~2]

	set bat_returncode=0
	
		rem 3.0 If there aren't any CALSTARS*.txt files, then exit the script
	if not exist "!bat_coincidencedir!\CALSTARS*.txt" (
		!bat_log! '    No "!bat_coincidencedir!\CALSTARS*.txt" found, so quitting %~nx0 script...
		!bat_logexec! dir "!bat_coincidencedir!\CALSTARS*.txt"
		goto :RECONSTITUTE_CALS_3_5
	)

	!bat_logexeclog! type "!bat_coincidencedir!\CalFileList.log"
	
		rem 3.1 Check if there is a bin file for each calstars file.  List the CALSTARS*.txt
		rem files.  Then for each one, determine the camera number.  Then, for 
		rem each camera, check if there is an FFccc_*.bin.
		
	set bat_need_reconstitution=false
	for /f "usebackq tokens=*" %%a in (`dir "!bat_coincidencedir!\CALSTARS*.txt" /a-d /b /on`) do (
		set bat_tempfile=%%a
		set bat_tempfile_full=!bat_concidencedir!\%%a
		set bat_camera_temp=!bat_tempfile:~9,3!
!bat_echolog! bat_tempfile_full=!bat_tempfile_full!
		
		if not exist "!bat_coincidencedir!\FF!bat_camera_temp!_*.bin" (
			set bat_need_reconstitution=true
		)
	)
	
		rem Mark all the files so that any changes will discoverable using the archived attribute
	!bat_logexeclog! set -s -h -r -a "!bat_coincidencedir!\*.*"

		rem 3.2 Reconstitute the FF*.bin files from the CALSTARS*.txt files
	!bat_echolog!
	!bat_echolog! Now reconstitute faux original ff*.bin files from the CALSTARS*.txt files...
	!bat_pushecho! start "''FTP_CalStarsReconstitution.exe'' ''!bat_coincidencedir!''" /w "!bat_exedir!\FTP_CalStarsReconstitution.exe" "!bat_coincidencedir!"
	               start "''FTP_CalStarsReconstitution.exe'' ''!bat_coincidencedir!''" /w "!bat_exedir!\FTP_CalStarsReconstitution.exe" "!bat_coincidencedir!"
		set bat_returncode=!errorlevel!
	!bat_popecho! bat_returncode=!bat_returncode!

	if !bat_returncode! neq 0 (
		if !bat_returncode! equ 1 (
			!bat_echolog! '    USER ABORT - Continuing...
			goto :RECONSTITUTE_CALS_3_2b
		)

		goto :RECONSTITUTE_CALS_END
	)
:RECONSTITUTE_CALS_3_2b
	

		rem 3.3 Iterate through the CalFileList.log and copy any missing cal files. 
	!bat_pushlog! Now we have a bunch of FF*.bin files.  So it's time to iterate through the 
		!bat_echolog! CalFileList and copy each file from a previous session...
		for /f "usebackq tokens=*" %%a in ("!bat_coincidencedir!\CalFileList.log") do (

			set bat_temp_calfile=
			rem Copy the cal that matches from anywhere located under the base dir
			for /f "usebackq tokens=*" %%b in (`dir "!bat_basedir!\%%a" /a-d /od /s/b`) do (
				set bat_temp_calfile=%%b
			)

			if defined bat_temp_calfile (
				if not exist "!bat_coincidencedir!\%%b" (
					!bat_logexeclog! copy /y "!bat_temp_calfile!" "!bat_coincidencedir!"
				)
			)
		)
	!bat_poplog!
	
		rem 3.4 Now we should be ready to run FTP_MeteorCal_AutoUpdate.exe. 
	!bat_pushecho! start "''!bat_exedir!\FTP_MeteorCal_AutoUpdate.exe'' -1 !bat_nstars! !bat_OminusC! ''!bat_coincidencedir!'' ''!bat_coincidencedir!''" /w "!bat_exedir!\FTP_MeteorCal_AutoUpdate.exe" !bat_nstars! !bat_OminusC! "!bat_coincidencedir!" "!bat_coincidencedir!"
	               start "''!bat_exedir!\FTP_MeteorCal_AutoUpdate.exe'' -1 !bat_nstars! !bat_OminusC! ''!bat_coincidencedir!'' ''!bat_coincidencedir!''" /w "!bat_exedir!\FTP_MeteorCal_AutoUpdate.exe" !bat_nstars! !bat_OminusC! "!bat_coincidencedir!" "!bat_coincidencedir!"
		set bat_returncode=!errorlevel!
	!bat_popecho!
	
:RECONSTITUTE_CALS_3_5
		rem 3.5 Now run Apply to apply the new cals to the detect
	!bat_pushecho! start  "''!bat_exedir!\FTP_ApplyCal2detectinfo.exe'' ''!bat_coincidencedir!\FTPdetectinfo_Coinc.txt''" /w "!bat_exedir!\FTP_ApplyCal2detectinfo.exe" "!bat_coincidencedir!\FTPdetectinfo_Coinc.txt"
	               start  "''!bat_exedir!\FTP_ApplyCal2detectinfo.exe'' ''!bat_coincidencedir!\FTPdetectinfo_Coinc.txt''" /w "!bat_exedir!\FTP_ApplyCal2detectinfo.exe" "!bat_coincidencedir!\FTPdetectinfo_Coinc.txt"
		set bat_returncode=!errorlevel!
	!bat_popecho!
	
		if /i "!bat_returncode!" neq "0" (
			!bat_echolog! '    	ERROR - !bat_batchfile!:RECONSTITUTE_CALS - ApplyCal2detectinfo failed [!bat_returncode!]
			goto :RECONSTITUTE_CALS_END
		)
		
		!bat_logexec! move /y "!bat_coincidencedir!\FTPdetectinfo_CoincC.txt" "!bat_coincidencedir!\FTPdetectinfo_Coinc.txt"


:RECONSTITUTE_CALS_3_6

		rem 3.6 Create the SortDetect_CameraList.txt file and the CalFileList.log files again...
	!bat_CALL! "!bat_exedir!\CreateCalFileList.bat" "!bat_coincidencedir!\FTPdetectinfo_CoincMP.txt" "!bat_coincidencedir!" "!bat_coincidencedir!"

	if exist "!bat_coincidencedir!\FF*.bin" (
		!bat_logexec! del /q "!bat_coincidencedir!\FF*.bin"
	)

	if exist "!bat_coincidencedir!\CALSTARS*.txt" (
		!bat_logexec! del /q "!bat_coincidencedir!\CALSTARS*.txt"
	)
	

:RECONSTITUTE_CALS_END
!bat_poplog! [return !bat_batchfile!:RECONSTITUTE_CALS] [bat_returncode=!bat_returncode!]
!bat_log!
!bat_return!














:HELP
	@echo off
	echo.
	echo %~nx0
	echo.
	echo.  Usage: %~nx0 "^<bat_coincidencedir^>" "^<bat_caldir^>"
	echo.
	echo.  
	echo. 
	echo. Calls the FTP_CalStarsReconstitution.exe to create FF*.bin
	echo. files from the CALSTARS*.txt files.
	echo.
	echo. Arguments:
	echo.     bat_coincidencedir
	echo.
	echo.     bat_nstars
	echo.
	echo.     bat_OminusC
	echo.
	echo. Returns:
	echo.     bat_returncode
	echo.
	echo.  3] If there are any CALSTARS*.txt files
	echo.     3.1] Run FTP_CalStarsReconstitution.exe 100 "!bat_coincidencedir!"
	echo.     3.2] For each CalFileList.log entry that doesn't exist in the bat_coincidencedir
	echo.          scan for it in a previous directory and then copy it to the bat_coincidencedir.
	echo.          If it still isn't found, either scan for it in the CAMS\Cal directory, or
	echo.          simply add it to a CalFileList_missing.log file and let the user know about it.
	echo.     3.3] Run FTP_MeteorCal_AutoUpdate.exe 50 1.6 "!bat_coincidencedir!" "!bat_coincidencedir!"
	echo.     3.4] Run FTP_ApplyCal2detectinfo.exe "!bat_coincidencedir!"
	echo. =============================================================
	echo.  
	echo.  
	echo.   Example:
	echo.         call "%~0" "C:\CAMS\Temp\Coincidence\2011_12_14" 50 2.5
	echo.
exit /b -99

	
:ABORT
	!bat_echolog! '    ABORTING
	!bat_echolog! '    bat_returncode=!bat_returncode!
	pause
exit /b !bat_returncode!

:DONE
	!bat_poplog! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	!bat_log! [%date% %time%] [!bat_batchfile!:DONE]  Done with %0 %*
	!bat_log!
exit /b !bat_returncode!
