@echo off
setlocal enableextensions enabledelayedexpansion

	REM There are two main sections:  SETUP and MAIN.
	REM 
	REM SETUP is responsible for getting the command line variables, reading the
	REM INI files, initializing the logging system, etc.  Before using GOTO :MAIN,
	REM most of the variables used throughout the script should be set.  
	REM
	REM The MAIN routine lays out the other sections of code by calling the routines
	REM in the proper order. 
	REM 

:CHECKHELP

	(echo.%*|findstr /L /i /c:"/help" /c:"/h" /c:"/?" >nul: 2>&1) && (goto :HELP)
    if "%~3" equ "" goto :HELP
	echo.
	goto :SETUP %*

:CHECKHELP_END

	
:SETUP

	TITLE %~0 %*
	echo Waiting to unlock the log files...
	!bat_WAIT! 6


		Rem Blank out bat_logdir so that it is reset by the AutoCams.setup.bat script
	set bat_logdir=

	call "%~dp0AutoCams.setup.bat" "%~dpnx0"
	!bat_CALL! "%~dp0indent_setup.bat"
	set bat_pushstring=

		rem There are three ways that this script will be call that will effect how the
		rem log files should be created.  
		rem  1]  Launched from Task Scheduler.  In this case, the logs should be created in
		rem      ...\CAMS\Logs. 
		rem  2]  Launched from AutoCams option 43.  In that case, the logs should probably 
		rem      start over in ...\CAMS\Logs dir.  For example, let's say that you've been
		rem      processing a session from 2016_01_11_00_00_00.  The logs should be already
		rem      directed to the ...\CAMS\Logs\2016_01_11_00_00_00 dir.  In that case, when
		rem      you run option 43, the logs should start over in ...\CAMS\Logs.
		rem  3]  Launched from a command line.  In that case, the logs should start the
		rem      same as the first method ...\CAMS\Logs.  
	!bat_logexeclog! set bat_logdir=!bat_exedir!\Logs


	set bat_current_date=!date!
	
	!bat_echolog!
	!bat_echolog! ======================================================
	!bat_echolog! [!bat_current_date! !time!] %0 %*
	!bat_echolog!
	!bat_push!
	
	!bat_CALL! "!bat_exedir!\ReadParams.bat"

   	set bat_tempfile=%~0.junk.txt
	set bat_warningfile=%~0.warn.txt
	set bat_sunsettable=Script Steve v1.00
	

	set bat_CD_no=0
	set bat_CD_copy=1
	set bat_CD_move=2
	set bat_CD_flag=!bat_CD_move!


	Rem call :READPARAMS
	if defined bat_CAPTURE.archive_flag (
		set bat_CD_flag=!bat_CAPTURE.archive_flag!
	)

	call :GETARGS %*
	call :SETCAPTUREEXE %*
	
	call :TARGETLOCATION %*

	call :PARSEDATE %*

		rem The GETSCHEDSTART routine replaces the old way of reading a twilight file, which is no longer used.
	call :GETSCHEDSTART %*

	rem call :GETWALLTIMEROFF %*
	call :PARSETIME %*
	call :NOTIFY %*
	rem call :ADJONDATE %*
	
		rem We still need to call DAYOFWEEK to calculate the weekday.  Problem is, it returns a value
		rem that is not localized.  Maybe figure out a way to handle this usig bat_LOCALIZE...?
		rem Or maybe there is something in the %windir%\system32\wbem directory for this?
	call :DAYOFWEEK %*
	
		rem The "searchstr" is the date used to search the twilight file.  Since this is no longer used
		rem it is probably obsolete
	rem call :GETSEARCHDIR %*
	
	rem call :GETSUNANGLE %*
	call :GETSTARTTIME %*

       

        FOR /F "tokens=*" %%G IN ('dir ^"%windir%\system32\wbem\csv.xsl^" /b /s ') DO set csvdir= %%G
        set csvdir=%csvdir:~1%


:SETUP_END
GOTO :MAIN


:MAIN
!bat_log! [GOTO !bat_batchfile!:MAIN] %*
	call :CALCDURATION "!bat_ON!" "!bat_OFF!"

	!bat_CALL! "!bat_exedir!\CheckTwilight.bat" "!bat_twilight_searchstr!"
	
	call :REVIEW %*
	
	call :WAITDELAY %*

		Rem NEED TO RECALCULATE THE DURATION BASED ON THE NEW TIME
		Rem JUST IN CASE THE WAIT DELAY WAS ABORTED EARLY OR
		Rem IN CASE THERE WAS SOME TIME LOSS DURING THE WAIT. 
		Rem This ensures that the bat_OFF time is always
		Rem honored.

	!bat_CALL! "!bat_exedir!\GETONTIME1.bat"

	rem call :CALCDURATION "!bat_ON!" "!bat_OFF!"
	rem call :GETSCHEDULEDSTART %*
	
	set bat_CAPTURE.duration=!bat_duration!
	set bat_duration=!bat_CAPTURE.sunangle_dusk!
	
	call :CAPTURE %*

	call :VARIABLES_LOG bat_log
	!bat_log! %~dpnx0 %*
	!bat_log! [!date! !time!]


	if defined bat_warningfile (
		!bat_logexeclog! DEL /Q "!bat_warningfile!"
	) else (
		!bat_echolog! `    ERROR [!bat_batchfile!:MAIN] Dangerous command:  DEL /Q "!bat_warningfile!"
	)

	
	if defined bat_tempfile (
		!bat_logexeclog! DEL /Q "!bat_tempfile!"
	) else (
		!bat_echolog! `    ERROR [!bat_batchfile!:MAIN] Dangerous command: DEL /Q "!bat_tempfile!"
	)
	
:MAIN_END
!bat_poplog! [RETURN !bat_batchfile!:MAIN] []
!bat_log!

goto :DONE




	rem =============================================================
	rem                          GETARGS
	rem 
	rem Parses out the command line arguments.  Remember, LaunchCapture
	rem is called like this: 
	rem    LaunchCapture.bat   998   0.02   "d:\"
	rem
	rem Returns:
	rem    bat_camera              is set to the first argument
	rem
	rem    bat_delaytemp           is set to the dummy duration [I don't think 
	rem                            is used anymore.  It's just in there for 
	rem                            backward compatibility.
	rem
	rem    bat_targetlocation      is the root of a CAMS setup [without the CAMS part]
	rem
	rem    bat_autonomous          optional 4th arg indicates whether it should run 
	rem                            as unattended and avoid any kind of prompts.
	rem =============================================================
	
:GETARGS
!bat_pushlog! [CALL !bat_batchfile!:GETARGS] %*
	set bat_camera=%~1
	set bat_delaytemp=%~2
	
		rem When we set directories, in case the user has added the trailing "\",
		rem we use a batch file trick to append it with two colons [::] so that
		rem if the backslash is present, we can easily replace a "\::" with nothing
		rem to remove the backslash without messing with the other backslashes within
		rem a normal path string. 
		rem 
		rem In case they haven't provided the trailing backslash, this technique 
		rem then uses a second statement to remove all double colons.
	set bat_targetlocation=%~dpnx3::
	set bat_targetlocation=!bat_targetlocation:\::=!
	set bat_targetlocation=!bat_targetlocation:::=!
	
	set bat_autonomous=
	if /i "%~4" equ "autonomous" (
		set bat_autonomous=autonomous
	)
	if /i "%~4" equ "auto" (
		set bat_autonomous=autonomous
	)

:GETARGS_END
!bat_poplog! [RETURN !bat_batchfile!:GETARGS] [!bat_camera!, !bat_delaytemp!, !bat_targetlocation!, !bat_autonomous!]
!bat_log!
!bat_return!





	rem =============================================================
	rem                          SETCAPTUREEXE
	rem 
	rem Gets the capture program name from the autocams.params.ini file
	rem so that if special instances are required for tracking, the
	rem capture programs can be copied to their renamed versions and
	rem capture is done through that program and script.
	rem
	rem Returns:
	rem    nothing
	rem
	rem This script is obsolete and is now handled by the autocams.params.ini
	rem file settings in the [CAPTURE] section.
	rem 
	rem =============================================================

:SETCAPTUREEXE
!bat_pushlog! [CALL !bat_batchfile!:SETCAPTUREEXE] %*
	Rem [in]	
	Rem [out]	bat_captureexesrc
	Rem [out]	bat_captureexe
	Rem	[out[	bat_capturebat
	Rem [out]	bat_CD_flag

	Rem These settings are now handled by the following parameters in the
	Rem AutoCams.Params.ini file:
	Rem  - capture_mode=capture, captureanddetect, or capturetwoanddetect
	Rem  - bat_capturebat=FTP_Capture.bat, CaptureAndDetect.bat, or CaptureTwoAndDetect.bat
	Rem  - bat_captureexe=FTP_Capture.exe, FTP_CaptureAndDetect.exe, or FTP_CaptureTwoAndDetect.exe
	Rem  - bat_captureexesrc=FTP_Capture.exe, FTP_CaptureAndDetect.exe, or FTP_CaptureTwoAndDetect.exe
	Rem  - archive_flag=-1, 0, 1, or 2
	Rem  - cameralist=213, or 213 216 266 267

%bat_COMMENT_BEGIN%
	if /i "!bat_CAPTURE.capture_mode!" equ "capture" (
		set bat_CD_flag=
		goto :SETCAPTUREEXE_END
	)
%bat_COMMENT_END%

:SETCAPTUREEXE_END
		!bat_echolog! ` Capture mode in AutoCams.Params.ini is:
		!bat_echolog! `     "!bat_CAPTURE.capture_mode!" 
		!bat_echolog! `     so using:
		!bat_echolog! `       archive_flag=!bat_CAPTURE.archive_flag!
		!bat_echolog! `       cameralist=!bat_CAPTURE.cameralist!
		!bat_echolog! `       !bat_capturebat!
		!bat_echolog! `       !bat_captureexe!
		!bat_echolog! `       !bat_captureexesrc!
!bat_poplog! [RETURN !bat_batchfile!:SETCAPTUREEXE] [src=!bat_captureexesrc!, dest=!bat_captureexe!]
!bat_echolog!
!bat_return!





	
	rem =============================================================
	rem                          TARGETLOCATION
	rem 
	rem After the GETARGS routine runs, the bat_targetdrive and 
	rem bat_targetlocation variables should be set [as they were
	rem effectively passed in the 3rd argument on the command line].
	rem This script ensures that the last part of the bat_targetlocation
	rem variable is not "\CAMS". 
	rem
	rem This could cause problems for people who install their cams systems
	rem in a directory like this:  C:\astro\CAMS\CAMS
	rem
	rem This routine would only strip off the last "\CAMS".
	rem
	rem Returns:
	rem    bat_targetdrive        Drive letter where CAMS is located
	rem
	rem    bat_targetlocation     Drive and path to the CAMS directory
	rem                           without the "\CAMS" part.
	rem
	rem =============================================================

:TARGETLOCATION
!bat_pushlog! [CALL !bat_batchfile!:TARGETLOCATION] %*
	Rem Ensures that if the last name of the specified directory
	Rem contains "CAMS" or "CAMS\", then it removes that from
	Rem the bat_targetlocation variable. 
	if /i "!bat_targetlocation:~-5!" equ "\CAMS" (
		set bat_targetlocation=!bat_targetlocation:~0,-5!
	) else (
		if /i "!bat_targetlocation:~-6!" equ "\CAMS\" (
			set bat_targetlocation=!bat_targetlocation:~0,-6!
		)
	)
	
	!bat_logexeclog! set bat_targetlocation=!bat_camsdir!
	
	set bat_targetdrive=!bat_targetlocation:~0,2!

:TARGETLOCATION_END
!bat_poplog! [RETURN !bat_batchfile!:TARGETLOCATION] [!bat_targetdrive!, !bat_targetlocation!]
!bat_log!
!bat_return!




	rem =============================================================
	rem                          PARSEDATE
	rem 
	rem Calls the PARSEDATE1.bat script to parse out the current date
	rem independent of locale
	rem
	rem Returns:
	rem   environment variables:
	rem 	ftp_wmdy                 Source value, e.g. %%date%% or wmic localdatetime
	rem 
	rem 	ftp_year                 The 4 digit year with leading zeros
	rem 
	rem 	ftp_mo                   The two digit month with leading zeros
	rem 
	rem 	ftp_da                   Two digit day with leading zeros
	rem 
	rem 	ftp_mo2                  Numeric month value [no leading zero]
	rem 
	rem 	ftp_da2                  Numeric day value [no leading zero]
	rem 
	rem 	bat_twilight_searchstr   e.g. 12/25/2014
	rem                              I believe this var is no longer used.
	rem =============================================================

:PARSEDATE
!bat_pushlog! [CALL !bat_batchfile!:PARSEDATE] %*

		rem ParseDate1.bat uses ENDLOCAL to pass values back. 
	!bat_CALL! "!bat_exedir!\PARSEDATE1.bat"

:PARSEDATE_END
!bat_poplog! [RETURN !bat_batchfile!:PARSEDATE] [!ftp_wmdy!, !ftp_year!, !ftp_mo!, !ftp_da!]
!bat_log!
!bat_return!




	rem =============================================================
	rem                          GETSCHEDSTART
	rem 
	rem Calls the "dusk.vbs" script and parses out the output into
	rem variables used for calculating duration, delay, etc. based
	rem on the calculated astro dusk/dawn, given the sun angle
	rem specified in the autocams.params.ini file. 
	rem
	rem Lat/Lon for the calculation are determined by locating the
	rem first camera in the bat_CAPTURE.cameralist autocams.params.ini
	rem variable and then locating that camera in the 
	rem "!bat_startlocation!\CAMS\Cal\CameraSites.txt" file.  The 
	rem Lat/Lon is defined there. 
	rem
	rem Returns:
	rem   environment variables:
	rem 	bat_on                   12/7/2014 1:27:34 AM
	rem 	bat_onUT                 12/7/2014 1:27:34 AM
	rem 	bat_duskutc              12/7/2014 1:27:34 AM
	rem 	bat_calcedon             12/7/2014 1:27:34 AM
	rem 	bat_dusklocal            12/6/2014 5:27:34 PM
	rem 	bat_offset1              -8
	rem 	bat_off                  12/7/2014 2:29:12 PM
	rem 	bat_offUT                12/7/2014 2:29:12 PM
	rem 	bat_dawnutc              12/7/2014 2:29:12 PM
	rem 	bat_offtime              12/7/2014 6:29:12 AM
	rem 	bat_offLocal             12/7/2014 6:29:12 AM
	rem 	bat_offset2              -8
	rem 	bat_duration             8.45
	rem 	bat_durtotalsecs         30420
	rem 	ftp_hhmmssdd             12/6/2014 10:02:07 PM
	rem 	bat_delaycalc            0
	rem 	bat_scheduledstarttime   17:00:00
	rem 	bat_walltimeroff         8:00:00
	rem 	bat_camera_temp          213
	rem =============================================================

:GETSCHEDSTART
!bat_pushlog! [CALL !bat_batchfile!:GETSCHEDSTART] %*

	set bat_scheduledstarttime=Error

	!bat_log! Calling "dusk.vbs" to get the twilight dusk/dawn and camera on/off info.
	!bat_log! for /f "delims=" ..a in ^('cscript //nologo dusk.vbs %ftp_year% %ftp_mo% %ftp_da%'^) do ^(
	          for /f "delims=" %%a in  ('cscript //nologo dusk.vbs %ftp_year% %ftp_mo% %ftp_da%') do (
		!bat_logexec! set bat_VBTemp=%%a
	)

	for /f "tokens=1-16 delims=;" %%a in ("%bat_VBTemp%") do (
                
                set bat_onUT=%%a
		set bat_on=%%a
		set bat_duskutc=%%a
		set bat_calcedon=%%a
		
		set bat_dusklocal=%%b
		
		set bat_offset1=%%c
		
		set bat_off=%%d
		set bat_offUT=%%d
		set bat_dawnutc=%%d
		
		set bat_offtime=%%e
		set bat_offLocal=%%e
		
		set bat_offset2=%%f
		
		set bat_duration=%%g
		set bat_dur=%%g
		
		Set /a bat_durtotalsecs=%%h
		
                set bat_UTcurrent=%%i

		set ftp_hhmmssdd=%%j
		
		set bat_delaycalc=%%k
		
		set bat_walltimeron=%%l
		set bat_scheduledstarttime=%%l
		
		set bat_walltimeroff=%%m
		
		set bat_camera_temp=%%n
		
		set bat_lat=%%o
		
		set bat_lon=%%p
	)
	
	
	!bat_pushlog! dusk.vbs calculated the following values:
		!bat_log!   bat_on:					!bat_on!
		!bat_log!   bat_onUT:				!bat_onUT!
		!bat_log!   bat_duskutc:				!bat_duskutc!
		!bat_log!   bat_calcedon:			!bat_calcedon!
		!bat_log!
		!bat_log!   bat_dusklocal:			!bat_dusklocal!
		!bat_log!
		!bat_log!   bat_offset1:				!bat_offset1!
		!bat_log!
		!bat_log!   bat_off:					!bat_off!
		!bat_log!   bat_offUT:				!bat_offUT!
		!bat_log!   bat_dawnutc:				!bat_dawnutc!
		!bat_log!
		!bat_log!   bat_offtime:				!bat_offtime!
		!bat_log!   bat_offLocal:			!bat_offLocal!
		!bat_log!
		!bat_log!   bat_offset2:				!bat_offset2!
		!bat_log!
		!bat_log!   bat_duration:			!bat_duration!
		!bat_log!
		!bat_log!   bat_durtotalsecs:		!bat_durtotalsecs!
		!bat_log!
		!bat_log!   bat_UTcurrent:			!bat_UTcurrent!
		!bat_log!
		!bat_log!   ftp_hhmmssdd:			!ftp_hhmmssdd!
		!bat_log!
		!bat_log!   bat_delaycalc:			!bat_delaycalc!
		!bat_log!
		!bat_log!   bat_walltimeron:		!bat_walltimeron!
		!bat_log!   bat_scheduledstarttime	!bat_scheduledstarttime!
		!bat_log!
		!bat_log!   bat_walltimeroff:		!bat_walltimeroff!
		!bat_log!
		!bat_log!	bat_camera_temp:			!bat_camera_temp!
		!bat_log!
		!bat_log!   bat_lat:					!bat_lat!
		!bat_log!
		!bat_log!   bat_lon:					!bat_lon!
	!bat_pop!


:GETSCHEDSTART_END
!bat_poplog! [RETURN !bat_batchfile!:GETSCHEDSTART] [Wall timer ON=!bat_scheduledstarttime!]
!bat_log!
	set bat_VBTemp=
!bat_return!


	rem =============================================================
	rem                          GETWALLTIMEROFF
	rem 
	rem Obsolete with the new dusk.vbs script

:GETWALLTIMEROFF
!bat_pushlog! [CALL !bat_batchfile!:GETWALLTIMEROFF] %*
	
	!bat_echolog! for /f "usebackq tokens=1-3* skip=2 delims=!bat_LOCALIZE.delimiter.csv!" ..a in ^(`find /i "Wall Timer OFF" "!bat_sunsettable!"`^) do ...

:GETWALLTIMEROFF_END
!bat_poplog! [RETURN !bat_batchfile!:GETWALLTIMEROFF] [Wall timer OFF=!bat_walltimeroff!]
!bat_log!
!bat_return!



	rem =============================================================
	rem                         PARSETIME
	rem 
	rem Uses the DOS TIME env var and parses it out according the the
	rem current local into its hr, min, sec, parts.  These parts can
	rem be used elsewhere in the script. 
	rem 
	rem Returns:
	rem   When run at 10:57:13.52 PM on 10/05/2013, it should result in:
	rem    ftp_hhmmssdd         22:57:13.52
	rem    ftp_hhmmssdd_local   10:57:13.52 PM
	rem    ftp_hr               22
	rem    ftp_mi               57
	rem    ftp_se               13
	rem    ftp_sed              52
    rem    ftp_jd               2456413    Julian Date
	rem =============================================================
	
:PARSETIME
!bat_pushlog! [CALL !bat_batchfile!:PARSETIME] %*

	!bat_logexeclog! set /a bat_midnightsecs=86400
	!bat_CALL! "!bat_exedir!\PARSETIME1.bat"

:PARSETIME_END
!bat_popecho! [RETURN !bat_batchfile!:PARSETIME] [!ftp_hhmmssdd!, !ftp_hr!, !ftp_mi!, !ftp_se!, !ftp_jd!]
!bat_log!
!bat_return!




:NOTIFY
	echo.
	echo.     This script is designed to be launched at the same time of the day
	echo.     all year ^(!bat_scheduledstarttime!^).  Then it waits a different 
	echo.     number of seconds before starting capture, based on the calculation 
	echo.     of dusk, which is based on your LAT/LONG. Then it launches 
	echo.     !bat_captureexe! with the specified camera number, the calculated 
	echo.     duration [dusk to dawn from the calculations performed by the 
	echo.     "dusk.vbs" script], and the specified storage location. 
	echo.
	echo.     If you are running this manually, check the "autocams.params.ini" 
	echo.     file for the [CAPTURE] sunangle_dusk and sunangle_dawn values and 
	echo.     set the wall timer according to the indicated Wall Timer ON and
	echo.     Wall Timer OFF times. CheckTwilight.bat is provided for this purpose. 
	echo.

:NOTIFY_END
!bat_return!



	
	rem =============================================================
	rem                         ADJONDATE
	rem 
	rem Adjust ON Date.  This routine 
	rem 
	rem Returns:
	rem     bat_on=!bat_on!, ftp_hhmmssdd=!ftp_hhmmssdd!, bat_calcedon=!bat_calcedon!, bat_off=!bat_off!, !ftp_year!, !ftp_mo2!, !ftp_da2!
	rem 
	rem   When run at 10:57:13.52 PM on 10/05/2013, it should result in:
	rem    ftp_hhmmssdd         22:57:13.52
	rem    ftp_hhmmssdd_local   10:57:13.52 PM
	rem    ftp_hr               22
	rem    ftp_mi               57
	rem    ftp_se               13
	rem    ftp_sed              52
    rem    ftp_jd               2456413    Julian Date
    rem
    rem Note: This routine no longer uses the twilight file.
	rem =============================================================
	
:ADJONDATE
!bat_pushlog! [CALL !bat_batchfile!:ADJONDATE] %*

	rem normalize to leading zero for the hours digits
	!bat_logexeclog! set bat_temp=0!bat_off!
	!bat_logexeclog! set bat_temp=!bat_temp:~-8!

		rem If the the current time UTC is less than the OFF time UTC, then 
		rem we need to calculate delay and stuff.  Otherwise, we have 
		rem start immediately.
	if "!ftp_hhmmssdd:~0,8!" lss "!bat_temp!" (
		!bat_push!
			!bat_log! ... "!ftp_hhmmssdd:~0,8!" lss "!bat_off!"...

			!bat_logexeclog! set bat_on=!ftp_hhmmssdd:~0,8!
			!bat_logexeclog! set /a bat_midnightsecs=0
			!bat_log! set /a ftp_jd-=1
			!bat_log! set /a ftp_jd-=1 = !ftp_jd!

			call :jdate2date !ftp_jd! ftp_year ftp_mo2 ftp_da2
			!bat_logexeclog! set ftp_wmdy=dow !ftp_mo2!/!ftp_da2!/!ftp_year!

			set ftp_wmdy=!bat_current_date!
			set bat_formatdate=!bat_LOCALIZE.format.date!

			if not defined bat_LOCALIZE.format.date (
				!bat_echolog!
				!bat_echolog! !bat_batchfile!:ADJONDATE    WARNING - Missing bat_LOCALIZE.date.format from autocams.params.ini.  Using the default date format of "www yyyy/mm/dd"
				!bat_echolog!
				set bat_LOCALIZE.format.date=www yyyy/mm/dd
			)

			if /i "!bat_LOCALIZE.format.date:~2,1!" equ "/" (
					if /i "!ftp_wmdy:~2,1%!" NEQ "/" (
							 set ftp_wmdy=0!ftp_wmdy!
					) 
				)

			if /i "!bat_LOCALIZE.format.date:~5,1!" equ "/" (
				if /i "!ftp_wmdy:~5,1!" NEQ "/" (
						set ftp_wmdy=!ftp_wmdy:~0,3!0!ftp_wmdy:~3,6!
				)
			)

			!bat_log! bat_current_date=!bat_current_date!

			for /L %%a in (0,1,14) do (
				if /i "!bat_LOCALIZE.format.date:~%%a,2!" == "dd" (!bat_logexeclog!   set ftp_da=!bat_current_date:~%%a,2!)
				if /i "!bat_LOCALIZE.format.date:~%%a,2!" == "mm" (!bat_logexeclog!   set ftp_mo=!bat_current_date:~%%a,2!)
				if /i "!bat_LOCALIZE.format.date:~%%a,4!" == "yyyy" (!bat_logexeclog! set ftp_year=!bat_current_date:~%%a,4!)
				if /i "!bat_LOCALIZE.format.date:~%%a,3!" == "www" (!bat_logexeclog!  set ftp__ww=!bat_current_date:~%%a,3!)
			)


			rem Remove the leading zero from the numeric month [ftp_mo2] to keep it from being treated as an octal.
			!bat_logexeclog! set ftp_mo2=!ftp_mo!
			if !ftp_mo! LSS 10 (
				!bat_logexeclog! set ftp_mo2=!ftp_mo:~-1!
			)


			rem Remove the leading zero from the numeric day [ftp_da2] to keep it from being treated as an octal.
			!bat_logexeclog! set ftp_da=!ftp_da!
			!bat_logexeclog! set ftp_da2=!ftp_da!
			if !ftp_da! LSS 10 (
				!bat_logexeclog! set ftp_da2=!ftp_da:~-1!
			)
		!bat_pop!
	) else (
		!bat_log!  ELSE "!ftp_hhmmssdd:~0,8!" GEQ "!bat_temp!"...
	)
	
	

	call :jdate2date %ftp_jd% ftp_year ftp_mo2 ftp_da2
	call :CALCDELAY "!bat_on!" "!ftp_hhmmssdd!"



:ADJONDATE_END
!bat_poplog! [RETURN !bat_batchfile!:ADJONDATE] [bat_on=!bat_on!, ftp_hhmmssdd=!ftp_hhmmssdd!, bat_calcedon=!bat_calcedon!, bat_off=!bat_off!, !ftp_year!, !ftp_mo2!, !ftp_da2!]
!bat_log!
!bat_return!





	


	rem =============================================================
	rem                         DAYOFWEEK
	rem 
	rem Get's the name of the weekday as well as calculates the
	rem day number in the year.
	rem 
	rem Get the day of the week and the day number in the year of the 
	rem current date by getting the julian value of the current date, 
	rem then getting the julian value of Jan 1 of that same year and
	rem then subtracting Jan 1 from the current julian dates. 
	rem 
	rem Returns:
	rem     bat_day_number
	rem     bat_weekday
	rem     bat_JD
	rem     bat_JD_jan1
	rem 
	rem =============================================================
	
:DAYOFWEEK
!bat_pushlog! [CALL !bat_batchfile!:DAYOFWEEK] %*

	rem Jan 01, YYYY so we can subtract its julian date from current julian date to get the day number of the year.
	call :date2jdate bat_JD_jan1 !ftp_year! 01 01
	call :date2jdate bat_JD !ftp_year! !ftp_mo2! !ftp_da2!
	call :jdate2date !bat_JD! bat_yyyy bat_mm bat_dd
	
	set /a bat_day_number=bat_JD - bat_JD_jan1 + 1
	
	set /a bat_wdayn=((!bat_JD!*100)/7)-(!bat_JD!/7*100)
	if !bat_wdayn! equ 85 (
		set bat_weekday=Sunday
	) else (
		if !bat_wdayn! equ 0 (
			set bat_weekday=Monday
		) else (
			if !bat_wdayn! equ 14 (
				set bat_weekday=Tuesday
			) else (
				if !bat_wdayn! equ 28 (
					set bat_weekday=Wednesday
				) else (
					if !bat_wdayn! equ 42 (
						set bat_weekday=Thursday
					) else (
						if !bat_wdayn! equ 57 (
							set bat_weekday=Friday
						) else (
							if !bat_wdayn! equ 71 (
								set bat_weekday=Saturday
							)
						)
					)
				)
			)
		)
	)

:DAYOFWEEK_END
!bat_poplog! [RETURN !bat_batchfile!:DAYOFWEEK] [!bat_weekday!]
!bat_log!
!bat_return!




:GETSEARCHDIR
!bat_pushlog! [CALL !bat_batchfile!:GETSEARCHDIR] %*
	set ftp_searchdir=%ftp_year%_%ftp_mo2%_%ftp_da2%
:GETSEARCHDIR_END
!bat_poplog! [RETURN !bat_batchfile!:GETSEARCHDIR] [!ftp_searchdir!]
!bat_log!
!bat_return!


	

:GETSUNANGLE
!bat_pushlog! [CALL !bat_batchfile!:GETSUNANGLE] %*

	set bat_sunangleoff=!bat_CAPTURE.sunangle_dawn!
	set bat_sunangleon=!bat_CAPTURE.sunangle_dusk!
		

GETSUNANGLE_END
!bat_poplog! [RETURN !bat_batchfile!:GETSUNANGLE] [on=!bat_sunangleon!, off=!bat_sunangleoff!]
!bat_log!
!bat_return!



	Rem Get's the start time from the twilight file
	rem
	rem This routine is obsolete
:GETSTARTTIME
!bat_pushlog! [CALL !bat_batchfile!:GETSTARTTIME] %*

 
:GETSTARTTIME_END
!bat_poplog! [RETURN !bat_batchfile!:GETSTARTTIME] [bat_duration=!bat_duration!
!bat_log! `                                      bat_delay=!bat_delay!,
!bat_log! `                                      bat_date=!bat_date!,
!bat_log! `                                      bat_off=!bat_off!,
!bat_log! `                                      bat_dst=!bat_dst!,
!bat_log! `                                      bat_duskutc=!bat_duskutc!,
!bat_log! `                                      bat_dusklocal=!bat_dusklocal!,
!bat_log! `                                      bat_dawnutc=!bat_dawnutc!]
!bat_log!
!bat_return!


	rem =============================================================
	rem                    REVIEW 
	rem 
	rem Need to change this to a call to CheckTwilight for the given 
	rem date [and possibly time].
	rem 
	rem CheckTwilight script should return just the essentials, for 
	rem launching the capture scripts, such as:
	rem 
	rem    bat_camera             - first camera in the list of cameras.
	rem                             For a two or four camera system,
	rem                             there also ought to be a way
	rem                             to determine the cameralist.
	rem 
	rem    bat_duration           - decimal number of hours for capture
	rem                             or a negative number to use the 
	rem                             sun angle method.
	rem 
	rem    bat_targetlocation
	rem 
	rem    bat_delaycalc          - Number of seconds to wait before 
	rem                             launching the capture script.
	rem 
	rem 
	rem 
	rem =============================================================

:REVIEW
!bat_pushlog! [CALL !bat_batchfile!:REVIEW] %*

	!bat_pushecho! find /i "Delay will be from the earliest" "Steve Script V1.00"
		!bat_echolog!
		!bat_echolog! Task Scheduler Start Time=!bat_scheduledstarttime! ^(Wall timer ON local time^)
		!bat_echolog! Wall Timer OFF=!bat_walltimeroff! ^(local time^)
		!bat_echolog!

		!bat_push!
				rem call :CALCDAYNUMBER !bat_twilight_searchstr!
		
		
				!bat_echolog! Date=                !bat_twilight_searchstr!  - Date of these calculations ^(!bat_weekday!^)
				!bat_echolog! Day number=          !bat_day_number!         - Day of the year 
				!bat_echolog! Task Sched Start     !bat_scheduledstarttime!    - Local time Windows Task Scheduler is supposed to launch this script
				!bat_echolog! Current Time         !ftp_hhmmssdd!    - Current local time
				!bat_echolog!
				!bat_echolog! Camera ON Local      !bat_dusklocal! (UT!bat_offset1!)- Local time of the first frames. 
				!bat_echolog! Camera ON UTC        !bat_onUT!     - UTC time ^(camera time^) of the first frame
				!bat_echolog! Camera ON sun angle  !bat_CAPTURE.sunangle_dusk! - degrees below horizon
				!bat_echolog!
				!bat_echolog! Camera OFF Local=    !bat_offLocal! (UT!bat_offset2!)    - Local time capture should end in the morning
				!bat_echolog! Camera OFF UTC       !bat_offUT!    - UTC time ^(camera time^) of the last frame
				!bat_echolog! Camera OFF sun angle !bat_CAPTURE.sunangle_dawn! - degrees below horizon
				!bat_echolog!
				!bat_echolog! Duration=            !bat_duration!      - Number of hours to capture ^(24:00:00 - ON + OFF^)
				!bat_echolog! Actual Delay=        !bat_delaycalc! sec      - Number of seconds to delay from the start time ^(either 
				!bat_echolog!                                               !bat_scheduledstarttime!^)
				!bat_echolog! `                               Note: Dawn is calculated for the next morning

				!bat_CALL! "!bat_exedir!\CheckFreeSpace.bat"

				if "!bat_SIMULATION.capture!" neq "true" (
					!bat_echolog!
					!bat_echolog! Disk Space Available !bat_freeMB! MB
					!bat_echolog! Disk Space Required= !bat_reqdisk! MB     - !bat_diskspacememo!
					!bat_echolog!
						if !bat_reqdisk! gtr !bat_freeMB! (
							!bat_logexec! call :GETWINDOWSVER

							set /a bat_sizediff=!bat_reqdisk!-!bat_freeMB!

							ECHO ********* WARNING! ************>!bat_warningfile!
							echo    [!date!  !time!] >>!bat_warningfile!
							echo    You only have !bat_freeMB! MB free on drive !bat_targetdrive!. Within the >>!bat_warningfile!
							echo    next !bat_AUTOCAMS.diskspacewarning.days! days, you will require !bat_reqdisk! MB.  You will need to free up>>!bat_warningfile!
							echo    !bat_sizediff! MB ASAP or the capture will fail.>>!bat_warningfile!
							echo.>>!bat_warningfile!
							echo    Tips ^(for Windows version !bat_winver!^):>>!bat_warningfile!
							echo      a^) Use Autocams option 61 to archive old months SubmissionFiles>>!bat_warningfile!
							echo      b^) Try emptying the recycle bin >>!bat_warningfile!
							if "bat_winver" lss "6.0" (
								echo           ^(rd /s !bat_targetdrive!\recycler^)>>!bat_warningfile!
							) else (
								echo           ^(rd /s !bat_targetdrive!\$Recycle.Bin^)>>!bat_warningfile!
							)
							echo      c^) Use Disk Cleanup to free up temporary space and other areas.>>!bat_warningfile!
							echo      e^) Move "!bat_camsdir!\SubmissionFiles" to an external hard drive.>>!bat_warningfile!
							echo      e^) Move "!bat_camsdir!\CapturedFiles" to an external hard drive.>>!bat_warningfile!
							echo      f^) As a last resort, try specifying a different drive for capture >>!bat_warningfile!
							echo.
							!bat_logexec! type "!bat_warningfile!"

							if /i "!bat_freeMB!" neq "" (
								start /max notepad "!bat_warningfile!"
							)
						)& rem if !bat_estSizeMB!...
				)& rem "!bat_SIMULATION.capture!" neq "true"

			)& rem "%%a" neq "----------"...
		)& rem for /f "usebackq tokens=1-16 skip=2 delims=," ...
		!bat_pop!
	!bat_pop!

:REVIEW_END
!bat_poplog! [RETURN !bat_batchfile!:REVIEW] []
!bat_log!
!bat_return!





	Rem start the execution of the meteor detection pass in background priority

	Rem start /min sndrec32 /play /close %windir%\media\ding.wav

:GETDELAY
!bat_pushlog! [CALL !bat_batchfile!:GETDELAY] %*
	Rem set /a bat_waituntil=!bat_delay!
	Rem set /a bat_startmin=!ftp_mi!+1
	set /a bat_delayhh=!bat_delaycalc!/3600
	set bat_delayhh=00!bat_delayhh:~-2!
	set bat_delayhh=!bat_delayhh:~-2!
	call :STRIPZEROS !bat_delayhh! bat_delay_h
	set /a bat_delaymm=(!bat_delaycalc!-(!bat_delay_h!*3600))/60
	Rem set /a bat_delaymm=(!bat_delaycalc!-(!bat_delayhh!*3600))/60
	set bat_delaymm=00!bat_delaymm:~-2!
	set bat_delaymm=!bat_delaymm:~-2!
	call :STRIPZEROS !bat_delaymm! bat_delay_m
	set /a bat_delayss=(!bat_delaycalc!-(!bat_delay_h!*3600))-(!bat_delay_m!*60)
	set bat_delayss=00!bat_delayss:~-2!
	set bat_delayss=!bat_delayss:~-2!

	set bat_unique=ftp_capture_!ftp_year!!ftp_mo!!ftp_da!!ftp_hr!!bat_startmin!

:GETDELAY_END
!bat_poplog! [RETURN !bat_batchfile!:GETDELAY] [!bat_delaycalc! seconds, bat_delayhh=!bat_delayhh!, bat_delaymm=!bat_delaymm!, bat_delayss=!bat_delayss!, bat_unique=!bat_unique!]
!bat_log!
!bat_return!




:WAITDELAY
!bat_pushlog! [CALL !bat_batchfile!:WAITDELAY] %*
	!bat_echolog!
	!bat_echolog! Waiting !bat_delaycalc! seconds ^(delay of !bat_delayhh!:!bat_delaymm!:!bat_delayss!^) from the current time of 
	!bat_echolog! !ftp_hhmmssdd_local! local time until !bat_date! !bat_dusklocal!.
	!bat_echolog!
	!bat_echolog! Then running: call "!bat_capturebat!" !bat_CD_flag! "!bat_CAPTURE.cameralist!" !bat_duration! !bat_targetlocation! !bat_autonomous!
	!bat_echolog!
	!bat_echolog! !bat_batchfile! - Waiting until !bat_dusklocal! for call "!bat_capturebat!" !bat_CD_flag! "!bat_CAPTURE.cameralist!" !bat_duration! !bat_targetlocation!
	        title !bat_batchfile! - Waiting until !bat_dusklocal! for call "!bat_capturebat!" !bat_CD_flag! "!bat_CAPTURE.cameralist!" !bat_duration! !bat_targetlocation!

	!bat_echolog!
	!bat_echolog! !bat_WAIT!  !bat_delaycalc!
	
	set /a bat_tenth=(!bat_delaycalc!)/10

	
	if /i "!bat_CAPTURE.waitdelay!" equ "no wait" (
		!bat_echolog! `    SKIPPING WAITDELAY - CAPTURE.waitdelay=!bat_CAPTURE.waitdelay!
		goto :WAITDELAY_END
	)


	set bat_titlestring=Waiting until !bat_dusklocal! for call "!bat_capturebat!" !bat_CD_flag! !bat_camera! !bat_duration! !bat_targetlocation!
	if "!bat_SIMULATION.capture!" equ "true" (
		!bat_echolog! `    SIMULATION - SKIPPING WAITDELAY
		goto :WAITDELAY_END
	)& rem if "!bat_SIMULATION.capture!"...


	!bat_push!
			 title %0- ---------- !bat_titlestring!
			 !bat_echolog! `     %0- ---------- !bat_titlestring!
		 !bat_WAIT! !bat_tenth!
						   title %0- #---------  !bat_titlestring!
			 !bat_echolog! `     %0- #---------  !bat_titlestring!
		 !bat_WAIT! !bat_tenth!
						   title %0- ##--------  !bat_titlestring!
			 !bat_echolog! `     %0- ##--------  !bat_titlestring!
		 !bat_WAIT! !bat_tenth!
						   title %0- ###-------  !bat_titlestring!
			 !bat_echolog! `     %0- ###-------  !bat_titlestring!
		 !bat_WAIT! !bat_tenth!
						   title %0- ####------  !bat_titlestring!
			 !bat_echolog! `     %0- ####------  !bat_titlestring!
		 !bat_WAIT!  !bat_tenth!
						   title %0- #####-----  !bat_titlestring!
			 !bat_echolog! `     %0- #####-----  !bat_titlestring!
		 !bat_WAIT!  !bat_tenth!
						   title %0- ######----  !bat_titlestring!
			 !bat_echolog! `     %0- ######----  !bat_titlestring!
		 !bat_WAIT!  !bat_tenth!
						   title %0- #######---  !bat_titlestring!
			 !bat_echolog! `     %0- #######---  !bat_titlestring!
		 !bat_WAIT!  !bat_tenth!
						   title %0- ########--  !bat_titlestring!
			 !bat_echolog! `     %0- ########--  !bat_titlestring!
		 !bat_WAIT!  !bat_tenth!
						   title %0- #########-  !bat_titlestring!
			 !bat_echolog! `     %0- #########-  !bat_titlestring!
		 !bat_WAIT!  !bat_tenth!
						   title %0- ##########  !bat_titlestring!
			 !bat_echolog! `     %0- ##########  !bat_titlestring!
	!bat_pop!

:WAITDELAY_END
!bat_poplog! [RETURN !bat_batchfile!:WAITDELAY] [!bat_delaycalc!]
!bat_log!
!bat_return!




:CAPTURE
!bat_pushlog! [CALL !bat_batchfile!:CAPTURE] %*

	!bat_push!
	!bat_push!
	!bat_push!

				!bat_echolog! Logging the variables to pass on to the child capture environment...
				call :VARIABLES_LOG bat_log
				!bat_echolog! .......
	!bat_pop!
	!bat_pop!
	!bat_pop!


	!bat_log! if /i "!bat_CAPTURE.capture_mode!" equ "capturetwoanddetect" ^(
	if           /i "!bat_CAPTURE.capture_mode!" equ "capturetwoanddetect" (
		call :CAPTURETWO %*
		goto :CAPTURE_END
	)


	!bat_log! if /i "!bat_CAPTURE.capture_mode!" equ "capture8anddetect" ^(
	if           /i "!bat_CAPTURE.capture_mode!" equ "capture8anddetect" (
		call :CAPTURE8 %*
		goto :CAPTURE_END
	)

	!bat_log! if /i "!bat_CAPTURE.capture_mode!" equ "captureanddetect" ^(
	if           /i "!bat_CAPTURE.capture_mode!" equ "captureanddetect" (
		call :CAPTUREONE %*
		goto :CAPTURE_END
	)


:CAPTURE_END
!bat_poplog! [RETURN !bat_batchfile!:CAPTURE] %*
!bat_log!
!bat_return!



:CAPTUREONE
!bat_pushlog! [CALL !bat_batchfile!:CAPTUREONE] 

	REM Set the launch string. Notice that the thing to launch is an EXE instead of a customized script.  
	!bat_logexec! set bat_titlestring=start "!bat_captureBAT!" cmd /c ""!bat_captureBAT!" !bat_CD_flag! !bat_CAPTURE.cameralist! !bat_duration! !bat_targetlocation! !bat_autonomous!"

	!bat_echolog!
	!bat_echolog! [!date! !time!] ASYNCHRONOUSLY EXECUTING CAPTURE SCRIPT NOW, [without waiting] USING: 
	!bat_echolog! `                          !BAT_TITLESTRING!
	                                         !BAT_TITLESTRING!

:CAPTUREONE_END
!bat_poplog! [RETURN !bat_batchfile!:CAPTUREONE] [bat_titlestring=!bat_titlestring!]
!bat_log!
!bat_return!





:CAPTURETWO
!bat_pushlog! [CALL !bat_batchfile!:CAPTURETWO] 

	REM Set the launch string. Notice that the thing to launch is a customize script.  
	echo. set bat_titlestring=!bat_captureBAT! !bat_CD_flag! "!bat_CAPTURE.cameralist!" !bat_duration! !bat_targetlocation! !bat_autonomous!
	!bat_logexec! set bat_titlestring=start "!bat_captureBAT!" cmd /c ""!bat_captureBAT!" !bat_CD_flag! "!bat_CAPTURE.cameralist!" !bat_duration! !bat_targetlocation! !bat_autonomous!"

	!bat_echolog!
	!bat_echolog! [!date! !time!] ASYNCHRONOUSLY EXECUTING CAPTURE SCRIPT NOW USING: 
	!bat_echolog! `                          !BAT_TITLESTRING!
	                                         !BAT_TITLESTRING!

:CAPTURETWO_END
!bat_poplog! [RETURN !bat_batchfile!:CAPTURETWO] [bat_titlestring=!bat_titlestring!]
!bat_log!
!bat_return!




:CAPTURE8
!bat_pushlog! [CALL !bat_batchfile!:CAPTURE8] 

	REM Set the launch string. Notice that the thing to launch is a customize script.  
	!bat_logexec! set bat_titlestring=start "!bat_captureBAT!" cmd /c ""!bat_captureBAT!" !bat_duration!  !bat_autonomous!"

	!bat_echolog!
	!bat_echolog! [!date! !time!] ASYNCHRONOUSLY EXECUTING CAPTURE SCRIPT NOW USING: 
	!bat_echolog! `               !BAT_TITLESTRING!
                                  !BAT_TITLESTRING!

:CAPTURE8_END
!bat_poplog! [RETURN !bat_batchfile!:CAPTURE8] [!bat_titlestring!]
!bat_log!
!bat_return!











:POSTCAPTURE
!bat_pushecho! [CALL !bat_batchfile!:POSTCAPTURE] %*
	call GETCAPTUREDIR %*
	call GETARCHIVEDDIR %*

:POSTCAPTURE_END
!bat_popecho! [RETURN !bat_batchfile!:POSTCAPTURE] [bat_captureddir="!bat_captureddir!", bat_archiveddir="!bat_archiveddir!", bat_captureddate=!bat_captureddate!]
!bat_log!
!bat_return!


	

:GETCAPTUREDIR
!bat_pushecho! [CALL !bat_batchfile!:GETCAPTUREDIR] %*
	set bat_targetsearchspec=!bat_camsdir!\CapturedFiles

	Rem Iterate through the list of !bat_camsdir!\CapturedFiles directories, 
	Rem ordered ascending by date. The last one will be the one created by the 
	Rem FTP_CaptureAndDetect.exe/FTP_Capture.exe program.
	!bat_pushecho! for /f "usebackq tokens=*" %%a in ^(`dir "!bat_targetsearchspec!" /ad /s/b`^) do...
	for               /f "usebackq tokens=*" %%a in  (`dir "!bat_targetsearchspec!" /ad /s/b`) do (
		if /i "%%~nxa" neq "movies" (
			if /i "%%~nxa" neq "displayJPEGS" (
				if /i "%%~nxa" neq "gefdat" (
					!bat_log! bat_targetsearchdir=%%~a
					set bat_targetsearchdir=%%~a

					!bat_log! bat_captureddir=%%~a
					set bat_captureddir=%%~a

					!bat_log! set bat_captureddirname=%%~nxa
					set bat_captureddirname=%%~nxa

					set bat_capturedbasedir=%%~dpa::
					set bat_capturedbasedir=!bat_capturedbasedir:\::=!
					set bat_capturedbasedir=!bat_capturedbasedir:::=!
				)
			)
		)
	)
	!bat_pop!
	
	set bat_captureddate=!bat_captureddirname:~0,10!

	!bat_logexec! set bat_cap
:GETCAPTURE_END
!bat_popecho! [RETURN !bat_batchfile!:GETCAPTURE] [bat_captureddir=!bat_captureddir!]
!bat_log!
!bat_return!


:GETARCHIVEDDIR
!bat_pushecho! [CALL !bat_batchfile!:GETARCHIVEDDIR] %*
	set bat_targetsearchspec=!bat_camsdir!\ArchivedFiles

	Rem Iterate through the list of !bat_camsdir!\ArchivedFiles directories, 
	Rem ordered ascending by date. The last one will be the one created by the 
	Rem FTP_CaptureAndDetect.exe/FTP_Capture.exe program.
	!bat_echolog! for /f "usebackq tokens=*" %%a in ^(`dir "!bat_targetsearchspec!" /ad /s/b`^) do...
	for               /f "usebackq tokens=*" %%a in  (`dir "!bat_targetsearchspec!" /ad /s/b`) do (
		if /i "%%~nxa" neq "movies" (
			if /i "%%~nxa" neq "displayJPEGS" (
				if /i "%%~nxa" neq "gefdat" (
					set bat_targetsearchdir=%%~a
					!bat_log! bat_targetsearchdir=%%~a

					set bat_archiveddir=%%~a
					!bat_log! bat_archiveddir=%%~a

					!bat_echolog! set bat_archiveddirname=%%~nxa
					set bat_archiveddirname=%%~nxa

					set bat_archivedbasedir=%%~dpa::
					set bat_archivedbasedir=!bat_archivedbasedir:\::=!
					set bat_archivedbasedir=!bat_archivedbasedir:::=!
				)
			)
		)
	)
	
	set bat_archiveddate=!bat_archiveddirname:~0,10!
	!bat_logexec! set bat_archived

	set bat_detectfile=!bat_archiveddir!\FTPdetectinfo_0!bat_camera!_!bat_captureddate!.txt
	!bat_echolog! if not exist "!bat_detectfile!" ...
	if not exist "!bat_detectfile!" (
		!bat_echolog!
		!bat_echolog! ERROR - "!bat_detectfile!" not found...
		!bat_echolog!
		!bat_logexec! dir "!bat_archiveddir!\ftpdetect*.*" /s/b /a-d /od
		goto :POSTCAPTURE_END
	) else (
		Rem move /y "!bat_detectfile!" "!bat_archiveddir!\FTPdetectinfo.txt"
	)

:GETARCHIVEDDIR
!bat_popecho! [RETURN !bat_batchfile!:GETARCHIVEDDIR] [bat_archiveddir=!bat_archiveddir!]
!bat_log!
!bat_return!







:MULTISCAN
!bat_pushecho! [CALL !bat_batchfile!:MULTISCAN] %*
	!bat_pushecho! for /f "usebackq tokens=*" %%a in ^(`dir "!bat_targetsearchspec!" /ad /s/b`^) do...
	for                /f "usebackq tokens=*" %%a in  (`dir "!bat_targetsearchspec!" /ad /s/b`) do (
		if /i "%%~nxa" neq "movies" (
			if /i "%%~nxa" neq "displayJPEGS" (
				if /i "%%~nxa" neq "gefdat" (
					!bat_log! bat_targetsearchdir=%%~a
					set bat_targetsearchdir=%%~a
					!bat_log! bat_captureddir=%%~a
					set bat_captureddir=%%~a
				)
			)
		)
	)
	!bat_pop!

	echo.>"!bat_captureddir!\rerun_makeCaseListing.bat"
	echo @echo off>>"!bat_captureddir!\rerun_makeCaseListing.bat"
	echo setlocal enableextensions enabledelayedexpansion>>"!bat_captureddir!\rerun_makeCaseListing.bat"
	for /f "usebackq tokens=*" %%a in (`set`) do (
		set %%a>>"!bat_captureddir!\rerun_makeCaseListing.bat"
	)

	echo call makeCaseListing.bat "!bat_captureddir!">>"!bat_captureddir!\rerun_makeCaseListing.bat"

	!bat_CALLecho! makeCaseListing.bat "!bat_captureddir!"

:MULTISCAN_END
!bat_popecho! [RETURN !bat_batchfile!:MULTISCAN] []
!bat_log!
!bat_return!



:WAITFORMULTISCAN
!bat_pushecho! [CALL !bat_batchfile!:WAITFORMULTISCAN] %*
	!bat_WAIT! 60

	set bat_archivedsearchspec=!bat_camsdir!\ArchivedFiles
	!bat_echolog! for /f "usebackq tokens=*" %%a in ^(`dir "!bat_archivedsearchspec!" /ad /s/b`^) do...
	for /f "usebackq tokens=*" %%a in       (`dir "!bat_archivedsearchspec!" /ad /s/b`) do (
		if /i "%%~nxa" neq "movies" (
			if /i "%%~nxa" neq "displayJPEGS" (
				if /i "%%~nxa" neq "gefdat" (
					!bat_log! bat_archiveddir=%%~a
					set bat_archiveddir=%%~a
				)
			)
		)
	)

	!bat_pushecho! for /f "usebackq tokens=*" %%a in ^(`dir "!bat_archiveddir!\ftpdetectinfo_Case???.txt" /s/b /a-d`^) do ...
	for               /f "usebackq tokens=*" %%a in (`dir "!bat_archiveddir!\ftpdetectinfo_Case???.txt" /s/b /a-d`) do (
		if /i "%%~nxa" neq "movies" (
			if /i "%%~nxa" neq "displayJPEGS" (
				if /i "%%~nxa" neq "gefdat" (
					call WaitForFile rw "%%a"
				)
			)
		)
	)
	!bat_pop!
	
:WAITFORMULTISCAN_END
!bat_popecho! [RETURN !bat_batchfile!:WAITFORMULTISCAN] %*
!bat_log!
!bat_return!


:POSTMULTISCAN
!bat_pushecho! [CALL !bat_batchfile!:POSTMULTISCAN] %*
	!bat_CALLecho! PostMultiScanCalibration.bat "!bat_archiveddir!"

:POSTMULTISCAN_END
!bat_popecho! [RETURN !bat_batchfile!:POSTMULTISCAN] []
!bat_log!
!bat_return!


Rem	EXIT

goto :DONE




:date2jdate JD YYYY MM DD -- converts a gregorian calender date to julian day format
	Rem                        -- JD   [out] - julian days
	Rem                        -- YYYY [in]  - gregorian year, i.e. 2006
	Rem                        -- MM   [in]  - gregorian month, i.e. 12 for december
	Rem                        -- DD   [in]  - gregorian day, i.e. 31
	!bat_CALL! "!bat_exedir!\Date2JDate.bat" %1 %2 %3 %4
EXIT /b 0




:jdate2date JD YYYY MM DD -- converts julian days to gregorian date format
	Rem                     -- JD   [in]  - julian days
	Rem                     -- YYYY [out] - gregorian year, i.e. 2006
	Rem                     -- MM   [out] - gregorian month, i.e. 12 for december
	Rem                      -- DD   [out] - gregorian day, i.e. 31
	!bat_CALL! "!bat_exedir!\JDate2Date.bat" %1 %2 %3 %4
EXIT /b 0







:CALCDURATION
!bat_pushlog! [CALL !bat_batchfile!:CALCDURATION] %*
	call :PARSEOFFTIME "!bat_off!"
	call :PARSEONTIME "!bat_on!"
	

	Rem set /a bat_temp=!bat_onsec!-!bat_offsec!
	
	Rem                           bat_onh   bat_onm   bat_ons bat_offh  bat_offm  bat_offs
	Rem                            %1        %2        %3        %4       %5         %6
	Rem call :SUBTRACTONMINUSCUR !bat_th1! !bat_tm1! !bat_ts1! !bat_th4! !bat_tm4! !bat_ts4!
	
	!bat_log! set /a                  (!bat_midnightsecs!-!bat_onsec!)+!bat_offsec!
	          set /a                  (!bat_midnightsecs!-!bat_onsec!)+!bat_offsec!
	
	!bat_push!
		!bat_push!
			!bat_echolog! Total duration=^(!bat_midnightsecs!-!bat_onsec!^)+!bat_offsec!=!bat_durtotalsecs! seconds

			set /a bat_temp=(!bat_durtotalsecs!*100)/3600
			set /a bat_temph=(!bat_durtotalsecs!)/3600
			set /a bat_tempm=!bat_temp:~-2!
			set bat_tempm=00!bat_tempm!
			set bat_tempm=!bat_tempm:~-2!

			set /a bat_durh=!bat_durtotalsecs!/3600
			set /a bat_durm=(!bat_durtotalsecs!-(!bat_durh!*3600))/60
			set /a bat_durs=(!bat_durtotalsecs!-(!bat_durh!*3600))-(!bat_durm!*60)

			call :PADZERO bat_offh
			call :PADZERO bat_offm
			call :PADZERO bat_offs
			call :PADZERO bat_durh
			call :PADZERO bat_durm
			call :PADZERO bat_durs

			!bat_echolog! `   !bat_offh!:!bat_offm!:!bat_offs!	Camera OFF time ^(next morning^)
			!bat_echolog! `-  !bat_ONh!:!bat_ONm!:!bat_ONs!	Current ON time
			!bat_echolog! `-----------------
			!bat_echolog! `   !bat_durh!:!bat_durm!:!bat_durs!   !bat_msg! ^<---- duration h:mm:ss ^(!bat_duration! hours^)
			!bat_echolog!
		!bat_pop!
	!bat_pop!
	
	if /i "!bat_CAPTURE.sunangle!" equ "true" (
		set bat_dur=!bat_CAPTURE.sunangle_dusk!
		!bat_echolog! Using sun angle: [!bat_CAPTURE.sunangle_dusk!]
	)
	
:CALCDURATION_END
!bat_poplog! [RETURN !bat_batchfile!:CALDURATION] [bat_durtotalsecs=!bat_durtotalsecs!; 
!bat_log! `                                       ON=!bat_durh!, !bat_durm!, !bat_durs!;
!bat_log! `                                       OFF=!bat_offh!, !bat_offm!, !bat_offs!]
!bat_log!
!bat_return!





:CALCDELAY
!bat_pushlog! [CALL !bat_batchfile!:CALCDELAY] %*
	Rem Usage: call :CALCDELAY "!bat_on!" "!ftp_hhmmssdd!"
	call :PARSECURTIME "%~2"
	call :PARSEONTIME "%~1"
	
	set /a bat_temp=!bat_onsec!-!bat_cursec!
	
	Rem                       bat_onh   bat_onm   bat_ons bat_curh  bat_curm  bat_curs
	Rem                        %1        %2        %3        %4       %5         %6
	call :SUBTRACTONMINUSCUR !bat_th1! !bat_tm1! !bat_ts1! !bat_th3! !bat_tm3! !bat_ts3!
	
	set bat_msg= 
	
	!bat_push!
	if !bat_delay! lss 0 (
		Rem                       bat_curh  bat_curm  bat_curs bat_onh   bat_onm   bat_ons
		Rem                        %1        %2        %3        %4       %5         %6
		call :SUBTRACTONMINUSCUR !bat_th3! !bat_tm3! !bat_ts3! !bat_th1! !bat_tm1! !bat_ts1!
		
		
		!bat_log! Delay=!bat_delay! seconds
		set bat_msg=   IMMEDIATE [we're starting late]   
	) else (
		!bat_log! Delay=!bat_delaycalc! seconds
	)
	!bat_pop!
	
	call :PADZERO bat_ONh
	call :PADZERO bat_ONm
	call :PADZERO bat_ONs
	call :PADZERO bat_newh
	call :PADZERO bat_newm
	call :PADZERO bat_news
	
	call :DISPLAY

:CALCDELAY_END
!bat_poplog! [RETURN !bat_batchfile!:CALCDELAY] [bat_delay=!bat_delay!]
!bat_log!
!bat_return!




:PARSECURTIME
!bat_pushlog! [CALL !bat_batchfile!:PARSECURTIME] %*
	set bat_curtime=%~1
	set bat_curh=!bat_curtime:~0,2!
	set bat_curm=!bat_curtime:~3,2!
	set bat_curs=!bat_curtime:~6,2!!bat_curtime:~-2!

	Rem strip zeros
	for /f "tokens=* delims=0" %%m in ("!bat_curs!") do (set bat_nz=%%m)
	if not defined bat_nz (
		set bat_curs=0
	) else (
		set bat_curs=!bat_nz!
	)

	Rem round up to the next second
	set /a bat_curs=!bat_curs!+50
	set /a bat_curs/=100
	call :PADZERO bat_curs
	
	call :STRIPZEROS !bat_curs! bat_ts1
	call :STRIPZEROS !bat_curm! bat_tm1
	call :STRIPZEROS !bat_curh! bat_th1
	
	set /a bat_cursec=!bat_th1!*3600+!bat_tm1!*60+!bat_ts1!	

:PARSECURTIME_END
!bat_poplog! [RETURN !bat_batchfile!:PARSECURTIME] [bat_cursec=!bat_cursec!]
!bat_log!
!bat_return!




:PARSEONTIME
!bat_pushlog! [CALL !bat_batchfile!:PARSEONTIME] %*
	!bat_logexeclog! set bat_temp=0%~1
	!bat_logexeclog! set bat_ontime=!bat_temp:~-8!

	set bat_onh=!bat_ontime:~0,2!
	set bat_onm=!bat_ontime:~3,2!
	set bat_ons=!bat_ontime:~6,2!
	
	Rem stripzeros
	call :STRIPZEROS !bat_ons! bat_ts3
	call :STRIPZEROS !bat_onm! bat_tm3
	call :STRIPZEROS !bat_onh! bat_th3
	
	set /a bat_onsec=!bat_th3!*3600+!bat_tm3!*60+!bat_ts3!	

:PARSEONTIME_END
!bat_poplog! [RETURN !bat_batchfile!:PARSEONTIME] [bat_onsec=!bat_onsec!]
!bat_log!
!bat_return!





:PARSEOFFTIME
!bat_pushlog! [CALL !bat_batchfile!:PARSEOFFTIME] %*
	!bat_logexeclog! set bat_temp=0%~1
	!bat_logexeclog! set bat_offtime=!bat_temp:~-8!

	!bat_logexeclog! set bat_offh=!bat_offtime:~0,2!
	!bat_logexeclog! set bat_offm=!bat_offtime:~3,2!
	!bat_logexeclog! set bat_offs=!bat_offtime:~6,2!
	
	Rem stripzeros
	call :STRIPZEROS !bat_offs! bat_ts4
	call :STRIPZEROS !bat_offm! bat_tm4
	call :STRIPZEROS !bat_offh! bat_th4
	
	!bat_logexeclog! set /a bat_offsec=!bat_th4!*3600+!bat_tm4!*60+!bat_ts4!	
	
:PARSEOFFTIME_END
!bat_poplog! [RETURN !bat_batchfile!:PARSEOFFTIME] [bat_offsec=!bat_offsec!]
!bat_log!
!bat_return!




:PARSEOFFSET
!bat_pushlog! [CALL !bat_batchfile!:PARSEOFFSET] %*
	set /a bat_delay=%~1
	set /a bat_OFFh=!bat_delay!/3600
	set /a bat_OFFr=!bat_delay!-(!bat_OFFh!*3600)
	set /a bat_OFFm=!bat_OFFr!/60
	set /a bat_OFFs=!bat_OFFr!-(!bat_OFFm!*60)
:PARSEOFFSET_END
!bat_poplog! [RETURN !bat_batchfile!:PARSEOFFSET] [!bat_OFFh!, !bat_OFFr!, !bat_OFFm!, !bat_OFFs!
!bat_log!
!bat_return!




:SUBTRACTONMINUSCUR
!bat_pushlog! [CALL !bat_batchfile!:SUBTRACTIONMINUSCUR] %*
	Rem echo :SUBTRACTONMINUSCUR %1:%2:%3    %4:%5:%6
	set /a bat_remainder=0

	set /a bat_news=%6-%3
	if !bat_news! lss 0 (
		set /a bat_news=!bat_news!+60
		set /a bat_borrow=1
	) else (
		set /a bat_borrow=0
	)
	
	set /a bat_newm=%5-%2-!bat_borrow!
	if !bat_newm! lss 0 (
		set /a bat_newm=!bat_newm!+60
		set /a bat_borrow=1
	) else (
		set /a bat_borrow=0
	)
	
	set /a bat_newh=%4-%1-!bat_borrow!
	if !bat_newh! lss 0 (
		set /a bat_newh=!bat_newh!
		set /a bat_borrow=1
	) else (
		set /a bat_borrow=0
	)
	
:SUBTRACTONMINUSCUR_END
!bat_poplog! [RETURN !bat_batchfile!:SUBTRACTIONMINUSCUR] [difference in h,m,s=!bat_newh!, !bat_newm!, !bat_news!]
!bat_log!
!bat_return!





:DISPLAY
!bat_pushlog! [CALL !bat_batchfile!:DISPLAY] %*
	!bat_echolog! `   !bat_ONh!:!bat_ONm!:!bat_ONs!	Camera ON time (adjusted from !bat_calcedon!)
	Rem       echo +  !bat_OFFh!:!bat_OFFm!:!bat_OFFs!	Current Time
	!bat_echolog! `-  !bat_curh!:!bat_curm!:!bat_curs!	Current time
	!bat_echolog! `-----------------
	!bat_echolog! `   !bat_newh!:!bat_newm!:!bat_news! LT  !bat_msg! ^<---- Waiting h:mm:ss
	!bat_echolog!

:DISPLAY_END
!bat_poplog! [RETURN !bat_batchfile!:DISPLAY] []
!bat_log!
!bat_return!





:ADDTIMEPLUSOFFSET
!bat_pushlog! [CALL !bat_batchfile!:ADDTIMEPLUSOFFSET] %*
	set /a bat_remainder=0

	set bat_ts1=0&set bat_ts2=0
	call :STRIPZEROS !bat_curs! bat_ts1
	call :STRIPZEROS !bat_OFFs! bat_ts2

	set /a bat_news=!bat_ts1!+!bat_ts2!
	if !bat_news! gtr 59 (
		set /a bat_news=!bat_news!-60
		set /a bat_carry=1
	) else (
		set /a bat_carry=0
	)

	call :STRIPZEROS !bat_curm! bat_tm1
	call :STRIPZEROS !bat_OFFm! bat_tm2
	set /a bat_newm=!bat_tm1!+!bat_carry!+!bat_tm2!
	if !bat_newm! gtr 59 (
		set /a bat_newm=!bat_newm!-60
		set /a bat_carry=1
	) else (
		set /a bat_carry=0
	)

	call :STRIPZEROS !bat_curh! bat_th1
	call :STRIPZEROS !bat_OFFh! bat_th2
	set /a bat_newh=!bat_th1!+!bat_carry!+!bat_th2!
	if !bat_newh! gtr 23 (
		set /a bat_newh=!bat_newh!-24
		set /a bat_carry=1
	) else (
		set /a bat_carry=0
	)

	call :PADZERO bat_OFFh
	call :PADZERO bat_OFFm
	call :PADZERO bat_OFFs
	call :PADZERO bat_newh
	call :PADZERO bat_newm
	call :PADZERO bat_news

:ADDTIMEPLUSOFFSET_END
!bat_poplog! [RETURN !bat_batchfile!:ADDTIMEPLUSOFFSET] [!bat_OFFh!, !bat_OFFm!, !bat_OFFs!, !bat_newh!, !bat_newm!, !bat_news!]
!bat_log!
!bat_return!




:STRIPZEROS %in_val1% out_var1  -- converts in_val1 and sets out_var1 to the new value
	Rem     -- in_val1    [in]  - zero leading number
	Rem     -- out_var1   [out] - variable name to set the output value to

	setlocal ENABLEDELAYEDEXPANSION enableextensions

	set bat_inval1=%~1

	for /f "tokens=* delims=0" %%m in ("!bat_inval1!") do (set bat_nz=%%m)
	if not defined bat_nz (
		set bat_outvar1=0
	) else (
		set bat_outvar1=!bat_nz!
	)
	( ENDLOCAL & REM RETURN VALUES
		if "%~2" neq "" (set %~2=%bat_outvar1%) else echo.%bat_outvar1%
	)

:STRIPZEROS_END
EXIT /b


:PADZERO
	set %1=00!%1!
	set %1=!%1:~-2!
:PADZERO_END
!bat_return!





		Rem Usage: call :CALCDAYNUMBER
		rem
		rem Requires:
		rem   ftp_year         Can be obtained by calling ParseDate1.bat
		rem   ftp_mo
		rem   ftp_da
		rem
		rem The algorithm is simple: Just get the julian date of today's date.
		rem Then get the julian date of Jan 1 of that same year.
		rem Then subtract the difference
		rem Then add 1 (since Jan 2 - Jan 1 yields 1, where Jan 2 should yield 2)
		rem
		
:CALCDAYNUMBER
!bat_pushlog! [CALL !bat_batchfile!:CALCDAYNUMBER] %*

	!bat_CALL! "!bat_exedir!\date2jdate.bat" bat_JD_jan1 !ftp_year! 01 01
	!bat_CALL! "!bat_exedir!\date2jdate.bat" bat_JD !ftp_year! !ftp_mo! !ftp_da!
	
	set /a bat_day_number=!bat_JD! - !bat_JD_jan1! + 1

:CALCDAYNUMBER_END
!bat_poplog! [RETURN !bat_batchfile!:CALCDAYNUMBER] [todayJulian=!bat_JD!, Jan 1 Julian=!bat_JD_jan1!, Day of Year=!bat_day_number!]
!bat_log!
exit /b !bat_day_number!
!bat_return!


:CALCDAYNUMBER
!bat_pushlog! [CALL !bat_batchfile!:CALCDAYNUMBER] %*

%bat_COMMENT_BEGIN%
	Rem Usage: call :CALCDAYNUMBER day
	Rem   where day is the day number from the sunsettable.txt file.  In that file,
	Rem   the day number in in the file for the first date (01/01/2011) has a value
	Rem   of zero.  It should be reported as day 1 and 12/31/2011 is 365 and
	Rem   01/01/2012 is day 365, which should be reported as day 1 of year 2012. 
	Rem   The only thing that should be an issue between different people using this
	Rem   is the bat_offset number below (109600), which may need to be adjusted 
	Rem   depending how how many days of offset from a daylight savings date
	Rem   is used as the base date.  
	
	set /a bat_day=%1*100
	set /a bat_offset=109600
	set /a bat_year=36525
	
	set /a bat_divide=(((!bat_day!+!bat_offset!)*10000)/!bat_year!)
	set /a bat_int=((!bat_day!+!bat_offset!)/36525)
	set /a bat_mod=!bat_divide!-(!bat_int!*10000)
	set /a bat_res1=!bat_mod!*36525
	set /a bat_res2=!bat_res1!+100000
	set /a bat_res3=(!bat_res2!/1000000)+1
	set bat_res3=!bat_res3!   
	set bat_res3=!bat_res3:~0,3!
%bat_COMMENT_END%

	if not defined bat_JD (
		!bat_echolog! `    WARNING - bat_JD variable should be defined by this time and it's not. 
		GOTO :CALCDAYNUMBER_END
	)
	
	set bat_todayJulian=!bat_JD!
	set /a bat_tomorrowJulian+=1
	
	

:CALCDAYNUMBER_END
!bat_poplog! [RETURN !bat_batchfile!:CALCDAYNUMBER] [todayJulian=!bat_JD!, tomorrowJulian=!bat_tomorrowJulian!, Day of Year=!bat_day_number!]
rem !bat_poplog! [RETURN !bat_batchfile!:CALCDAYNUMBER] [TodayJulian=!bat_res1!, TomorrowJulian!bat_res2!, Day of Year=!bat_res3!]
!bat_log!
ENDLOCAL
!bat_return!


:CALCREQDISK
!bat_pushlog! [CALL !bat_batchfile!:CALCREQDISK] %*

	!bat_logexeclog! set bat_diskspacememo=OK

	call :CHECKDISKSPACE
        Rem !bat_echolog! duration "!bat_duration!"

	for /f "tokens=1,2 delims=!bat_LOCALIZE.separator.decimal!" %%x in ("!bat_duration!") do (

		set /a bat_h=%%x
		set /a bat_d=%%y
		call :STRIPZEROS %%x bat_h
		call :STRIPZEROS %%y bat_d
	)


	Rem convert hours into seconds
	set /a bat_duration_secs=0
	!bat_CALL! "!bat_exedir!\str_math.bat" "!bat_duration! * 3600"
	set bat_duration_secs=!bat_retval!


	Rem Remember that there are 256 frames per file at !bat_CAPTURE.fps! fps
	Rem so Total files = seconds / (256 / 29.97)
	set /a bat_estCapFiles=!bat_duration_secs! * !bat_CAPTURE.fps! / 25600

	Rem note: 1,228,820 bytes per ff*.bin file is 1.1719 MB per file
	Rem       therefore, 4055 files * 11719mb/file / 10000 = 4752 MB or 4.752 GB
	set /a bat_estSizeMB=!bat_estCapFiles!*11719/10000

	!bat_log! Single camera one night file count   : !bat_estCapFiles! files
	!bat_log! Single camera one night requirements : !bat_estSizeMB! MB
	if defined bat_CAPTURE.number.of.cameras (
		set /a bat_estSizeMB=bat_estSizeMB * bat_CAPTURE.number.of.cameras
	)

	!bat_log! All cameras one night requirements   : !bat_estSizeMB! MB
	set /a bat_reqdisk=!bat_estSizeMB!*!bat_AUTOCAMS.diskspacewarning.days!
	set bat_freeMB=!bat_targetfreespace!
	
	!bat_log! All cameras !bat_AUTOCAMS.diskspacewarning.days! nights requirements   : !bat_reqdisk! MB
	set /a bat_sizediff=!bat_reqdisk!-!bat_freeMB!

	if !bat_reqdisk! GEQ !bat_freeMB! (
		set bat_diskspacememo=********* DISK SPACE WARNING! ************
	)


:CALCREQDISK_END
!bat_poplog! [RETURN !bat_batchfile!:CALCREQDISK] [bat_reqdisk=!bat_reqdisk!, bat_freeMB=!bat_freeMB!, bat_sizediff=!bat_sizediff!, bat_diskspacememo=!bat_diskspacememo!]
!bat_log!



:CHECKDISKSPACE
!bat_pushlog! [CALL !bat_batchfile!:CHECKDISKSPACE] %*
	!bat_echo! Waiting for WMIC...

			rem This loop will scan for a file called csv.xsl in all subdirs of wbem.  The last
			rem one found will be in a directory that is named after a locale.  Problem is, I can
			rem imagine several directories for multi-user systems, each with a possible different
			rem locale. 
        FOR /F "tokens=*" %%G IN ('dir/b /s ^"%windir%\system32\wbem\csv.xsl^"') DO (
        	set bat_localedir= %%G
        )

        set bat_localedir=%bat_localedir:~1%

	wmic cpu>nul 2>&1
	if !errorlevel! equ 9009 (
		!bat_logexec! ERROR starting WMIC [9009]
		exit /b 9009
		set bat_free=9999999
		set bat_size=9999999
		set bat_freeMB=999999999
	)
	!bat_echo!

	set bat_targetdrive=!bat_targetlocation:~0,2!
	
	Rem This is the new way.
	!bat_log! for /f "usebackq tokens=1-3" ..a in ^(`dir "!bat_targetdrive!\" /ad`^) do ^(
	for           /f "usebackq tokens=1-3" %%a in  (`dir "!bat_targetdrive!\" /ad`) do (
		set bat_free=%%c
		set bat_free=!bat_free:%bat_LOCALIZE.separator.thousands%=!
		set bat_freeMB=!bat_free:~0,-6!
		set bat_targetfreespace=!bat_freeMB!
	)
	

:CHECKDISKSPACE_END
!bat_poplog! [RETURN !bat_batchfile!:CHECKDISKSPACE] [!bat_freeMB!, !bat_targetfreespace!]
!bat_log!
!bat_return!








:GETWINDOWSVER
!bat_pushlog! [CALL !bat_batchfile!:GETWINDOWSVER] %*

        FOR /F "tokens=*" %%G IN ('dir/b /s ^"%windir%\system32\wbem\csv.xsl^"') DO set csvdir= %%G
        set csvdir=%csvdir:~1%


	wmic /?>nul:
	if %errorlevel% equ 9009 (
		exit /b
		set bat_free=9999999
		set bat_size=9999999
		set bat_freeMB=999999999
	)
	Rem echo :GETWINDOWVER wmic os get version, caption, windowsdirectory /format:list^|findstr /r /c:"=" ... "!bat_tempfile!"
	wmic os get version, windowsdirectory /format:"%csvdir%"|findstr /r /c:",">"!bat_tempfile!"
	Rem type "!bat_tempfile!"
	for /f "usebackq delims=, tokens=1-4*" %%a in (`type "!bat_tempfile!"`) do (
		if "%%b" gtr "!" (
			set bat_winver=%%b
		)
	)
:GETWINDOWSVER_END
!bat_poplog! [RETURN !bat_batchfile!:GETWINDOWSVER] [!bat_winver!]
!bat_log!
!bat_return!


:CALCTIMEREM

:CALCTIMEREM_END
!bat_return!



:
:COPYEXE
!bat_pushlog! [CALL !bat_batchfile!:COPYEXE] %*
	!bat_logexec! copy /y "!bat_exedir!\!bat_captureexesrc!" "!bat_exedir!\!bat_captureexe!"

:COPYEXE_END 
	!bat_poplog! [RETURN !bat_batchfile!:COPYEXE] [!bat_captureexe!]
!bat_log!
!bat_return!




:VARIABLES_LOG
!bat_pushlog! [CALL !bat_batchfile!:VARIABLES_LOG] []
	!%~1! set bat_
	!bat_push!
	for /f "usebackq tokens=*" %%a in (`set bat_`) do (
		!%~1! %%a
	)
	!%~1!
	!bat_pop!

	!%~1! set ftp_
	!bat_push!
	for /f "usebackq tokens=*" %%a in (`set ftp_`) do (
		!%~1! %%a
	)
	!%~1!
	!bat_pop!
:VARIABLES_LOG_END
!bat_poplog! [RETURN !bat_batchfile!:VARIABLES_LOG] []
!bat_log!
!bat_return!



:READPARAMS
!bat_pushlog! [CALL !bat_batchfile!:READPARAMS] %*
	!bat_CALL! "!bat_exedir!\ReadParams.bat"
:READPARAMS_END
!bat_poplog! [RETURN !bat_batchfile!:READPARAMS] [!bat_paramsfile!=!bat_retval!]
!bat_log!
!bat_return!













:HELP
	echo ================================================================
    echo     %~nx0		batch file name
    echo.
    echo     Usage: %~nx0 cameranumber duration "targetdir" [autonomous] [/?]
    echo.
    echo   Where:
    echo     cameranumber     The camera number
    echo.
    echo     duration         The duration is only a dummy value that 
    echo                      is no longer used ^(enter any number, such as 
    echo                      0^).  Instead, the script was updated to 
    echo                      calculate the duration value from the launch 
    echo                      time and/or the !bat_sunsettable! file.  
    echo.
    echo     "targetdir"      You should put quotes around it just in case
    echo                      there are spaces in the dir/path.  This value
    echo                      should represent the parent dir of the CAMS
    echo                      directory.  For example: "C:\" if the CAMS
    echo                      directory is on "C:\CAMS".
    echo.
    echo     autonomous       This enables autonomous mode. In 
    echo                      autonomous mode, it performs all the
    echo                      processing steps after capture, including:
    echo                      autocal, apply cal, submission, zip, 
    echo                      and ^(in the future, ftp upload^)
    echo.
    echo   This script runs as follows:
    echo      1^) Looks up the dusk/dawn and other values from the 
    echo         "!bat_sunsettable!" file
    echo      2^) Calculates the duration
    echo      3^) Delays until the camera ON time
    echo      4^) Launches the "!bat_capturebat!" script. 
    echo.
    echo   %~n0 can be run by either of the following methods:
    echo       - Windows Task Scheduler
    echo       - Command-line
    echo       - AutoCams Utility menu
    echo.
    echo   It can't be launched by double-clicking on it from Windows
    echo   because the required arguements wouldn't be supplied.
    echo.
    echo   Preferably, %~n0 was designed to be launched from Windows Task
    echo   Scheduler with all the parameters supplied as command-line
    echo   arguments.  
    echo.
    echo   CheckTwilight.bat was provided for you to preview and validate 
    echo   the calculated times and durations.  You will set your wall 
    echo   timer and the Windows Task Scheduler according to the 
    echo   CheckTwilight.bat report. 
    echo.
    echo   If you run it from a command shell or batch script, you must
    echo   specify the camera number, duration, and target location as 
    echo   command-line arguments.  If the command-line arguments don't 
    echo   provide all the necessary parameters, this help text will be 
    echo   displayed.
    echo.
    echo   If it is executed after the scheduled Camera ON time, then 
    echo   there will be no delay and CaptureAndDetect.bat will be launced
    echo   immediately and the duration will be calculated based on that
    echo   start time.
    echo.
    echo   Example ^(assume this is run at 17:00:00 local and the 
    echo   "!bat_sunsettable!" Camera ON/OFF times are ON=18:51:09, OFF=
    echo   06:45:40^):
    echo.
    echo       %~n0 000299 -1 "h:"          The camera capture is delayed for
    echo                                          1:51:09 hours until 
    echo                                          the Camera ON time, after
    echo                                          which, Capture is
    echo                                          launched with the calculated
    echo                                          duration value ^(11.91 hours^), 
    echo.
    echo   Example ^(assume this is run at 19:30:15 local and the 
    echo   "!bat_sunsettable!" Camera ON/OFF times are ON=18:51:09, OFF=
    echo   06:45:40^):
    echo.
    echo       %~n0 000299 -1 "h:"          The camera capture is not delayed, 
    echo                                          19:30:15 will be substituted 
    echo                                          as the new camera ON time 
    echo                                          and Capture is launched with 
    echo                                          the calculated duration value 
    echo                                          ^(11.26 hours^), which will 
    echo                                          calculate an accurate duration 
    echo                                          based on the camera ON time and 
    echo                                          the camera OFF time.
    echo.
    echo.
    echo   See the CaptureAndDetect.bat or FTP_Capture.bat for more on the capture program.
    echo.
    echo   The AutoCams.Params.ini file contains settings that govern which
    echo   script that this script chains to for capture. 
    echo   The current capture script is: "!bat_capturebat!".
	echo.
	echo.Uses Autocams.params.ini settings:
	echo.   [CAPTURE].sunangle
	echo.   [CAPTURE].sunangle_dusk
	echo.   [CAPTURE].sunangle_dawn
	echo.   [CAPTURE].waitdelay
	echo.   [CAPTURE].capture_mode

:HELP_END
EXIT /b -99
    
goto :DONE


:DONE
	!bat_echolog!
	!bat_echolog! [!date!, !time!] !bat_batchfile!:DONE]
	!bat_popecho! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	Rem Wait a few seconds to allow the user to view what's happening.
	echo.Waiting three minutes to allow you to look at things.  It's at a state
	echo.where it can't interfere with the capture program just launched. 
	echo.Feel free to abort the Wait timer at any time by pressing any key.
	!bat_WAIT! 180
	exit /b 0


