
@echo off
setlocal enableextensions enabledelayedexpansion

	(echo.%*|findstr /L /i /c:"/help" /c:"/h" /c:"/?" >nul: 2>&1) && (goto :HELP)


:SETUP
	call "%~dp0AutoCams.setup.bat" "%~dpnx0"
	call "%~dp0indent_setup.bat"

	if not defined bat_COMMENT_BEGIN (
		set bat_pushstring=
	)

rem	if not defined bat_LOCALIZE.locale (
		call "!bat_exedir!\ReadParams.bat"
rem	)
	
	!bat_log!
	!bat_log! =============================================================
	!bat_log! [!date! !time!] - Version: [!bat_vers! - %~dpnx0 %*]
	!bat_log! [!bat_batchfile!:SETUP %*]  Please wait...
	!bat_pushlog!



:MAIN
	call :PARSETIME %*

goto :DONE




	rem =============================================================
	rem                         PARSETIME
	rem 
	rem Uses the TIME DOS env var for the current time and parses it 
	rem out according the the current local into its hr, min, sec, 
	rem parts.  These parts can be used elsewhere in the script. 
	rem 
	rem Returns:
	rem   When run at 10:57:13.52 PM on 10/05/2013, it should result in:
	rem    ftp_hhmmssdd         22:57:13.52
	rem    ftp_hhmmssdd_local   10:57:13.52 PM  [has AM and PM]
	rem    ftp_hr               22
	rem    ftp_mi               57
	rem    ftp_se               13
	rem    ftp_sed              52              [means SEconds Decimal for thousanths of a second]
    rem    ftp_jd               2456413    Julian Date
	rem =============================================================
	
:PARSETIME
rem !bat_pushlog! [CALL !bat_batchfile!:PARSETIME] %*
	set ftp_hhmmssdd=
	set ftp_hhmmssdd_local=
	set ftp_hr=
	set ftp_mi=
	set ftp_se=
	set ftp_sed=
	set ftp_jd=
	set ftp_cams_zulu=
	set ftp_8601_zulu=
	set ftp_8601_local=
	set ftp_cams_local=
	
	set ftp_wmdy=
	set ftp_year=
	set ftp_mo=
	set ftp_mo2=
	set ftp_da=
	set ftp_da2=
	set ftp_www=
	set bat_twilight_searchstr=
	set ftp_LocalDate=

	REM LOCALIZE STARTING HERE
	if /i "%~1" equ "" (
		set ftp_hhmmssdd=!time!
	) else (
		CALL :PARSE_ARGS %~1
	)
	
	set ftp_hr=!ftp_hhmmssdd:~0,2!
	set ftp_hr=!ftp_hr: =!
	set ftp_hr=00!ftp_hr!
	set ftp_hr=!ftp_hr:~-2!

	set ftp_mi=!ftp_hhmmssdd:~3,2!
	set ftp_se=!ftp_hhmmssdd:~6,2!
	set ftp_sed=!ftp_hhmmssdd:~9,6!
	set ftp_hhmmssdd=!ftp_hr!:!ftp_mi!:!ftp_se!.!ftp_sed!

	call "!bat_exedir!\PARSEDATE1.bat" %*

        call :date2jdate ftp_jd !ftp_year! !ftp_mo2! !ftp_da2!
	set /a bat_midnightsecs=86400
	
	if !ftp_hr! GTR 12 (
		set /a ftp_temp=ftp_hr-12
		set ftp_hhmmssdd_local=!ftp_temp!:!ftp_mi!:!ftp_se!.!ftp_sed! PM
	) else (
		set ftp_hhmmssdd_local=!ftp_hhmmssdd! AM
	)
	
	set bat_temp_timestampstr=!ftp_year!/!ftp_mo!/!ftp_da! !ftp_hr!:!ftp_mi!:!ftp_se! !ftp_ampm!
	CALL "!bat_exedir!\getTimestamp2.bat" -U -R ftp_cams_zulu  -D "'!bat_temp_timestampstr!'" -F "{yyyy}_{mm}_{dd}_{hh}_{nn}_{ss}"
	CALL "!bat_exedir!\getTimestamp2.bat" -R    ftp_8601_local -D "'!bat_temp_timestampstr!'" -F "{yyyy}_{mm}_{dd}_{hh}_{nn}_{ss}{tz}"
	CALL "!bat_exedir!\getTimestamp2.bat" -R    ftp_cams_local -D "'!bat_temp_timestampstr!'" -F "{yyyy}_{mm}_{dd}_{hh}_{nn}_{ss}"
	CALL "!bat_exedir!\getTimestamp2.bat" -U -R ftp_8601_zulu  -D "'!bat_temp_timestampstr!'" -F "{yyyy}-{mm}-{dd}T{hh}-{nn}-{ss}{tz}"

:PARSETIME_END
rem !bat_poplog! [RETURN !bat_batchfile!:PARSETIME] [ftp_hhmmssdd=!ftp_hhmmssdd!, ftp_hr=!ftp_hr!, ftp_mi=!ftp_mi!, ftp_se=!ftp_se!, ftp_jd=!ftp_jd!, ftp_hhmmssdd_local=!ftp_hhmmssdd_local!]
rem !bat_log!
!bat_return!






	REM #######################################################################
	REM Parse the command line arguments into constituent parts.
	REM
	
:PARSE_ARGS [IN] date  time  ampm
rem !bat_pushlog! [CALL !bat_batchfile!:PARSE_ARGS] %*

	set ftp_hhmmssdd=%~2
	set ftp_ampm=%~3

:PARSE_ARGS_END
rem !bat_poplog! [RETURN !bat_batchfile!:PARSE_ARGS] [ftp_hhmmssdd=!ftp_hhmmssdd!, ftp_ampm=!ftp_ampm!, %~1, %~2, %~3]
!bat_log!
!bat_return!





:date2jdate JD YYYY MM DD -- converts a gregorian calender date to julian day format
rem !bat_pushlog! [CALL !bat_batchfile!:date2jdate] %1 %2 %3 %4 %5
	rem                        -- JD   [out] - julian days
	rem                        -- YYYY [in]  - gregorian year, i.e. 2006
	rem                        -- MM   [in]  - gregorian month, i.e. 12 for december
	rem                        -- DD   [in]  - gregorian day, i.e. 31
        call "!bat_exedir!\Date2JDate.bat" %1 %2 %3 %4

:date2jdate_END
rem !bat_poplog! [RETURN !bat_batchfile!:date2jdate] [%1=!%1!]
EXIT /b 0




:jdate2date JD YYYY MM DD -- converts julian days to gregorian date format
rem !bat_pushlog! [CALL !bat_batchfile!:jdate2date] %1 %2 %3 %4 %5
	rem                     -- JD   [in]  - julian days
	rem                     -- YYYY [out] - gregorian year, i.e. 2006
	rem                     -- MM   [out] - gregorian month, i.e. 12 for december
	rem                      -- DD   [out] - gregorian day, i.e. 31
        call "!bat_exedir!\JDate2Date.bat" %1 %2 %3 %4

:jdate2date_END
rem !bat_log! [RETURN !bat_batchfile!:jdate2date] [%1 %2=!%2!, %3=!%3!, %4=!%4!]
EXIT /b 0


:HELP
	echo.
	echo.Usage: call "bat_exedir\%~nx0"
	echo.
	echo.Returns:
	echo.  environment variables: ftp_hhmmssdd, ftp_hr, ftp_mi, ftp_se, ftp_jd
	echo.
	echo.The returned time "en-US" locale format where
	echo.the local time is returned as: hh:mm:ss
	echo.
	echo.Example:
	echo.  If the time is 10:39:57.956 AM on October 5, 2013, this routine should result with:
	echo.    ftp_hhmmssdd=07:15:22
	echo.    ftp_hr=07
	echo.    ftp_mi=15
	echo.    ftp_se=22
	echo.    ftp_jd=2456262      // Julian date
	echo.
	echo.This is the "en-US" local version of this script and it will serve
	echo.as a template for others to develop localized versions of this script.
	echo.
	echo.The trick is in the construct of the ENDLOCAL command, where it passes
	echo.back variables that are local to this routine.
	echo.
	echo.Example: 
	echo.   call "%~dpnx0" "01/10/2016 01:04:42 AM"
	echo.
	echo.  Results: 
	echo.   ftp_cams_zulu=2016_01_11_09_04_42
	echo.   ftp_8601_zulu=2016_01_11_09_04_42
	echo.   ftp_8601_local=2016_01_10_01_04_42-08:00
	echo.


	echo.   ftp_hhmmssdd=%ftp_hhmmssdd%
	echo.   ftp_hr
	echo.   ftp_mi
	echo.   ftp_se
	echo.   ftp_sed
	echo.   ftp_jd
	echo.   ftp_hhmmssdd_local
	echo.   ftp_cams_zulu
		
	echo.  REM from ParseDate1.bat
	echo.   ftp_wmdy
	echo.   ftp_year
	echo.   ftp_mo
	echo.   ftp_mo2
	echo.   ftp_da
	echo.   ftp_da2
	echo.   ftp_www
	echo.   bat_twilight_searchstr
	echo.   ftp_LocalDate
		
	echo.   ftp_cams_zulu
	echo.   ftp_8601_local
	echo.   ftp_8601_zulu
	echo.
	echo.



:DONE
	!bat_pop!
	!bat_log! [!date!, !time!] !bat_batchfile!:DONE] %~dpnx0 !*
	!bat_log! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	!bat_log!

		REM from ParseDate1.bat
rem !bat_echo! !ftp_wmictime_date! !ftp_wmictime_time!.!ftp_wmictime_usec!      !ftp_year!!ftp_mo!!ftp_da! !ftp_hr!!ftp_mi!!ftp_se!.!ftp_sed!

	ENDLOCAL & (
		SET ftp_hhmmssdd=%ftp_hhmmssdd%
		set ftp_hr=%ftp_hr%
		set ftp_mi=%ftp_mi%
		set ftp_se=%ftp_se%
		set ftp_sed=%ftp_sed%
		set ftp_jd=%ftp_JD%
		set ftp_hhmmssdd_local=%ftp_hhmmssdd_local%
		set ftp_cams_zulu=%ftp_cams_zulu%
		
		REM from ParseDate1.bat
		SET ftp_wmdy=%ftp_wmdy%
		set ftp_year=%ftp_year%
		set ftp_mo=%ftp_mo%
		set ftp_mo2=%ftp_mo2%
		set ftp_da=%ftp_da%
		set ftp_da2=%ftp_da2%
		set ftp_www=%ftp_www%
		set bat_twilight_searchstr=%bat_twilight_searchstr%
		set ftp_LocalDate=%ftp_LocalDate%
		
		set ftp_cams_zulu=%ftp_cams_zulu%
		set ftp_8601_local=%ftp_8601_local%
		set ftp_8601_zulu=%ftp_8601_zulu%
		set ftp_cams_local=%ftp_cams_local%
	)


exit /b 0

