@echo off
setlocal enableextensions enabledelayedexpansion


	REM #################################################################################
:CHECKHELP

	(echo.%*|findstr /L /i /c:"/help" /c:"/h" /c:"/?" >nul: 2>&1) && (goto :HELP)

:CHECKHELP_END
goto :SETUP %*



	REM #################################################################################
:SETUP
	call "%~dp0AutoCams.setup.bat" "%~dpnx0"
	!bat_CALL! "%~dp0indent_setup.bat"

	!bat_echolog! =============================================================
	!bat_echolog! %date% %time% - Version: !bat_vers! - [%~dpnx0 %*]
	!bat_echolog! [!bat_batchfile!:SETUP %*]  Please wait...
	!bat_pushecho!
	

	!bat_CALL! "!bat_exedir!\ReadParams.bat"

	!bat_CALL! "!bat_exedir!\ReadCams2Global.bat"
	
	set bat_returncode=0
	set /a bat_boards_max=!bat_CAMS2BOARDS.number_of_boards!-1

	!bat_WAIT! 3	"!bat_batchfile!:SETUP"


	call :PARSEARGS %*
	if !errorlevel! neq 0 (
		call :DELETABLE
		goto :DONE
	)

	call :MAIN %*

	!bat_log! start /low /min SCHTASKS /Run /tn Monitor_Poll_Client
	          start /low /min SCHTASKS /Run /tn Monitor_Poll_Client

goto :DONE



	REM #################################################################################
:MAIN
!bat_pushlog! [CALL !bat_batchfile!:MAIN] %*

		REM Loop through the Submitted directories of all the boards listed in the Cams2Global.ini file
	CALL :LOOPSUBMITTED
	if !errorlevel! neq 0 (goto :MAIN_END)

		REM Now switch to processing the queue...
		REM 
	call :LOOPQUEUE
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	call :COPY_CAL_FILES
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	call :LOOPZIPS
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	call :PREP_QUEUE "!bat_ZipFileNameFull!" "!bat_capturedfilesdir!" "!bat_caldir!" "!bat_cmdline!"
	if !errorlevel! neq 0 (goto :MAIN_END)
	
	call :UPLOAD_QUEUE
	set bat_returncode=!errorlevel!
	if !bat_returncode! neq 0 (
		goto :MAIN_END
	)

	call :CLEANUP
	if !errorlevel! neq 0 (goto :MAIN_END)

	


:MAIN_END
!bat_popecho! [RETURN !bat_batchfile!:MAIN] []
!bat_log!
!bat_return! !bat_returncode!






	REM #################################################################################
	REM Loops through the Submitted directories of all the boards listed in the Cams2Global.ini
	REM file.  
	REM 
	REM RETURNS:
	REM     Nothing
	REM 
:LOOPSUBMITTED
!bat_pushlog! [CALL !bat_batchfile!:LOOPSUBMITTED] %*

	set bat_returncode=1
	
	set /a bat_count_zips=0

	for /L %%a in (0,1,!bat_boards_max!) do (
	
			REM Move all the zip files and their MD5 files, if they have MD5 files, to the queue dir...
		CALL :MOVETOQUEUE %%a
		if !errorlevel! neq 0 (
				REM bat_count_zips is zero if there are no Submitted sets to move.
			set /a bat_count_zips+=1
		)
	)
	
		REM If bat_count_zips NEQ 0, that means that it found some Submitted sets to move
	if !bat_count_zips! NEQ 0 (
		set bat_returncode=0
	)
	
	
	rem By this point, there should be no eligible zip files to move from Submitted for any of the 
	rem boards.  There either weren't any or they were just moved by this routine.

:LOOPSUBMITTED_END
!bat_popecho! [RETURN !bat_batchfile!:LOOPSUBMITTED] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3	:LOOPSUBMITTED_END
!bat_return! !bat_returncode!





	REM #################################################################################
	REM Moves all the "*.zip" and "*.md5.txt" files that exist in the "Submitted" dir associated
	REM with the specified board, which is passed as the first argument.  
	REM
	REM It will only move "*.zip" files that have "*.md5.txt" files associated. 
	REM
	REM NOTE: Eventually, we will need to resolve what happens to zip files that just sit there
	REM       forever without associated "md5.txt" files. 
	REM 
	REM     
:MOVETOQUEUE
!bat_pushlog! [CALL !bat_batchfile!:MOVETOQUEUE] %*

	set bat_returncode=1

	set /a bat_board_no=%~1
	
		REM Create the queue dir if it doesn't exist...
	if not exist "!bat_CAMS2QUEUE.queue_dir!" (
		!bat_logexeclog! MD "!bat_CAMS2QUEUE.queue_dir!"
	)
	
		REM Move all the zip files and their MD5 files, if  they have MD5 files, to the queue dir...
	for /f "usebackq tokens=*" %%b in (`dir /b /a-d "!bat_CAMS2DIR.%~1!\Submitted\*.zip"`) do (
			rem ZIP FOUND
		if exist "%%b.md5.txt" (
				rem TARGET FOUND
			!bat_logexec! MOVE /Y "%%b"           "!bat_CAMS2QUEUE.queue_dir!"
			!bat_logexec! MOVE /Y "%%b.md5.txt"   "!bat_CAMS2QUEUE.queue_dir!"
		)
	)

:MOVETOQUEUE_END
!bat_popecho! [RETURN !bat_batchfile!:MOVETOQUEUE] [bat_returncode=!bat_returncode!, 
!bat_log! `                                          bat_camsdir=!bat_camsdir!,
!bat_log!
rem !bat_WAIT! 3	:MOVETOQUEUE_END
!bat_return! !bat_returncode!






	REM #################################################################################
	REM Loop through the queue dir [D:\cams2_queue] and start processing uploads with retries
	REM if necessary. 
	REM     
	REM RETURNS:
	REM     bat_returncode             
	REM     
	REM     
	REM     
:LOOPQUEUE
!bat_pushlog! [CALL !bat_batchfile!:LOOPQUEUE] %*

	set bat_returncode=0

	for /f "usebackq tokens=*" %%a in (`dir /b /a-d /on "*.zip"`) do (
		if exist "!bat_CAMS2QUEUE.queue_dir!\%%a.md5.txt" (
			call :PROCESSZIPSET "%%a"
		)
	)
	

:LOOPQUEUE_END
!bat_popecho! [RETURN !bat_batchfile!:LOOPQUEUE] [bat_returncode=!bat_returncode!, ]
!bat_log!
rem !bat_WAIT! 3	:LOOPQUEUE_END
!bat_return! !bat_returncode!





	REM #################################################################################
	REM Process this zip/md5 set and start the upload...
	REM 
	REM Needs to be flexible enough to support use of various other client tools, such as
	REM FTP, BitTorrent, FileZilla command line, winscp, etc. 
	REM     
	REM RETURNS:
	REM     bat_returncode                     if successful.
	REM     
:PROCESSZIPSET
!bat_pushlog! [CALL !bat_batchfile!:PROCESSZIPSET] %*

	rem Call an external script to do the upload.



:PROCESSZIPSET_END
!bat_popecho! [RETURN !bat_batchfile!:PROCESSZIPSET] [bat_returncode=!bat_returncode!
!bat_log! `                                         bat_ZipFileNameFull=!bat_ZipFileNameFull!]
!bat_log!
rem !bat_WAIT! 3	:PROCESSZIPSET_END
!bat_return! !bat_returncode!






	REM #################################################################################
	REM Fetch the list of ALL CALSTARS*.* files on the FTP server so we can perform 
	REM filtered searches using FINDSTR later.
	REM 
	REM Returns:
	REM    bat_ftpfile2			    a temp file containing a list of all the CALSTARS*.* 
	REM                             files on the server
	REM    
:GET_SERVER_FILE_LIST
!bat_pushlog! [CALL !bat_batchfile!:GET_SERVER_FILE_LIST] 

		REM Set the name of the file containing the server files list...
	!bat_logexeclog! set bat_ftpfile2=!bat_exedir!\ftp.temp_2.txt

		!bat_CALL! "!bat_exedir!\ReadRemoteConfig.bat"

			REM Now download a list of matching files on the server. 
		rem  site=bat_calftpsite
		!bat_log! echo.user !bat_calftpuser! } "!bat_ftpfile!"
		
		echo.user !bat_calftpuser!>"!bat_ftpfile!"
		echo.!bat_calftppassword!>>"!bat_ftpfile!"
		echo.bin>>"!bat_ftpfile!"
		echo.cd !bat_calftpdir!>>"!bat_ftpfile!"
		echo.literal pasv>>"!bat_ftpfile!"
		echo.lcd "!bat_calstarsdir!">>!bat_ftpfile!"
		echo.literal pasv>>"!bat_ftpfile!"
		echo.ls "CALSTARS??????_*.*">>!bat_ftpfile!"
		echo.quit>>"!bat_ftpfile!"

		!bat_echolog! CALL FTP -n -s:"!bat_ftpfile!" !bat_calftpsite!][FINDSTR /i /r /c:"^CALSTARS......_.*" } "!bat_ftpfile2!"
		              CALL FTP -n -s:"!bat_ftpfile!" !bat_calftpsite! |FINDSTR /i /r /c:"^CALSTARS......_.*">"!bat_ftpfile2!"
		set bat_returncode=!errorlevel!
	
			REM CALSTARS000257_2017_09_15_03_00_00.txt
		!bat_echolog! SORT "!bat_ftpfile2!" ][ SORT /+14 }"!bat_ftpfile2!.bak"
		              SORT "!bat_ftpfile2!"  | SORT /+14 >"!bat_ftpfile2!.bak"
		!bat_logexeclog! MOVE /y "!bat_ftpfile2!.bak"  "!bat_ftpfile2!"
	
		!bat_logexec! TYPE "!bat_ftpfile2!"
		set bat_returncode=!errorlevel!

:GET_SERVER_FILE_LIST_END
!bat_popecho! [RETURN !bat_batchfile!:GET_SERVER_FILE_LIST] [bat_returncode=!bat_returncode!, bat_ftpfile2=!bat_ftpfile2!]
!bat_log!
rem !bat_WAIT! 3	:GET_SERVER_FILE_LIST_END
!bat_return! !bat_returncode!







	REM #################################################################################
	REM Gets the next sequence number after the last CALSTARS*.zip file that already
	REM exists on the server.  
	REM 
	REM RETURNS:
	REM     bat_ZipSeq              NUMERIC value without leading "0" as the next one in 
	REM                             sequence and it is used in the creation of the new 
	REM                             Zip file.
	REM 
	REM     bat_TempCalFromZipSeq   TEXT version of the sequnce value with leading "0". 
	REM 
	REM This needs to change so that it lists files on the FTP server...
	REM
:GET_ZIP_SEQ [IN "!bat_TempCalFromZipName!"]
!bat_pushlog! [CALL !bat_batchfile!:GET_ZIP_SEQ] %*

		REM This section lists the already existing zip files on the server
		REM to determine what the last file in sequence is.  Then establish the next one in
		REM sequence afte that.  Reading them to get the current sequence should be another 
		REM routine.
		REM 
		REM FINDSTR /i /r /c:"CALSTARS000750_2016_12_31_14_37_59_...zip" "ftp_temp_6.txt"
		REM           to get: CALSTARS000750_2016_12_31_14_37_59_01.zip

	set bat_tempzip=
	set bat_tempzipname=
	
		REM Filter by the given CALSTARS filename [example: CALSTARS0750_2016_12_31_14_37_59.txt]
		REM If there is more than one matching zip file, then we can determine the sequence number by 
		REM incrementing the sequence number
		
	!bat_pushlog! for /f "usebackq tokens=*" ..a in ^(`FINDSTR /i /r /c:"%~1_...zip" "!bat_ftpfile2!"`^) do ^(
	for               /f "usebackq tokens=*" %%a in  (`FINDSTR /i /r /c:"%~1_...zip" "!bat_ftpfile2!"`) do (
		set bat_tempzip=%%a
		set bat_tempzipname=%%~na
	)
	!bat_poplog! ^)
	
	if not defined bat_tempzipname (
		set /a bat_ZipSeq=0
	) else (
		set /a bat_ZipSeq=!bat_tempzipname:~-2!
	)
	
	set /a bat_ZipSeq+=1
	
	set bat_TempCalFromZipSeq=00!bat_ZipSeq!
	set bat_TempCalFromZipSeq=!bat_TempCalFromZipSeq:~-2!
	

:GET_ZIP_SEQ_END
	set bat_tempzip=
	set bat_tempzipname=
!bat_popecho! [RETURN !bat_batchfile!:GET_ZIP_SEQ] [bat_ZipSeq=!bat_ZipSeq!, bat_TempCalFromZipSeq=!bat_TempCalFromZipSeq!]
!bat_log!
rem !bat_WAIT! 3	:GET_ZIP_SEQ_END
!bat_return!





	REM #################################################################################
	REM This routine creates the MD5 hash flag file that will be uploaded alongside
	REM the CALSTARS*.zip file. 
	REM 
	REM RETURNS:
	REM     bat_TempMD5Dir               C:\CAMS\CapturedFiles\2016_12_31_14_37_59
	REM     bat_TempMD5Name              CALASTARS0750_2016_12_31_14_37_59_01_md5
	REM     bat_TempMD5File              CALASTARS0750_2016_12_31_14_37_59_01_md5.txt
	REM     bat_TempMD5Full              C:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALASTARS0750_2016_12_31_14_37_59_01_md5.txt
	REM     
:CREATE_MD5_FILE [ IN "!bat_TempCalFromZipName!"]
!bat_pushlog! [CALL !bat_batchfile!:CREATE_MD5_FILE] 

	                 set bat_TempMD5Dir=%~dp1::
	                 set bat_TempMD5Dir=!bat_TempMD5Dir:\::=!
	!bat_logexeclog! set bat_TempMD5Dir=!bat_TempMD5Dir::::=!
	
	!bat_logexeclog! set bat_TempMD5Name=%~n1_md5
	!bat_logexeclog! set bat_TempMD5File=%~n1_md5.txt
	!bat_logexeclog! set bat_TempMD5Full=%~dp1!bat_TempMD5File!

		REM Make the MD5 file now of the zip file [%~1]...
	!bat_CALL! "!bat_exedir!\md5.exe" -o"!bat_TempMD5Full!" "%~1"
	set bat_returncode=!errorlevel!


:CREATE_MD5_FILE_END
!bat_popecho! [RETURN !bat_batchfile!:CREATE_MD5_FILE] [bat_returncode=!bat_returncode!,
!bat_log! `                                         bat_TempMD5Dir=!bat_TempMD5Dir!,
!bat_log! `                                         bat_TempMD5Name=!bat_TempMD5Name!,
!bat_log! `                                         bat_TempMD5File=!bat_TempMD5File!,
!bat_log! `                                         bat_TempMD5Full=!bat_TempMD5Full!,
!bat_log! `                                         zip file=%~1]
!bat_log!
rem !bat_WAIT! 3	:CREATE_MD5_FILE_END
!bat_return! !bat_returncode!








	REM #################################################################################
	REM Updates the queue file with a queue entry containing the information required to
	REM re-incorporate the results into the capture session.  
	REM 
	REM Also copies the zip file that is uploaded to the AutoCal_Remote_Queue dir. 
	REM 
:PREP_QUEUE [IN "bat_ZipFileNameFull" "bat_capturedfilesdir" "bat_caldir" "bat_autocal_continue_cmdline"]
!bat_pushlog! [CALL !bat_batchfile!:PREP_QUEUE] [%1, %2, %3, %4]

		REM Make the queue entry to the local queue [bat_autocal_remote_queue] so it can be compared later
	!bat_logexec! echo.%~1,%~2,%~3,%4}}"!bat_autocal_remote_queue!"
	              echo.%~1,%~2,%~3,%4>>"!bat_autocal_remote_queue!"

		REM Move the zip file to the queue folder on the server
	!bat_logexeclog! MOVE /y "!bat_ZipFileNameFull!" "!bat_autocal_remote_queue_dir!\"
	!bat_logexeclog! set bat_ZipFileNameFull=!bat_autocal_remote_queue_dir!\!bat_TempCalFromZipFile!
	
		REM Move the _md5.txt file to the queue folder on the server
	!bat_logexeclog! MOVE /y "!bat_TempMD5Full!" "!bat_autocal_remote_queue_dir!\"
	!bat_logexeclog! set bat_TempMD5Full=!bat_autocal_remote_queue_dir!\!bat_TempMD5File!
	!bat_logexeclog! set bat_TempMD5Dir=!bat_autocal_remote_queue_dir!
	
	!bat_logexeclog! RD /S /Q "%~dp1CALSTARS"
	
:PREP_QUEUE_END
!bat_popecho! [RETURN !bat_batchfile!:PREP_QUEUE] [bat_returncode=!bat_returncode!
!bat_log! `                                         bat_ZipFileNameFull=!bat_ZipFileNameFull!
!bat_log! `                                         bat_TempMD5Full=!bat_TempMD5Full!]
!bat_log!
rem !bat_WAIT! 3	:PREP_QUEUE_END
!bat_return!




	REM #################################################################################
	REM This script is responsible for mapping a drive letter to the server/share where
	REM the file will be uploaded.  This could be a mapped share or ftp. However,
	REM to download, another share will need to be used.  It might be that we need to
	REM write a custom script to call for the upload...
	REM 
	REM Should use "!bat_exedir!\!bat_CALIBRATION.REMOTE_config_file!"
	REM
	REM RETURNS:
	REM    bat_mapped_From              Location of the local server cache dir. 
	REM 
:MAP_DRIVE
!bat_pushlog! [CALL !bat_batchfile!:MAP_DRIVE] %*

	set bat_returncode=0

	!bat_echolog! net use m: /d
	!bat_echolog! net use m: \\server\uploadFROMshare /user:user password /persistent:no
	!bat_echolog! net use N: /d
	!bat_echolog! net use N: \\server\uploadRESULTSshare /user:user password /persistent:no
	!bat_echolog! set bat_returncode=!errorlevel!

	REM Check to ensure that you have a valid drive mapping...
	
	REM until we fix this, hard-code the connection to test locally...
	!bat_logexeclog! set bat_mapped_From=!bat_CALIBRATION.REMOTE_server_dir!
	!bat_logexeclog! set bat_mapped_Results=!bat_CALIBRATION.REMOTE_server_dir!

:MAP_DRIVE_END
!bat_popecho! [RETURN !bat_batchfile!:MAP_DRIVE] [bat_returncode=!bat_returncode!, bat_mapped_From=!bat_mapped_From!, bat_mapped_Results=!bat_mapped_Results!]
!bat_log!
rem !bat_WAIT! 3	:MAP_DRIVE_END
!bat_return!




	REM #################################################################################
	REM 
	REM RETURNS:
	REM     bat_calftpsite
	REM     bat_calftpuser
	REM     bat_calftppassword
	REM     bat_calftpdir
	REM 
	REM     bat_zipfile
	REM     bat_ftpfile
	REM     
:UPLOAD_ZIP [IN bat_ZipFileNameFull]
!bat_pushlog! [CALL !bat_batchfile!:UPLOAD_ZIP] %*

rem	!bat_logexeclog! COPY /y "!bat_autocal_remote_queue_dir!\!bat_TempCalFromZipName!!bat_TempCalFromZipExt!" "!bat_mapped_From!"
rem	set bat_returncode=!errorlevel!
	
	!bat_CALL! "!bat_exedir!\ReadRemoteConfig.bat"
	

		REM Set these variables so that the ftp_transfer_seti.bat script can use them.
	!bat_logexeclog! set bat_ftpsite=!bat_calftpsite!
	!bat_logexeclog! set bat_ftpuser=!bat_calftpuser!
	!bat_logexeclog! set bat_ftppassword=!bat_calftppassword!
	!bat_logexeclog! set bat_ftpdir=!bat_calftpdir!
	
		REM We use bat_zipfile instead of bat_ZipFileNameFull to be compatible with the ftp_transfer_seti.bat script. 
	!bat_logexeclog! set bat_zipfile=%~1

		REM Upload the ZIP file and perform all the normal md5 and other validation checks
	!bat_log!  "!bat_exedir!\ftp_transfer_seti.bat" FTPTRANSFER "!bat_zipfile!"
	!bat_CALL! "!bat_exedir!\ftp_transfer_seti.bat" FTPTRANSFER "!bat_zipfile!"
	
		REM Move the zip file to the queue folder on the server
rem	!bat_logexeclog! MOVE /y "!bat_ZipFileNameFull!" "!bat_autocal_remote_queue_dir!\"
rem	!bat_logexeclog! set bat_ZipFileNameFull=!bat_autocal_remote_queue_dir!\!bat_TempCalFromZipFile!
	
	
	!bat_echolog!
	!bat_echolog! Now build the "!bat_ftpfile!" FTP command file for uploading the MD5 file to the FTP server.
	!bat_echolog!
	
		REM Now upload the matching MD5 flag file...
	echo.user !bat_calftpuser!>"!bat_ftpfile!"
	echo.!bat_calftppassword!>>"!bat_ftpfile!"
	echo.bin>>"!bat_ftpfile!"
	echo.cd !bat_calftpdir!>>"!bat_ftpfile!"
	echo.literal pasv>>"!bat_ftpfile!"
	echo.lcd "!bat_TempMD5Dir!">>!bat_ftpfile!"
	echo.literal pasv>>"!bat_ftpfile!"
	echo.put "!bat_TempMD5File!">>!bat_ftpfile!"
	echo.quit>>"!bat_ftpfile!"

	!bat_logexec! type "!bat_ftpfile!
	!bat_echolog!
	
	!bat_logexec! dir "!bat_TempMD5Full!"
	!bat_echolog!
	
		REM Now upload the MD5 flag file
	!bat_logexec! CALL FTP -n -s:"!bat_ftpfile!" !bat_calftpsite!
	set bat_returncode=!errorlevel!



:UPLOAD_ZIP_END
!bat_popecho! [RETURN !bat_batchfile!:UPLOAD_ZIP] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3	:UPLOAD_ZIP_END
!bat_return!









	REM #################################################################################
	REM This routine cleans up all the temp files and stuff before exiting the program.
	REM I need to figure out a way to handle the case when the network is down and things
	REM can be processed again. Also, I need to figure out a way for the 
	REM AutoCal_LocalDownload.bat script to reinsert itself into the autocams autonomous 
	REM processing mid-stream after AZ_AUTOCAL. 
	REM 
:CLEANUP
!bat_pushlog! [CALL !bat_batchfile!:CLEANUP] %*



:CLEANUP_END
!bat_popecho! [RETURN !bat_batchfile!:CLEANUP] [!bat_getdirname!, !bat_mainspec!]
!bat_log!
rem !bat_WAIT! 3	:CLEANUP_END
!bat_return!




	REM ############################################################################################
	REM This routine might make a good template for scanning for matching CALSTARS files.
	REM
:PROCESS_REMAINING_QUEUE
!bat_pushlog! [CALL !bat_batchfile!:PROCESS_REMAINING_QUEUE] %*


:PROCESS_REMAINING_QUEUE_LOOP
	!bat_CALL! "!bat_exedir!\GetFileCount.bat" "!bat_TempCalFrom!\CALSTARS???????_????_??_??_??_??_??.zip"
	set bat_CountFrom=!errorlevel!
	
	if not defined bat_CountFrom (
		goto :PROCESS_REMAINING_QUEUE_END
	)

	if !bat_CountFrom! LSS 1 (
		goto :PROCESS_REMAINING_QUEUE_END
	)
	
	!bat_echolog!
	!bat_echolog! ######################################################
	!bat_echolog! There are "!bat_CountFrom!" files to process...
	!bat_echolog!
	
	set bat_TempCalFromZipFull=
	set bat_capturedfilesdir=
	set bat_TempCalFromZipName=
	set bat_TempCalFromZipExt=
rem	set bat_TempCalFrom=!bat_TempCalFrom!
	set bat_captureddate=
	set bat_capturedtime=
	set bat_captureddatetime=
	set bat_calstarsdir=
	set bat_capturedfilesdir=
	set bat_archivedfilesdir=
	set bat_TempCalResults=





	
		REM Loop through the remaining files and skip all but the oldest one...
	!bat_pushlog! for /f "usebackq tokens=*" ..a in ^(`dir "!bat_TempCalFrom!\CALSTARS???????_????_??_??_??_??_??.zip" /a-d /b /o-d`^) do ^(
	for               /f "usebackq tokens=*" %%a in  (`dir "!bat_TempCalFrom!\CALSTARS???????_????_??_??_??_??_??.zip" /a-d /b /o-d`) do (
		!bat_logexeclog! set bat_TempCalFromZipFull=!bat_TempCalFrom!\%%a
	)
	!bat_poplog! ^)
	

	
	call :PARSEZIP "!bat_TempCalFromZipFull!"

	call :MAIN

	
		REM Go back to the counter section in case new files have arrived...
		REM If we had used a FOR loop, it would not accommodate new files arriving
		REM after the loop started.
	goto :PROCESS_REMAINING_QUEUE_LOOP

	

:PROCESS_REMAINING_QUEUE_END
!bat_popecho! [RETURN !bat_batchfile!:PROCESS_REMAINING_QUEUE] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3	:PROCESS_REMAINING_QUEUE_END
!bat_return!



	REM ########################################################################
	REM This routine re-launches the MonitorDirs.exe program to begin monitoring for new
	REM uploads.  This script should be used to launch the monitor in the first place
	REM just in case new files were uploaded while the monitor was not running. 
	REM 
:RELAUNCH_MONITOR
!bat_pushlog! [CALL !bat_batchfile!:RELAUNCH_MONITOR] %*

	process | Find /i "monitordir.exe"
	if !errorlevel! equ 0 (
		goto :RELAUNCH_MONITOR_END
	)

	!bat_echolog! Relaunching the MonitorDir.exe program...
	!bat_echolog!
	
	!bat_echolog! !bat_CALIBRATION.REMOTE_autocal_localupload_cmdline!
	              !bat_CALIBRATION.REMOTE_autocal_localupload_cmdline!
	

:RELAUNCH_MONITOR_END
!bat_popecho! [RETURN !bat_batchfile!:RELAUNCH_MONITOR] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3	:RELAUNCH_MONITOR_END
!bat_return!




	REM ########################################################################
	REM 
:DELETABLE
!bat_pushlog! [CALL !bat_batchfile!:DELETABLE]


	!bat_logexeclog! set bat_autocal_remote_queue_dir=!bat_camsdir!\AutoCal_Remote_Queue
	!bat_logexeclog! set bat_autocal_remote_queue=!bat_autocal_remote_queue_dir!\AutoCal_Remote_Queue.txt
	
	!bat_echolog! bat_TempCalFromZipName=!bat_TempCalFromZipName!
	
		REM Convert: "CALSTARS0000750_2016_12_31_14_37_59_01_md5.txt" 
		REM to:      "CALSTARS0000750_2016_12_31_14_37_59_01_Results.zip"
		REM    
	!bat_logexeclog! set bat_TempCalResultsZipName=!bat_TempMD5Name:_md5.txt=_Results.zip!
	
		REM Convert: "CALSTARS0000750_2016_12_31_14_37_59_01_md5.txt" 
		REM to:      "CALSTARS0000750_2016_12_31_14_37_59_01_Results.txt"
		REM 
	!bat_logexeclog! set bat_TempCalResultsMD5Name=!bat_TempMD5Name:_md5.txt=_Results.txt!
	


	REM ====================================================================================
	REM creating the _deletable.txt file
	REM 
	
		REM Convert: "CALSTARS0000750_2016_12_31_14_37_59_01_Results" 
		REM to:      "CALSTARS0000750_2016_12_31_14_37_59_01_Deletable.txt"
		REM 
	!bat_logexeclog! set bat_DeleteFileName=!bat_TempCalFromZipName:_Results=_Deletable.txt!
	!bat_logexeclog! set bat_DeleteFileFull=!bat_CalStarsDir!\!bat_TempCalFromZipName:_Results=_Deletable.txt!
	
	echo."!bat_TempCalFromZipName!">"!bat_DeleteFileFull!"
	echo."!bat_TempMD5Name!">>"!bat_DeleteFileFull!"
	echo."!bat_TempCalResultsZipName!">>"!bat_DeleteFileFull!"
	echo."!bat_TempCalResultsMD5Name!">>"!bat_DeleteFileFull!"
	echo."!bat_DeleteFileName!">>"!bat_DeleteFileFull!"

		!bat_log! `    Note: Building the "..\ftp.temp.txt" file...

			REM Put _Deletable.txt on the ftp
		echo.user !bat_calftpuser!>"!bat_ftpfile!"
		echo.!bat_calftppassword!>>"!bat_ftpfile!"
		echo.bin>>"!bat_ftpfile!"
		echo.cd !bat_calftpdir!>>"!bat_ftpfile!"
		echo.literal pasv>>"!bat_ftpfile!"
		echo.lcd "!bat_CalStarsDir!">>!bat_ftpfile!"
		echo.literal pasv>>"!bat_ftpfile!"
		echo.put "!bat_DeleteFileName!">>"!bat_ftpfile!"
		echo.quit>>"!bat_ftpfile!"

		!bat_logexec! TYPE "!bat_ftpfile!"

		!bat_logexec! call FTP -n -s:"!bat_ftpfile!" "!bat_calftpsite!"
		set bat_returncode=!bat_logexec_returncode!

		!bat_echolog!
		!bat_echolog! '    FTP exit code=!bat_returncode!
		!bat_echolog!

:DELETABLE_END
!bat_popecho! [RETURN !bat_batchfile!:CLEANUP] [bat_returncode=!bat_returncode!]
!bat_log!
rem !bat_WAIT! 3
!bat_return!





	REM #################################################################################
	REM This script will be called like this:
	REM 
	REM   "Submitted_LocalUpload.bat" /capturedir "^<bat_capturedfilesdir^>"
	REM
	REM
:PARSEARGS
!bat_pushlog! [CALL !bat_batchfile!:PARSEARGS] %*

	set bat_returncode=0
	goto :PARSEARGS_END
	
	
	
		REM Set the DEFAULT working dirs
	!bat_logexeclog! set bat_capturedfilesdir=
	!bat_logexeclog! set bat_captureddirname=
	!bat_logexeclog! set bat_caldir=
	!bat_logexeclog! set bat_camera=&	rem 
	!bat_logexeclog! set bat_archivedfilesdir=


		REM Set default for bat_calstarsdir.  bat_calstarsdir is the base
		REM directory for all the work is done in this routine.
		REM
	!bat_logexeclog! set bat_calstarsdir=

		REM ==================================================================================
		REM Set defaults that might be missing from autocams.params.ini
		REM
	if not defined bat_CALIBRATION.REMOTE_skip (
	
			REM Set the default for FTP_CalStarsExtraction.exe to skip at least 5 minutes 
			REM between sampled FF files [71 for NTSC and 59 for PAL]
			REM 5 minutes in the equation below is 305 seconds.  The extra 5 seconds 
			REM helps round it up to the next second.
			REM
		set /a bat_CALIBRATION.REMOTE_skip=305 * !bat_CAPTURE.fps! / 25600
		!bat_logexeclog! set bat_CALIBRATION.REMOTE_skip=71
	)

	if not defined bat_CALIBRATION.REMOTE_nstars (
			REM Set the number of stars to include in the CALSTARS and reconsituted FF files at 300
		!bat_logexeclog! set bat_CALIBRATION.REMOTE_nstars=300
	)

	if not defined bat_CALIBRATION.REMOTE_autocal_enabled (
		set bat_CALIBRATION.REMOTE_autocal_enabled=true
	)

	if not defined bat_CALIBRATION.REMOTE_autocal_remote_cmdline (
		!bat_logexeclog! set bat_CALIBRATION.remote_autocal_remote_cmdline=start "" /low /min "!bat_exedir!\MonitorDir.exe"        /FilenameChange /RunOnce /AddFileName /Creation /Activate     "!bat_calstarsdir!" "!bat_exedir!\AutoCal_Remote.bat /zip"
	)

	if not defined remote_autocal_localupload_cmdline (
		!bat_logexeclog! set bat_CALIBRATION.REMOTE_autocal_localupload_cmdline=start "" /low /min "!bat_exedir!\monitordir.exe"   /FilenameChange /RunOnce /AddFileName /Creation /Activate     "!bat_calstarsdir!" "!bat_exedir!\AutoCal_LocalDownload.bat /zip"
	)

		REM This config file contains details of connecting to a remote autocal server
	if not defined bat_CALIBRATION.REMOTE_config_file (
		!bat_logexeclog! set bat_CALIBRATION.REMOTE_config_file=AutoCal_Remote_config.txt
	)

	

:PARSEARGS_LOOP
rem echo.:PARSEARGS_LOOP [%1] [%2]

	if /i "%~1" equ "" (
		goto :PARSEARGS_LOOPBREAK
	)
	
	if /i "%~1" equ "/capturedir" (
		!bat_logexeclog! set bat_capturedfilesdir=%~2

		shift
		shift
		goto :PARSEARGS_LOOP
	)

	if /i "%~1" equ "/caldir" (
		!bat_logexeclog! set bat_caldir=%~2
		shift
		shift
		goto :PARSEARGS_LOOP

	)
	
	if /i "%~1" equ "/detect" (
		!bat_logexeclog! set bat_detectfile=%~2
		shift
		shift
		goto :PARSEARGS_LOOP

	)
	
		REM /session can be used when the bat_capturedfilesdir does not have a dirname that actually is a CAMS date
	if /i "%~1" equ "/session" (
		!bat_logexeclog! set bat_captureddirname=%~2
		shift
		shift
		goto :PARSEARGS_LOOP

	)
	

:PARSEARGS_LOOPBREAK
!bat_log! [CALL !bat_batchfile!:PARSEARGS_LOOPBREAK] %*


		REM bat_capturedfilesdir should be a date_time in the format:
		REM    "C:\CAMS\CapturedFiles\2016_01_26_14_36_59"
		REM      or 
		REM    "C:\CAMS\SubmissionFiles\750\CapturedFiles\2016_01_26_14_36_59"
		REM    
	if not defined bat_capturedfilesdir (
		set bat_returncode=1
		!bat_echolog!
		!bat_echolog!
		!bat_echolog! `    ERROR [!bat_returncode!] - bat_capturedfilesdir is not defined.
		!bat_echolog!
		!bat_echolog!
		!bat_WAIT! 10
		goto :PARSEARGS_END
	)
	
	
	if not defined bat_captureddirname (
		!bat_logexeclog! set bat_captureddirname=!bat_capturedfilesdir:~-19!
	)

	!bat_logexeclog! set bat_captureddate=!bat_captureddirname:~0,10!
	!bat_logexeclog! set bat_capturedtime=!bat_captureddirname:~-8!
	!bat_logexeclog! set bat_captureddatetime=!bat_captureddate!_!bat_capturedtime!
	
	!bat_logexeclog! set bat_archivedfilesdir=!bat_capturedfilesdir:CapturedFiles=ArchivedFiles!

	!bat_CALL! "!bat_exedir!\GetDetectFile.bat" bat_detectfile "!bat_archivedfilesdir!" "!bat_capturedfilesdir!"
	!bat_TITLE!

		REM Set up the variables for the working directories
	!bat_logexeclog! set bat_calstarsdir=!bat_capturedfilesdir!\CALSTARS
	!bat_logexeclog! set bat_calstars_capturedfilesdir=!bat_calstarsdir!\CapturedFiles\!bat_captureddatetime!
	!bat_logexeclog! set bat_calstars_archivedfilesdir=!bat_calstarsdir!\ArchivedFiles\!bat_captureddatetime!
	!bat_logexeclog! set bat_calstars_caldir=!bat_calstarsdir!\Cal

	if not defined bat_capturedfilesdir (
		set bat_returncode=2
		!bat_echolog!
		!bat_echolog! ERROR - Unable to preceed because /capturedir argument is missing.
		!bat_echolog!
		goto :PARSEARGS_END
	)
	
	if not defined bat_caldir (
		set bat_returncode=3
		!bat_echolog!
		!bat_echolog! ERROR - Unable to preceed because /caldir argument is missing.
		!bat_echolog!
		goto :PARSEARGS_END
	)
	

	!bat_logexeclog! set bat_DirListingCalStars=!bat_calstars_capturedfilesdir!\DirListingCalstars.txt"

	!bat_logexeclog! set bat_GCFC_tempfile=!bat_capturedfilesdir!\GCFC.log
	
	!bat_logexeclog! set bat_autocal_remote_queue_dir=!bat_camsdir!\AutoCal_Remote_Queue
	!bat_logexeclog! set bat_autocal_remote_queue=!bat_autocal_remote_queue_dir!\AutoCal_Remote_Queue.txt
	!bat_logexeclog! set bat_ftp_tempfile5=!bat_camsdir!\ftp.temp_queue.txt

	!bat_logexeclog! set bat_ftpfile=!bat_exedir!\ftp.temp.txt



:PARSEARGS_END
!bat_poplog! [RETURN !bat_batchfile!:PARSEARGS] [bat_returncode=!bat_returncode!,
!bat_log! `                                      bat_capturedfilesdir=!bat_capturedfilesdir!,
!bat_log! `                                      bat_caldir=!bat_caldir!,
!bat_log! `                                      bat_captureddatetime=!bat_captureddatetime!,
!bat_log! `                                      bat_capturedfilesdir=!bat_capturedfilesdir!,
!bat_log! `                                      bat_archivedfilesdir=!bat_archivedfilesdir!,
!bat_log! `                                      bat_detectfile=!bat_detectfile!,
!bat_log! `                                      bat_autocal_remote_queue_dir=!bat_autocal_remote_queue_dir!,
!bat_log! `                                      bat_autocal_remote_queue=!bat_autocal_remote_queue_dir!\AutoCal_Remote_Queue.txt
!bat_log! `                                      bat_calstarsdir=!bat_calstarsdir!,
!bat_log! `                                      bat_calstars_capturedfilesdir=!bat_calstars_capturedfilesdir!,
!bat_log! `                                      bat_calstars_archivedfilesdir=!bat_calstars_archivedfilesdir!,
!bat_log! `                                      bat_calstars_caldir=!bat_calstars_caldir!,
!bat_log! `                                      bat_DirListingCalStars=!bat_DirListingCalStars!,
!bat_log! `                                     ]
!bat_log!
rem !bat_WAIT! 3	:PARSEARGS_END
!bat_return! !bat_returncode!

:GET_QUEUE_ITEMS
!bat_pushlog! [CALL !bat_batchfile!:GET_QUEUE_ITEMS] [%1, %2, %3]

			REM making downloadlist from queue

			REM If ftp.temp_queue.txt already exists, delete it...
		if exist "!bat_ftp_tempfile5!" (
			!bat_logexeclog! DEL /q "!bat_ftp_tempfile5!"
		)

			REM The first field in the queue is the original zip file [without the _Results suffix] 
			REM and save all the queue entries into a "ftp.temp_queue.txt" file.  That file 
			REM contains the name of all the "*_Results.zip" files to search for. 
			REM 
		!bat_pushlog! for /f "usebackq tokens=1* delims=," ..A in ^("!bat_autocal_remote_queue!"^) do ^(
		for               /f "usebackq tokens=1* delims=," %%A in  ("!bat_autocal_remote_queue!") do (
		  set bat_temp=%%~dpnA
		  !bat_logexeclog! set bat_search=!bat_temp!.zip
		  echo.!bat_search!>>!bat_ftp_tempfile5!
		)
		!bat_poplog! ^)

			REM Output the resulting list of "_Results.zip" files to process...
		!bat_logexec! TYPE "!bat_ftp_tempfile5!"
		set bat_returncode=!errorlevel!
		

:GET_QUEUE_ITEMS_END
!bat_poplog! [RETURN !bat_batchfile!:GET_QUEUE_ITEMS] [!bat_returncode!]
!bat_log!
!bat_WAIT! 3	:GET_QUEUE_ITEMS_END
!bat_return! !bat_returncode!




	REM ######################################################################
	REM
:FILTER_LIST
!bat_pushlog! [CALL !bat_batchfile!:FILTER_LIST] [%1, %2, %3]




		REM Now iterate the list of source files and exclude those with ".zip" for the master list of zip files.  Some of those may
		REM need to be excluded because of the presence of matching "_Results" files. 
		REM 
		REM For each "_Results.zip" file found on the server, 
		REM
	!bat_pushlog! for /f "usebackq tokens=*" ..a in ^(`FINDSTR /i /r /c:"CALSTARS......_...._.._.._.._.._.._..\.zip" "!bat_ftp_tempfile5!"`^) do ^(
	for               /f "usebackq tokens=*" %%a in  (`FINDSTR /i /r /c:"CALSTARS......_...._.._.._.._.._.._..\.zip" "!bat_ftp_tempfile5!"`) do (
		!bat_pushlog! [[[[ %%~na ]]]]

			REM	c:\CAMS\CapturedFiles\2016_12_31_14_37_59\CALSTARS000750_2016_12_31_14_37_59_01.zip
		!bat_logexeclog! set bat_capturedzipfile=%%a

						 set bat_capturedfilesdir=%%~dpa::
						 set bat_capturedfilesdir=!bat_capturedfilesdir:\::=!
		!bat_logexeclog! set bat_capturedfilesdir=!bat_capturedfilesdir:::=!

			REM	C:\CAMS\Temp\CALSTARS000750_2016_12_31_14_37_59_01.zip
		!bat_logexeclog! set bat_tempzipfile=!bat_ftp_tempdir!\%%~nxa
		!bat_logexeclog! set bat_tempzipname=%%~na&	rem	CALSTARS000750_2016_12_31_14_37_59_01

		
			REM	CALSTARS0750_2016_12_31_14_37_59_01.zip
		!bat_logexeclog! set bat_ZipFileNameFull=!bat_autocal_remote_queue_dir!\!bat_tempzipname!.zip
		
		
		
			REM Convert: "CALSTARS000750_2016_12_31_14_37_59_01" 
			REM to:      "CALSTARS000750_2016_12_31_14_37_59_01.txt"
			REM 
		rem !bat_logexeclog! set bat_resultsflagfile=!bat_tempzipname:=_Results.txt!

			REM Convert: "CALSTARS000750_2016_12_31_14_37_59_01" 
			REM to:      "CALSTARS000750_2016_12_31_14_37_59_01_downloaded.txt"
			REM 
		rem !bat_logexeclog! set bat_downloadedflagfile=!bat_tempzipname:_Results=_Deletable.txt!



			REM First, check if there is an MD5 file [which is named as "CALSTARS0*_Results.txt"] to see if it's finished uploading. 
			REM
			REM Only download if there is [A] a "*_Results.txt" flag file and [B] if there is NOT a "*_downloaded.txt" flag file.

		!bat_log!
		!bat_logexeclog! FINDSTR /r /i /c:"!bat_tempzipname!.zip" "!bat_ftpfile2!"
		if /i "!errorlevel!" NEQ "0" (

			!bat_log! **** UPLOADING no ".zip" file on the server: "!bat_ZipFileNameFull!"...
			!bat_log!
			
			call :UPLOAD_ZIP "!bat_ZipFileNameFull!"
			set bat_returncode=!errorlevel!
			if !bat_returncode! neq 0 (
				goto :DELETABLE
				
			)

		) else (

			!bat_log! **** NO UPLOADING ".zip" file already on the server: "!bat_ZipFileNameFull!"...
			!bat_log!
				
			
			
		)

		!bat_poplog!............................
	)
	!bat_poplog! ^) end of: for /f "usebackq tokens=*" ..a in ^(`FINDSTR /i /r /c:"CALSTARS......_...._.._.._.._.._.._../.zip" "!bat_ftpfile2!"`^) 


:FILTER_LIST_END
!bat_poplog! [RETURN !bat_batchfile!:FILTER_LIST] [!bat_returncode!]
!bat_log!
!bat_WAIT! 3	:FILTER_LIST_END
!bat_return! !bat_returncode!



	REM ########################################################################
	REM 
:UPLOAD_QUEUE
 !bat_log! [CALL !bat_batchfile!:UPLOAD_QUEUE] %*
  
   CALL :GET_QUEUE_ITEMS
   
   CALL :FILTER_LIST



:UPLOAD_QUEUE_END
!bat_poplog! [RETURN !bat_batchfile!:UPLOAD_QUEUE] [bat_returncode=!bat_returncode!]
!bat_log!
!bat_return! !bat_returncode!


	REM ########################################################################
	REM 
:ERROR [IN bat_returncode IN "error string"]
!bat_log! [CALL !bat_batchfile!:ERROR] %*

	set bat_returncode=%~1
	!bat_echolog!
	!bat_echolog!
	!bat_echolog! [!date! !time! !bat_batchfile!:SETUP_DIRS] ERROR [!%~1!] - %~2
	!bat_echolog!
	!bat_echolog!
	
	!bat_WAIT! 15


:ERROR_END
!bat_poplog! [RETURN !bat_batchfile!:ERROR] [bat_returncode=!bat_returncode!]
!bat_log!
!bat_return! !bat_returncode!





	REM #################################################################################
:HELP
	@echo off
	echo.
	echo.%~nx0
	echo.
	echo Usage: %~nx0 /capturedir "bat_capturedfilesdir" /caldir "!bat_caldir!" /detect "!bat_detectfile!" /session "!bat_capturedfilesdir!"
	echo.   
	echo.Uploads the zip file[s] from the all the "Submitted" dirs to the queue dir. 
	echo.
	echo.The main purpose of this script is to improve the upload robustness by using the "queue" method
	echo.and thus allowing the script to perform "retries" until this script is terminated, or until all 
	echo.the zips have been uploaded.  
	echo.
	echo.This script should run in a custom named instance of 
	echo.command.exe [cmd_Submitted_LocalUpdate.exe].  That allows the user to examine whether
	echo.it's still running and to be able to terminate it manually or automatically with a script. 
	echo.
	echo.It should be started in its own cmd shell.  The shell should be renamed to cmd_[batch name].exe.
	echo.Like this: 
	echo.    "cmd_Submitted_LocalUpload.bat.exe"
	echo.
	echo.    That way, it can be killed by the user or by our pre-capture scripts that kill all remaining running tasks. 
	echo.
	echo.The argument pairs can be placed in any order.  [Note: Way may need to add a new argument "/submitted"]
	echo.  arg1             /capturedir bat_capturedfilesdir
	echo.  arg2             /caldir bat_caldir
	echo.
	echo.QUEUE FORMAT:
	echo.   Queue Name: ".\CAMS\Submitted_Queue\Submitted_Queue.txt"
	echo.      The queue items will be entered as follows:
	echo.         ????_??_??_??????_??_??_??_??.zip,"^<bat_submitteddir^>","^<command line^>""
	echo.
	echo.   The original source zip file that got uploaded will remain in the queue folder
	echo.   until after it has been safely verified that it uploaded.  That way, if necessary, it
	echo.   can be resubmitted easier. Once it is verified as uploaded, its set is MOVED to the
	echo.   associated Transmitted dir. 
	echo.
	echo.Steps involved to upload with retries:
	echo.- Determine the capture session dir name based on the "Submitted\*.zip".
	echo.- Delete the existing zip file from Submitted 
	echo.- Copy files from SubmissionFiles\FTP to Submitted
	echo.  - Optionally copy Thumbnails, but only if the INI file says to.
	echo.  - Optionally copy FF*.bin files, but only if the INI file says to.
	echo.
	echo.- Make a new abridged zip file from Submitted
	echo.- Make a matching MD5.txt file of the ZIP file
	echo.- Move the zip file to the "queue" dir.
	echo.- Move the md5.txt file to the "queue" dir.
	echo.
	echo.- Process the queue dir as follows:
	echo.  - List all the zip files
	echo.  - For each that has a matching .md5.txt file, call the upload script. 
	echo.  - If it succeeds, upload the md5.txt file
	echo.  - If it fails, wait [bat_CAMS2UPLOADS.retry_delay] seconds and try again.
	echo.  - If the md5.txt file fails, wait [bat_CAMS2UPLOADS.retry_delay] seconds and try again.
	echo.  - Once the md5.txt file succeeds, move the zip and  the md5.txt files to the Transmitted dir ["bat_CAMS2DIR.n\Transmitted"]
	echo.
	echo.
	echo.Upload Script:
	echo.- Should be flexible enough to support the use of various other clients, such as FTP, BitTorrent, FileZilla command line, winscp, etc. 
	echo.- The protocol works like this:
	echo.  - Unzip the zip file to a temp dir and check for success errorlevel/return code
	echo.    - If the zip file won't unzip to temp dir, then move it back to its original Submitted dir
	echo.  - Create the MD5 hash file for the zip file and save the hash value in a variable
	echo.  - Upload the zip file
	echo.  - Download  the zip file to a temp dir
	echo.  - Unzip the downloaded zip to a temp dir and check for success errorlevel
	echo.  - Check the MD5 of the temp downloaded zip and compare against original MD5
	echo.  - Cleanup the temp dirs
	echo.  - If downloaded zip file not OK, then wait [bat_CAMS2UPLOADS.retry_delay] seconds and try again.
	echo.
	echo.
	echo.- This script needs to end before capture starts. 
	echo.
	echo.- The scheduled tasks should be staggered maximum run time
	echo.
	echo.NOTE: Many settings are governed by the "Cams2Global.ini" file which will be
	echo.      in the same folder as this script.  Initially, this script will be 
	echo.      installed, along with all the other upgraded Cams16 scripts in the 
	echo.      RunFolder of the first board. 
	echo.

exit /b 99




	REM #################################################################################
:DONE

	
	!bat_popecho! [goto !bat_batchfile!:DONE] [bat_returncode=!bat_returncode!]
	!bat_echolog! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	!bat_echolog!
	
	

exit /b !bat_returncode!
